"use strict";

function _toConsumableArray(arr) {
    if (Array.isArray(arr)) {
        for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) {
            arr2[i] = arr[i];
        }
        return arr2;
    } else {
        return Array.from(arr);
    }
}

var Board = function Board(game, world, state) {
    this.game = game;
    this.world = world;
    this.phaser = global.phaser;
    this.gameState = state;

    this.tutorial = null;

    this.canKingJumpLimit = 0;
    this.canChecerJumpLimit = 0;

    this.possibleCheckersShown = false;

    this.tutorialSelector = null;

    this.boardOfCheckers = [
        [],
        [],
        [],
        [],
        [],
        [],
        [],
        []
    ];

    this.moving = false;

    this.possibleCheckersToMove = [];

    this.possibleMovesGroupSprites = [];
    this.possibleMovesGraph = [];
    this.possibleMovesGraphKing = [];
    this.possibleMovesGroupRowCol = [];
    this.possibleSimpleMovesGraph = [];
    this.paths = [];

    this.createBoard();

    //for test purposes
    if (global.enviroment == global.enviroments.test) {
        this.addCustomCheckersToBoard(global.customBoard);
    } else {
        this.addCheckersFromStorageToBoard();
    }

    this.gameHistory = new GameHistory();

    this.initParticles();
};

Board.prototype = {

    addPlayerTextIndicator: function addPlayerTextIndicator() {
        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
        }

        var playerTexts = [
            [global.managers.language.get("Move_1"), global.managers.language.get("Move_2")],
            [global.managers.language.get("Move_3"), global.managers.language.get("Move_4")],
            [global.managers.language.get("Move_7"), global.managers.language.get("Move_8")],
            [global.managers.language.get("Move_5"), global.managers.language.get("Move_6")],
            [global.managers.language.get("Move_7"), global.managers.language.get("Move_8")]
        ];

        this.winnersTexts = [
            [global.managers.language.get("Win_1"), global.managers.language.get("Win_2")],
            [global.managers.language.get("Win_3"), global.managers.language.get("Win_4")],
            [global.managers.language.get("Win_7"), global.managers.language.get("Win_8")],
            [global.managers.language.get("Win_5"), global.managers.language.get("Win_6")],
            [global.managers.language.get("Win_7"), global.managers.language.get("Win_8")]
        ];

        if (global.resolution == "480x800") {
            var pos = this.world.height / 2 - 260;
        } else {
            var pos = this.indicatorPossitions[1];
        }

        var textPlayer1 = this.game.add.text(-300, pos, playerTexts[global.settings.stonesSkin][0], textStyle);
        textPlayer1.anchor.set(0.5);
        this.textPlayer1 = textPlayer1;
        textPlayer1.visible = true;

        var textPlayer2 = this.game.add.text(this.world.width + 300, pos, playerTexts[global.settings.stonesSkin][1], textStyle);
        textPlayer2.anchor.set(0.5);
        this.textPlayer2 = textPlayer2;
        textPlayer2.visible = true;

        if (global.storage.data.lastPlayer == 1) {
            this.currentPlayer = this.player2;
        } else {
            this.currentPlayer == this.player1;
        }


        if (this.currentPlayer == this.player1) {
            if (global.settings.playAs != 1) {
                this.tweenTo(this.textPlayer1, this.world.width / 2, pos, false);
                this.game.world.bringToTop(this.currentPlayer1Indicator);
            } else {
                this.tweenTo(this.textPlayer2, this.world.width / 2, pos, false);
                this.game.world.bringToTop(this.currentPlayer2Indicator);
            }
        } else {
            if (global.settings.playAs != 1) {
                this.tweenTo(this.textPlayer2, this.world.width / 2, pos, false);
                this.game.world.bringToTop(this.currentPlayer2Indicator);
            } else {
                this.tweenTo(this.textPlayer1, this.world.width / 2, pos, false);
                this.game.world.bringToTop(this.currentPlayer1Indicator);
            }
        }
    },

    setTopPossitions: function setTopPossitions() {
        if (global.resolution == "480x800") {
            this.indicatorPossitions = global.calculatePositions(50, this.world.height / 2 - 264, 10, 2);
        } else {
            this.indicatorPossitions = global.calculatePositions(60, this.world.height / 2 - 334, 10, 2);
            this.indicatorPossitions[1] += 40;
        }
    },

    //--------------------------------------------------------------------------
    //----------------------------CREATE BOARD----------------------------------
    //--------------------------------------------------------------------------

    /*
     * Create board background.
     * Add on click functionality for board.
     */
    createBoard: function createBoard() {
        var _this = this;

        //write board to canvas
        var boardBackground = this.game.add.sprite(this.world.width / 2, this.world.height / 2, global.getBoardSprite());
        boardBackground.anchor.set(0.5);
        boardBackground.inputEnabled = true;

        //listener that get positons from board and handle all game logic
        boardBackground.events.onInputDown.add(function () {
            this.getGameState().blockUndo();

            var to = _this.getRowColValue(_this.game.input.x, _this.game.input.y);
            if (to != null) {
                _this.moveTo(to[0], to[1]);
            }
        }, this);

        this.boardBackground = boardBackground;
    },

    /*
     * Add checkers to board based on stored board.
     */
    addCheckersFromStorageToBoard: function addCheckersFromStorageToBoard() {
        if (global.storage.data.board == null) {
            this.addCheckersToBoard();
            return;
        }

        global.rules.aiLevel = global.storage.data.lastGameAi;

        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (global.storage.data.board[row][col] == 1) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), false, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else if (global.storage.data.board[row][col] == 2) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), false, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else if (global.storage.data.board[row][col] == 10) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), true, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else if (global.storage.data.board[row][col] == 20) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), true, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else {
                    this.boardOfCheckers[row][col] = null;
                }
            }
        }
        this.gameState.unblockUndo();
    },

    /*
     * Add checkers to board based on custom board mainly for testing/debugging.
     */
    addCustomCheckersToBoard: function addCustomCheckersToBoard(board) {

        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (board[row][col] == 1) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), false, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else if (board[row][col] == 2) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), false, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else if (board[row][col] == 10) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), true, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else if (board[row][col] == 20) {
                    var pos = this.getXYValue(row, col);
                    var checker = new Checker(this, row, col, pos[0], pos[1], global.getStonesSprites(), true, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else {
                    this.boardOfCheckers[row][col] = null;
                }
            }
        }
    },

    /*
     * Add all checkers to board.
     * For new game.
     */
    addCheckersToBoard: function addCheckersToBoard() {
        if (global.resolution == "480x800") {
            var gameSpaceScale = 51;
            var gameBoardStartX = this.world.width / 2 - 178;
            var gameBoardStartY = this.world.height / 2 - 178;
            var margin = 0;
            var bottomCheckersMargin = 0;
        } else {
            var gameSpaceScale = 74;
            var gameBoardStartX = this.world.width / 2 - 262;
            var gameBoardStartY = this.world.height / 2 - 262;
            var margin = 0.7;
            var bottomCheckersMargin = 142;
        }

        for (var row = 0; row < 3; row++) {
            for (var col = 0; col < 8; col++) {
                if (col % 2 != 0 && row == 0) {
                    var startXPos = margin * col + gameBoardStartX + col * gameSpaceScale;
                    var startYPos = gameBoardStartY;
                    var checker = new Checker(this, row, col, startXPos, startYPos, global.getStonesSprites(), false, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else if (col % 2 == 0 && row == 1) {
                    var startXPos = margin * col + gameBoardStartX + col * gameSpaceScale;
                    var startYPos = gameBoardStartY + gameSpaceScale;
                    var checker = new Checker(this, row, col, startXPos, startYPos, global.getStonesSprites(), false, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else if (col % 2 != 0 && row == 2) {
                    var startXPos = margin * col + gameBoardStartX + col * gameSpaceScale;
                    var startYPos = gameBoardStartY + gameSpaceScale * 2;
                    var checker = new Checker(this, row, col, startXPos, startYPos, global.getStonesSprites(), false, '2');
                    this.boardOfCheckers[row][col] = checker;
                } else {
                    this.boardOfCheckers[row][col] = null;
                }
            }
        }

        for (var row = 3; row < 5; row++) {
            for (var col = 0; col < 8; col++) {
                this.boardOfCheckers[row][col] = null;
            }
        }

        for (var row = 5; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (col % 2 != 0 && row == 6) {
                    var startXPos = margin * col + gameBoardStartX + col * gameSpaceScale;
                    var startYPos = bottomCheckersMargin + gameBoardStartY + 306;
                    var checker = new Checker(this, row, col, startXPos, startYPos, global.getStonesSprites(), false, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else if (col % 2 == 0 && row == 5) {
                    var startXPos = margin * col + gameBoardStartX + col * gameSpaceScale;
                    var startYPos = bottomCheckersMargin + gameBoardStartY + 306 - gameSpaceScale;
                    var checker = new Checker(this, row, col, startXPos, startYPos, global.getStonesSprites(), false, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else if (col % 2 == 0 && row == 7) {
                    var startXPos = margin * col + gameBoardStartX + col * gameSpaceScale;
                    var startYPos = bottomCheckersMargin + gameBoardStartY + 306 + gameSpaceScale;
                    var checker = new Checker(this, row, col, startXPos, startYPos, global.getStonesSprites(), false, '1');
                    this.boardOfCheckers[row][col] = checker;
                } else {
                    this.boardOfCheckers[row][col] = null;
                }
            }
        }
        this.storeBoard();
    },

    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------

    /*
     * Show with which checkers player can move.
     */
    showPossibleCheckersOnBoard: function showPossibleCheckersOnBoard(captureMode) {
        if (global.rules.aiLevel == 0) {
            if (global.storage.data.lastPlayer != null) {
                if (global.storage.data.lastPlayer == 1) {
                    var player = 2;
                } else {
                    var player = 1;
                }

            } else {
                var player = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.currentPlayer.getPlayer();
            }
        } else {
            var player = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.currentPlayer.getPlayer();
        }

        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        this.possibleCheckersToMove = [];
        this.possibleMovesGroupSprites = [];
        this.possibleMovesGraph = [];
        this.possibleMovesGraphKing = [];
        this.possibleMovesGroupRowCol = [];
        this.possibleSimpleMovesGraph = [];
        this.paths = [];
        this.possibleCheckersPathsToMove = [];

        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.cleanShowPossibleMovesOnBoardArray();
                    this.uncheckEveryCheckedChecker();
                    this.removeAllFromRemoveMarking();

                    if (this.boardOfCheckers[row][col].getPlayer() == player && this.boardOfCheckers[row][col].whatAmI() == "checker") {
                        if (this.boardOfCheckers[row][col].canCheckerBeMoved()) {
                            this.possibleCheckersToMove.push(this.boardOfCheckers[row][col]);
                            this.possibleCheckersPathsToMove.push([this.boardOfCheckers[row][col].getPossibleMovesGraph(), this.boardOfCheckers[row][col].getPossibleSimpleMovesGraph()]);
                        }
                    } else if (this.boardOfCheckers[row][col].getPlayer() == player && this.boardOfCheckers[row][col].whatAmI() == "king") {
                        if (this.boardOfCheckers[row][col].canKingBeMoved()) {
                            this.possibleCheckersToMove.push(this.boardOfCheckers[row][col]);
                            this.possibleCheckersPathsToMove.push([this.boardOfCheckers[row][col].getPossibleMovesGraphKing(), this.boardOfCheckers[row][col].getPossibleSimpleMovesGraph()]);
                        }
                    }
                }
            }
        }

        //------------------------------------------------------------------
        if (captureMode == 0) {
            var allPaths = [];

            for (var i = 0; i < this.possibleCheckersToMove.length; i++) {
                var paths = this.possibleCheckersPathsToMove[i][0];
                allPaths.push(paths);
            }

            for (var i = 0; i < allPaths.length; i++) {
                allPaths[i] = this.getAllPossibleWays(allPaths[i]);
            }

            this.possibleCheckersToMove = [];

            var maxHelp = 0;
            for (var i = 0; i < allPaths.length; i++) {
                for (var j = 0; j < allPaths[i].length; j++) {
                    allPaths[i][j] = allPaths[i][j].split("->");
                    if (allPaths[i][j].length > maxHelp) {
                        maxHelp = allPaths[i][j].length;
                    }
                }
            }

            if (global.rules.kingPriority == false) {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {

                        if (allPaths[i][j].length >= 2) {
                            var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                            this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                        }
                    }
                }
            } else {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                        if (this.boardOfCheckers[pos[0]][pos[1]].whatAmI() == "king") {
                            if (this.boardOfCheckers[pos[0]][pos[1]].canKingJump(pos[0], pos[1])) {
                                this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                            }
                        }
                    }
                }
            }

            if (this.possibleCheckersToMove.length == 0) {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        if (allPaths[i][j].length >= 2) {
                            var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                            this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                        }
                    }
                }
            }


            if (this.possibleCheckersToMove.length == 0) {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                        this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                    }
                }
            }

            //------------------------------------------------------------------
        } else if (captureMode == 1) {
            //Mandatory max number of men
            var allPaths = [];

            for (var i = 0; i < this.possibleCheckersToMove.length; i++) {
                var paths = this.possibleCheckersPathsToMove[i][0];
                allPaths.push(paths);
            }

            for (var i = 0; i < allPaths.length; i++) {
                allPaths[i] = this.getAllPossibleWays(allPaths[i]);
            }

            this.possibleCheckersToMove = [];

            var maxHelp = 0;
            for (var i = 0; i < allPaths.length; i++) {
                for (var j = 0; j < allPaths[i].length; j++) {
                    allPaths[i][j] = allPaths[i][j].split("->");
                    if (allPaths[i][j].length > maxHelp) {
                        maxHelp = allPaths[i][j].length;
                    }
                }
            }

            if (global.rules.kingPriority == false) {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        if (allPaths[i][j].length == maxHelp) {
                            var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                            this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                        }
                    }
                }
            } else {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                        if (this.boardOfCheckers[pos[0]][pos[1]].whatAmI() == "king") {
                            if (this.boardOfCheckers[pos[0]][pos[1]].canKingJump(pos[0], pos[1])) {
                                this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                            }
                        }
                    }
                }
            }

            if (this.possibleCheckersToMove.length == 0) {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        if (allPaths[i][j].length == maxHelp) {
                            var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                            this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                        }
                    }
                }
            }

            //------------------------------------------------------------------
        } else {
            //Not mandatory
            var allPaths = [];

            for (var i = 0; i < this.possibleCheckersToMove.length; i++) {
                var paths = this.possibleCheckersPathsToMove[i][0];
                allPaths.push(paths);
            }

            for (var i = 0; i < allPaths.length; i++) {
                allPaths[i] = this.getAllPossibleWays(allPaths[i]);
            }

            this.possibleCheckersToMove = [];

            for (var i = 0; i < allPaths.length; i++) {
                for (var j = 0; j < allPaths[i].length; j++) {
                    allPaths[i][j] = allPaths[i][j].split("->");
                }
            }

            //king priority handeling
            if (global.rules.kingPriority == false) {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                        this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                    }
                }
            } else {
                for (var i = 0; i < allPaths.length; i++) {
                    for (var j = 0; j < allPaths[i].length; j++) {
                        var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                        if (this.boardOfCheckers[pos[0]][pos[1]].whatAmI() == "king") {
                            if (this.boardOfCheckers[pos[0]][pos[1]].canKingJump(pos[0], pos[1])) {
                                this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                            }
                        }
                    }
                }
                if (this.possibleCheckersToMove.length == 0) {
                    for (var i = 0; i < allPaths.length; i++) {
                        for (var j = 0; j < allPaths[i].length; j++) {
                            var pos = global.getRowAndColFromGraph(allPaths[i][j][0]);
                            this.possibleCheckersToMove.push(this.boardOfCheckers[pos[0]][pos[1]]);
                        }
                    }
                }
            }
        }

        // check if there is any move possible
        if (this.possibleCheckersToMove.length == 0) {

            var overlay = this.phaser.add.image(0, 0, "overlay");
            overlay.width = this.phaser.width;
            overlay.height = this.phaser.height;
            overlay.alpha = 0.65;
            overlay.inputEnabled = true;
            overlay.visible = true;
            this.overlay = overlay;
            global.setPauseState(true);


            // todo fix shit here
            if (global.rules.aiLevel != 0) {
                if (player == 1) {
                    //there is BUGINO BUG
                    var win = new ScreenResult(this.game, this.world, this, false, this.gameState);
                } else {
                    var win = new ScreenResult(this.game, this.world, this, true, this.gameState);
                }


            } else {
                if (this.currentPlayer.getPlayer() == 1) {
                    if (global.settings.playAs == 0) {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][1]);
                    } else {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][0]);
                    }
                } else {
                    if (global.settings.playAs == 0) {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][0]);
                    } else {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][1]);
                    }
                }
            }

            win.showScreen();
            return;
        }


        if (Number(this.possibleCheckersToMove[0].getPlayer()) - 1 == 0 || global.rules.aiLevel == 0) {
            for (var i = 0; i < this.possibleCheckersToMove.length; i++) {
                if (global.settings.highlight == true) {
                    // this.possibleCheckersToMove[i].setChecked(true);
                    if (!bot) {
                        this.possibleCheckersToMove[i].startPulseChecker();
                    }
                }
                this.possibleCheckersShown = true;
            }
        }

        var returnCheckers = [];
        for (var i = 0; i < this.possibleCheckersToMove.length; i++) {
            if (!this.presentInArray(returnCheckers, this.possibleCheckersToMove[i])) {
                returnCheckers.push(this.possibleCheckersToMove[i]);
            }
        }

        this.possibleCheckersToMove = returnCheckers;
        return this.possibleCheckersToMove;
    },

    presentInArray: function presentInArray(array, chceker) {
        for (var i = 0; i < array.length; i++) {
            if (array[i] == chceker) {
                return true;
            }
        }
        return false;
    },

    isPossibleCheckersShown: function isPossibleCheckersShown() {
        return this.possibleCheckersShown;
    },

    deletePossibleSimpleMoves: function deletePossibleSimpleMoves() { },

    isCheckerMarkedAsPossibleToMove: function isCheckerMarkedAsPossibleToMove(checker) {
        for (var i = 0; i < this.possibleCheckersToMove.length; i++) {
            if (this.possibleCheckersToMove[i].getRow() == checker.getRow() && this.possibleCheckersToMove[i].getCol() == checker.getCol()) {
                return true;
            }
        }
        return false;
    },

    /*
     *  Show where can player move on the board.
     */
    showPossibleMovesOnBoard: function showPossibleMovesOnBoard(moves, simpleMoves, captureMode) {
        var showSprites = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

        var possibleMovesConnected = [];

        if (captureMode == 0) {
            //Mandatory but can choose any
            this.paths = this.getAllPossibleWays(moves);

            if (this.paths[0].length <= 2) {
                this.paths = this.getAllPossibleWays(simpleMoves);
                var paths2 = this.getAllPossibleWays(moves);
                for (var i = 0; i < paths2.length; i++) {
                    this.paths.push(paths2[i]);
                }
            }

            for (var i = 0; i < this.paths.length; i++) {
                possibleMovesConnected.push(this.paths[i].split("->"));
            }

            for (var i = possibleMovesConnected.length - 1; i >= 0; i--) {
                for (var j = possibleMovesConnected[i].length - 1; j >= 1; j--) {
                    var pos = global.getRowAndColFromGraph(possibleMovesConnected[i][j]);
                    var XandY = this.getXYValue(pos[0], pos[1]);

                    if (global.storage.data.tutorial != 'done') {
                        if (pos[0] == 4 && pos[1] == 3) {
                            if (this.isPossibleMoveSpritePresent(this.possibleMovesGroupSprites, XandY[0], XandY[1])) {
                                if (showSprites) {
                                    this.tutorialSelector = this.game.add.sprite(XandY[0], XandY[1], "selector");
                                    this.tutorialSelector.anchor.setTo(0.5, 0.5);
                                    this.tutorialSelector.animations.add('run');
                                    this.tutorialSelector.animations.play('run', 15, true);
                                    this.possibleMovesGroupSprites.push(this.tutorialSelector);
                                }
                                this.possibleMovesGroupRowCol.push([pos[0], pos[1]]);
                            }
                        }
                    }

                    if (this.isPossibleMoveSpritePresent(this.possibleMovesGroupSprites, XandY[0], XandY[1])) {
                        if (showSprites) {
                            var selector = this.game.add.sprite(XandY[0], XandY[1], "selector");
                            selector.anchor.setTo(0.5, 0.5);
                            selector.animations.add('run');
                            selector.animations.play('run', 15, true);
                            this.possibleMovesGroupSprites.push(selector);
                        }
                        this.possibleMovesGroupRowCol.push([pos[0], pos[1]]);
                    }
                }
            }
        } else if (captureMode == 1) {
            //Mandatory max number of men
            this.paths = this.getAllPossibleWays(moves);

            if (this.paths[0].length <= 2) {
                this.paths = this.getAllPossibleWays(simpleMoves);
                var paths2 = this.getAllPossibleWays(moves);
                for (var i = 0; i < paths2.length; i++) {
                    this.paths.push(paths2[i]);
                }
            }

            for (var i = 0; i < this.paths.length; i++) {
                possibleMovesConnected.push(this.paths[i].split("->"));
            }

            var maxSize = 0;
            var maxElement = 0;
            for (var i = 0; i < possibleMovesConnected.length; i++) {
                if (possibleMovesConnected[i].length > maxSize) {
                    maxSize = possibleMovesConnected[i].length;
                    maxElement = i;
                }
            }

            for (var i = possibleMovesConnected.length - 1; i >= 0; i--) {
                if (possibleMovesConnected[i].length == maxSize) {
                    var pos = global.getRowAndColFromGraph(possibleMovesConnected[i][possibleMovesConnected[i].length - 1]);
                    var XandY = this.getXYValue(pos[0], pos[1]);

                    if (global.storage.data.tutorial != 'done') {
                        if (pos[0] == 4 && pos[1] == 3) {
                            if (this.isPossibleMoveSpritePresent(this.possibleMovesGroupSprites, XandY[0], XandY[1])) {
                                if (showSprites) {
                                    this.tutorialSelector = this.game.add.sprite(XandY[0], XandY[1], "selector");
                                    this.tutorialSelector.anchor.setTo(0.5, 0.5);
                                    this.tutorialSelector.animations.add('run');
                                    this.tutorialSelector.animations.play('run', 15, true);
                                    this.possibleMovesGroupSprites.push(this.tutorialSelector);
                                }
                                this.possibleMovesGroupRowCol.push([pos[0], pos[1]]);
                            }
                        }
                    }

                    if (this.isPossibleMoveSpritePresent(this.possibleMovesGroupSprites, XandY[0], XandY[1])) {
                        if (showSprites) {
                            var selector = this.game.add.sprite(XandY[0], XandY[1], "selector");
                            selector.anchor.setTo(0.5, 0.5);
                            selector.animations.add('run');
                            selector.animations.play('run', 15, true);
                            this.possibleMovesGroupSprites.push(selector);
                        }
                        this.possibleMovesGroupRowCol.push([pos[0], pos[1]]);
                    }
                }
            }

        } else {
            this.paths = this.getAllPossibleWays(simpleMoves);

            var paths2 = this.getAllPossibleWays(moves);
            for (var i = 0; i < paths2.length; i++) {
                this.paths.push(paths2[i]);
            }

            for (var i = 0; i < this.paths.length; i++) {
                possibleMovesConnected.push(this.paths[i].split("->"));
            }

            for (var i = possibleMovesConnected.length - 1; i >= 0; i--) {
                for (var j = possibleMovesConnected[i].length - 1; j >= 1; j--) {
                    var pos = global.getRowAndColFromGraph(possibleMovesConnected[i][j]);
                    var XandY = this.getXYValue(pos[0], pos[1]);

                    if (global.storage.data.tutorial != 'done') {
                        if (pos[0] == 4 && pos[1] == 3) {
                            if (this.isPossibleMoveSpritePresent(this.possibleMovesGroupSprites, XandY[0], XandY[1])) {
                                if (showSprites) {
                                    this.tutorialSelector = this.game.add.sprite(XandY[0], XandY[1], "selector");
                                    this.tutorialSelector.anchor.setTo(0.5, 0.5);
                                    this.tutorialSelector.animations.add('run');
                                    this.tutorialSelector.animations.play('run', 15, true);
                                    this.possibleMovesGroupSprites.push(this.tutorialSelector);
                                }
                                this.possibleMovesGroupRowCol.push([pos[0], pos[1]]);
                            }
                        }
                    }

                    if (this.isPossibleMoveSpritePresent(this.possibleMovesGroupSprites, XandY[0], XandY[1])) {
                        if (showSprites) {
                            var selector = this.game.add.sprite(XandY[0], XandY[1], "selector");
                            selector.anchor.setTo(0.5, 0.5);
                            selector.animations.add('run');
                            selector.animations.play('run', 15, true);
                            this.possibleMovesGroupSprites.push(selector);
                        }
                        this.possibleMovesGroupRowCol.push([pos[0], pos[1]]);
                    }
                }
            }
        }

        if (global.storage.data.tutorial != 'done') {
            var pos = this.getXYValue(4, 3);

            if (global.resolution == "480x800") {
                this.tutorial.moveHandTo(pos[0] + 39, pos[1] + 40, global.times.TUTORIAL_HAND_TIME - 300);
            } else {
                this.tutorial.moveHandTo(pos[0] + 55, pos[1] + 55, global.times.TUTORIAL_HAND_TIME - 300);
            }

            this.game.world.bringToTop(this.tutorialSelector);

            this.tutorialSelector.inputEnabled = true;
            this.tutorialSelector.events.onInputDown.add(this.moveToTutorial, this);
        }
    },

    moveToTutorial: function moveToTutorial() {
        global.managers.audio.playSound("suchnutie1");
        this.moveTo(4, 3);
    },

    getWorld: function getWorld() {
        return this.game.world;
    },

    /*
     * Check if possible position to jump on is already marked.
     */
    isPossibleMoveSpritePresent: function isPossibleMoveSpritePresent(sprites, x, y) {
        for (var i = 0; i < sprites.length; i++) {
            if (x == sprites[i].position.x && y == sprites[i].position.y) {
                return false;
            }
        }
        return true;
    },

    /*
     * Clear all arrays of positions and tagged sprites.
     */
    cleanShowPossibleMovesOnBoardArray: function cleanShowPossibleMovesOnBoardArray() {
        for (var i = 0; i < this.possibleMovesGroupSprites.length; i++) {
            this.possibleMovesGroupSprites[i].destroy(true);
        }

        this.possibleMovesGroup = [];
        this.possibleMovesGroupSprites = [];
        this.possibleMovesGraph = [];
        this.possibleMovesGraphKing = [];
        this.possibleSimpleMovesGraph = [];
        this.possibleMovesGroupRowCol = [];
    },

    /*
     * Gat data to be saved from actual checkers.
     */
    getBoardHistoryData: function getBoardHistoryData() {
        var dataBoard = [
            [],
            [],
            [],
            [],
            [],
            [],
            [],
            []
        ];
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    dataBoard[row][col] = this.boardOfCheckers[row][col].getDataToHistory();
                } else {
                    dataBoard[row][col] = null;
                }
            }
        }
        return dataBoard;
    },

    /*
     * Check if checker can be moved to selected position.
     * Handler for checker and king.
     */
    moveTo: function moveTo(row, col) {
        this.getGameState().blockUndo();
        var checked = this.getCheckedRowAndCol();

        //check if moving on black tiles
        if (row != -1 && col != -1) {
            if (global.playboard[row][col] == 0) {
                return;
            }
        }

        if (checked != null || checked != undefined) {
            this.game.world.bringToTop(this.boardOfCheckers[checked[0]][checked[1]].getCheckerSprite());
            for (var i = 0; i < this.possibleMovesGroupRowCol.length; i++) {
                if (this.possibleMovesGroupRowCol[i][0] == row && this.possibleMovesGroupRowCol[i][1] == col) {
                    var to = this.possibleMovesGroupRowCol[i];
                    this.gameHistory.addBoard([global.boardForGraph[checked[0]][checked[1]], global.boardForGraph[to[0]][to[1]]], this.getBoardHistoryData());
                    var checkersToRemove = this.makeCheckerMoveChain(to, false);
                    var processedCheckers = this.processCheckersToRemove(checkersToRemove);
                    this.gameHistory.addRemoved(processedCheckers);
                }
            }
        }
    },

    /*
     * Move checker with bot.
     * Handler for checker and king.
     */
    moveToBot: function moveToBot(row, col) {
        this.getGameState().blockUndo();
        var checked = this.getCheckedRowAndCol();

        if (row != -1 && col != -1) {
            if (global.playboard[row][col] == 0) {
                return;
            }
        }

        var i = checked;
        if (checked != null || checked != undefined) {
            this.game.world.bringToTop(this.boardOfCheckers[checked[0]][checked[1]].getCheckerSprite());
            for (var i = 0; i < this.possibleMovesGroupRowCol.length; i++) {
                if (this.possibleMovesGroupRowCol[i][0] == row && this.possibleMovesGroupRowCol[i][1] == col) {
                    var to = this.possibleMovesGroupRowCol[i];
                    this.gameHistory.addBoard([global.boardForGraph[checked[0]][checked[1]], global.boardForGraph[to[0]][to[1]]], this.getBoardHistoryData());
                    var checkersToRemove = this.makeCheckerMoveChain(to, true);
                    var processedCheckers = this.processCheckersToRemove(checkersToRemove);
                    this.gameHistory.addRemoved(processedCheckers);
                }
            }
        }

        //save to history
        // this.storeBoard();
    },

    moveToFake: function moveToFake(from, to) {
        var fromPos = global.getRowAndColFromGraph(from);
        var toPos = global.getRowAndColFromGraph(to);

        this.boardOfCheckers[toPos[0]][toPos[1]] = this.boardOfCheckers[fromPos[0]][fromPos[1]];
        this.boardOfCheckers[fromPos[0]][fromPos[1]] = null;
        return this.boardOfCheckers;
    },

    moveToFakeReverse: function moveToFakeReverse(from, to) {
        var fromPos = global.getRowAndColFromGraph(from);
        var toPos = global.getRowAndColFromGraph(to);

        this.boardOfCheckers[fromPos[0]][fromPos[1]] = this.boardOfCheckers[toPos[0]][toPos[1]];
        this.boardOfCheckers[toPos[0]][toPos[1]] = null;
        return this.boardOfCheckers;
    },

    stopAllPulsingCheckers: function stopAllPulsingCheckers() {
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.boardOfCheckers[row][col].stopPulseChecker();
                }
            }
        }
    },

    processCheckersToRemove: function processCheckersToRemove(checkers) {
        var processed = [];
        for (var i = 0; i < checkers.length; i++) {
            var data = [checkers[i].getRow(), checkers[i].getCol(), checkers[i].getPlayer(), checkers[i].whatAmI()];
            processed.push(data);
        }
        return processed;
    },

    /*
     * Store board to storage.
     */
    storeBoard: function storeBoard() {
        var boardToSave = [
            [],
            [],
            [],
            [],
            [],
            [],
            [],
            []
        ];
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    if (this.boardOfCheckers[row][col].getPlayer() == "1") {
                        if (this.boardOfCheckers[row][col].whatAmI() == "king") {
                            boardToSave[row][col] = 10;
                        } else {
                            boardToSave[row][col] = 1;
                        }
                    } else if (this.boardOfCheckers[row][col].getPlayer() == "2") {
                        if (this.boardOfCheckers[row][col].whatAmI() == "king") {
                            boardToSave[row][col] = 20;
                        } else {
                            boardToSave[row][col] = 2;
                        }
                    }
                }
            }
        }

        global.storage.data.board = boardToSave;
        global.storage.data.lastGameAi = global.rules.aiLevel;

        if (this.gameHistory != null) {
            if (this.currentPlayer != null) {
                var pl = this.currentPlayer.getPlayer() - 1;
                global.storage.data.lastPlayer = pl;
            }
            global.storage.data.historyMoves = this.gameHistory.getFullMoves();
            global.storage.data.removedCheckers = this.gameHistory.getFullCheckers();
            global.storage.data.historyBoardOfValues = this.gameHistory.getFullValues();
        }
        global.managers.storage.save();
    },

    //--------------------------------------------------------------------------
    //---------------------------MOVING AND REMOVING----------------------------
    //--------------------------------------------------------------------------

    /*
     * Move checkers on board, single or miltiple.
     */
    makeCheckerMoveChain: function makeCheckerMoveChain(posToGo, bot) {
        var _this6 = this;

        this.getGameState().blockUndo();
        var _this2 = this;

        this.moving = true;
        var pathNo = null;
        var rowTo = posToGo[0];
        var colTo = posToGo[1];

        var positions = [];
        for (var i = 0; i < this.paths.length; i++) {
            positions.push(this.paths[i].split("->"));
        }

        for (var i = positions.length - 1; i >= 0; i--) {
            for (var j = positions[i].length - 1; j >= 0; j--) {
                positions[i][j] = global.getRowAndColFromGraph(positions[i][j]);
                if (positions[i][j][0] == rowTo && positions[i][j][1] == colTo && pathNo == null) {
                    pathNo = [i, j];
                }
            }
        }

        // pathNo = [1, 3];
        var checked = this.getCheckedRowAndCol();
        this.uncheckEveryCheckedChecker();

        var movingPositions = [];
        movingPositions.push([checked[0], checked[1]]);

        var tweenList = [];
        for (var i = 1; i < pathNo[1] + 1; i++) {
            var row = positions[pathNo[0]][i][0];
            var col = positions[pathNo[0]][i][1];
            this.makeCheckerMoveChainRow = row;
            this.makeCheckerMoveChainCol = col;

            var desired = this.getXYValue(row, col);

            movingPositions.push([row, col]);

            var _this20 = this;
            var twee = this.boardOfCheckers[checked[0]][checked[1]].tweenTo(desired[0], desired[1]);
            twee.onComplete.add(function () {
                _this20.game.world.bringToTop(_this20.boardOfCheckers[checked[0]][checked[1]].getCheckerSprite());            
            });

            tweenList.push(twee);
        }

        var checkersToRemove = [];
        for (var i = 0; i < movingPositions.length - 1; i++) {
            var enemyChecker = this.isEnemyCheckerBetween(movingPositions[i][0], movingPositions[i][1], movingPositions[i + 1][0], movingPositions[i + 1][1]);
            if (enemyChecker != null) {
                checkersToRemove.push(enemyChecker);
            }
        }

        //removing checkers from board
        global.createTimer(350, function () {
            _this6.removeCheckersFromBoard(checkersToRemove);
        });

        if (tweenList.length != 0) {
            this.cleanShowPossibleMovesOnBoardArray();
            var tweenOne = tweenList[0];
            var tweenTmp;
            for (var i = 0; i < tweenList.length - 1; i++) {
                tweenList[i] = tweenList[i].chain(tweenList[i + 1]);
            }
            tweenList[0].start();

            if (global.storage.data.tutorial != 'done') {
                _this2.tutorial.destoryHand();
                _this2.tutorial.hideOverlay();
                global.managers.storage.save();
            }

            //do shit after complete
            tweenList[tweenList.length - 1].onComplete.add(function () {
                var rc = _this2.getXYValue(rowTo, colTo);

                if (global.storage.data.tutorial != 'done') {
                    global.storage.data.tutorial = 'done';
                    _this2.gameState.removeCheckerFromGroup();
                }

                _this2.boardOfCheckers[rowTo][colTo] = _this2.boardOfCheckers[checked[0]][checked[1]];

                _this2.boardOfCheckers[rowTo][colTo].setRow(rowTo);
                _this2.boardOfCheckers[rowTo][colTo].setCol(colTo);
                _this2.boardOfCheckers[checked[0]][checked[1]] = null;

                _this2.game.world.bringToTop(_this2.boardOfCheckers[rowTo][colTo].getCheckerSprite());
                _this2.boardOfCheckers[rowTo][colTo]

                _this2.uncheckEveryCheckedChecker();
                _this2.blockEveryChecker();

                _this2.unblockEveryChecker();
                if (!_this2.isGameFinished()) {

                    var player = "1";
                    if (global.settings.playAs == 1) {
                        var player = "2";
                    }

                    if (_this2.player1.getAiLevel() != 0 && _this2.currentPlayer.whoAmI() == 'bot') {
                        _this2.player1.makeMove();
                    } else {
                        var highlightBU = global.settings.highlight;
                        global.settings.highlight = false;


                        if (_this2.currentPlayer == "2") {
                            _this2.canChangeToKing(rowTo, colTo, rc[0], rc[1], _this2.player2);
                        } else {
                            _this2.canChangeToKing(rowTo, colTo, rc[0], rc[1], _this2.player1);
                        }
                        //checking no possible move after move
                        var possibleMoves = _this2.showPossibleCheckersOnBoard(1);
                        if (possibleMoves == undefined) {
                            // if (global.rules.aiLevel != 0) {
                            //     if (_this2.currentPlayer.getPlayer() == player) {
                            //         var win = new ScreenResult(_this2.game, _this2.world, _this2, true, _this2.gameState, _this2.winnersTexts[global.settings.stonesSkin][1]);
                            //     } else {
                            //         var win = new ScreenResult(_this2.game, _this2.world, _this2, false, _this2.gameState, _this2.winnersTexts[global.settings.stonesSkin][1]);
                            //     }
                            // } else {
                            //     console.log("bull-shit");
                            //     var p1 =  _this2.winnersTexts[global.settings.stonesSkin][0];
                            //     var p2 =  _this2.winnersTexts[global.settings.stonesSkin][1];

                            //     if(global.settings.playAs == 1){
                            //         console.log("win1");
                            //         var win = new ScreenResult(_this2.game, _this2.world, _this2, true, _this2.gameState, p1);
                            //         return;
                            //     }else{
                            //         console.log("win2");
                            //         var win = new ScreenResult(_this2.game, _this2.world, _this2, true, _this2.gameState, p2);
                            //         return;
                            //     }
                            // }
                            if (highlightBU != null) {
                                global.settings.highlight = highlightBU;
                                highlightBU = null;
                            }
                            _this2.cleanShowPossibleMovesOnBoardArray();
                            return;
                        }
                        if (highlightBU != null) {
                            global.settings.highlight = highlightBU;
                            highlightBU = null;
                        }


                        _this2.changeCurrentPlayer();
                        global.createTimer(global.times.TOP_CHECKERS_TWEEN_TIME * 3, function () {
                            if (global.rules.capture == 0) {
                                _this2.showPossibleCheckersOnBoard(0);
                            } else if (global.rules.capture == 1) {
                                _this2.showPossibleCheckersOnBoard(1);
                            } else {
                                _this2.showPossibleCheckersOnBoard(2);
                            }
                            _this2.unblockEveryChecker();
                        });
                    }
                }
                var e = _this2.currentPlayer;

                //check if can be changed to king
                _this2.canChangeToKing(rowTo, colTo, rc[0], rc[1], _this2.currentPlayer);

                _this2.moving = false;


                //save to history
                if (global.rules.aiLevel == 0) {
                    _this2.storeBoard();
                } else {
                    if (bot) {
                        _this2.storeBoard();
                    }
                }

                _this2.possibleCheckersShown = false;
            });
        }


        return checkersToRemove;
    },

    /*
     * Remove checkers from board, single or miltiple.
     */
    removeCheckersFromBoard: function removeCheckersFromBoard(checkers) {
        var _this3 = this;


        var tweenList = [];

        for (var i = 0; i < checkers.length; i++) {
            var removeRow = checkers[i].getRow();
            var removeCol = checkers[i].getCol();

            var randY = global.getRndInteger(this.world.width - 150, this.world.width + 150);
            var randX = -100;
            if (this.currentPlayer.getPlayer() == "1") {
                randX = this.world.width + 100;
            }

            tweenList.push(this.boardOfCheckers[removeRow][removeCol].tweenFromBoard(randX, randY));
            this.boardOfCheckers[removeRow][removeCol] = null;
        }

        if (tweenList.length != 0) {
            var tweenOne = tweenList[0];
            var tweenTmp;
            for (var i = 0; i < tweenList.length - 1; i++) {
                tweenList[i] = tweenList[i].chain(tweenList[i + 1]);
            }
            tweenList[0].start();

            tweenList[tweenList.length - 1].onComplete.add(function () {
                _this3.setCheckersInput(true);
            });

            this.cleanShowPossibleMovesOnBoardArray();
        }
    },

    /*
     * Set input for every checker in board.
     */
    setCheckersInput: function setCheckersInput(val) {
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.boardOfCheckers[row][col].setInput(val);
                }
            }
        }
    },

    /*
     * Check if inemy checker is between jumpfrom and jump to position.
     */
    isEnemyCheckerBetween: function isEnemyCheckerBetween(rowFrom, colFrom, rowTo, colTo) {
        var vertical = rowFrom - rowTo;
        var horizontal = colFrom - colTo;
        var moveUp = true;
        var moveLeft = true;
        if (vertical < 0) {
            moveUp = false;
        }
        if (horizontal < 0) {
            moveLeft = false;
        }

        if (moveUp && moveLeft) {
            var i = 0;
            while (rowFrom - i != rowTo && colFrom - i != colTo) {
                if (this.boardOfCheckers[rowFrom - i][colFrom - i] != null) {
                    if (this.boardOfCheckers[rowFrom - i][colFrom - i].getPlayer() != this.currentPlayer.getPlayer()) {
                        return this.boardOfCheckers[rowFrom - i][colFrom - i];
                    }
                }
                i++;
            }
        }
        if (!moveUp && !moveLeft) {
            var i = 0;
            while (rowFrom + i != rowTo && colFrom + i != colTo) {
                if (this.boardOfCheckers[rowFrom + i][colFrom + i] != null) {
                    if (this.boardOfCheckers[rowFrom + i][colFrom + i].getPlayer() != this.currentPlayer.getPlayer()) {
                        return this.boardOfCheckers[rowFrom + i][colFrom + i];
                    }
                }
                i++;
            }
        }
        if (!moveUp && moveLeft) {
            var i = 0;
            while (rowFrom + i != rowTo && colFrom - i != colTo) {
                if (this.boardOfCheckers[rowFrom + i][colFrom - i] != null) {
                    if (this.boardOfCheckers[rowFrom + i][colFrom - i].getPlayer() != this.currentPlayer.getPlayer()) {
                        return this.boardOfCheckers[rowFrom + i][colFrom - i];
                    }
                }
                i++;
            }
        }
        if (moveUp && !moveLeft) {
            var i = 0;
            while (rowFrom - i != rowTo && colFrom + i != colTo) {
                if (this.boardOfCheckers[rowFrom - i][colFrom + i] != null) {
                    if (this.boardOfCheckers[rowFrom - i][colFrom + i].getPlayer() != this.currentPlayer.getPlayer()) {
                        return this.boardOfCheckers[rowFrom - i][colFrom + i];
                    }
                }
                i++;
            }
        }

        return null;
    },

    /*
     * Check if checker can by changed to checker.
     */
    canChangeToKing: function canChangeToKing(rowTo, colTo, xPos, yPos, player) {
        if (this.boardOfCheckers[rowTo][colTo] == null) return;
        if (this.boardOfCheckers[rowTo][colTo].whatAmI() == "checker") {
            var i = player;
            if (rowTo == 0 && player.getPlayer() == "2") {
                this.boardOfCheckers[rowTo][colTo].changeToKing();
                this.fireParticle(xPos, yPos, global.particleAmount);
            } else if (rowTo == 7 && player.getPlayer() == "1") {
                this.boardOfCheckers[rowTo][colTo].changeToKing();
                this.fireParticle(xPos, yPos, global.particleAmount);
            }
        }
    },

    canCheckerByChangedToKingAfterMove: function canCheckerByChangedToKingAfterMove(checkers) {
        for (var i = 0; i < checkers.length; i++) {
            if (checkers[i].getRow() == 1 || checkers[i].getRow() == 6) {
                if (this.currentPlayer == '2') {
                    //check down
                    if (checkers[i].getCol() + 1 < 8) {
                        if (this.boardOfCheckers[checkers[i].getRow() + 1][checkers[i].getCol() + 1] == null) {
                            return i;
                        }
                    }

                    if (checkers[i].getCol() - 1 >= 0) {
                        if (this.boardOfCheckers[checkers[i].getRow() + 1][checkers[i].getCol() - 1] == null) {
                            return i;
                        }
                    }
                } else {
                    //check up
                    if (checkers[i].getCol() + 1 < 8) {
                        if (this.boardOfCheckers[checkers[i].getRow() - 1][checkers[i].getCol() + 1] == null) {
                            return i;
                        }
                    }

                    if (checkers[i].getCol() - 1 >= 0) {
                        if (this.boardOfCheckers[checkers[i].getRow() - 1][checkers[i].getCol() - 1] == null) {
                            return i;
                        }
                    }
                }
            }
        }
        return -1;
    },

    //--------------------------------------------------------------------------
    //---------------------------MANAGE CHECKERS---------------------------------
    //---------------------------------------------------------------------------

    /*
     * Uncheck every checker from board.
     */
    uncheckEveryCheckedChecker: function uncheckEveryCheckedChecker() {
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    if (this.boardOfCheckers[row][col].getChecked() == true) {
                        this.boardOfCheckers[row][col].setChecked(false);
                        this.boardOfCheckers[row][col].checkerUncheck();
                    }
                }
            }
        }
    },

    /*
     * Block every checker.
     */
    blockEveryChecker: function blockEveryChecker() {
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.boardOfCheckers[row][col].setBlocked(true);
                }
            }
        }
    },

    /*
     * Unblock every checker.
     */
    unblockEveryChecker: function unblockEveryChecker() {
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.boardOfCheckers[row][col].setBlocked(false);
                }
            }
        }
    },

    /*
     * Get row and col of checked checker.
     */
    getCheckedRowAndCol: function getCheckedRowAndCol() {
        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    if (this.boardOfCheckers[row][col].getChecked() == true) {
                        return [row, col];
                    }
                }
            }
        }
    },

    /*
     * Get X and Y position from row and col.
     */
    getXYValue: function getXYValue(desiredRow, desiredCol) {
        if (global.resolution == "480x800") {
            var gameSpaceScale = 51;
            var gameBoardStartX = this.world.width / 2 - 178;
            var gameBoardStartY = this.world.height / 2 - 178;
            var margin = 0;
        } else {
            var gameSpaceScale = 74;
            var gameBoardStartX = this.world.width / 2 - 262;
            var gameBoardStartY = this.world.height / 2 - 262;
            var margin = 0.7;
        }

        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (row == desiredRow && col == desiredCol) {
                    return [gameBoardStartX + col * gameSpaceScale + col * margin, gameBoardStartY + row * gameSpaceScale + row * margin];
                }
            }
        }
    },

    /*
     * Get row and col from clicked position.
     */
    getRowColValue: function getRowColValue(x, y) {
        if (global.resolution == "480x800") {
            var gameSpaceScale = 51;
            var gameBoardStartX = this.world.width / 2 - 178;
            var gameBoardStartY = this.world.height / 2 - 178;
        } else {
            var gameSpaceScale = 74;
            var gameBoardStartX = this.world.width / 2 - 262;
            var gameBoardStartY = this.world.height / 2 - 262;
        }

        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (gameBoardStartX + col * gameSpaceScale - 25 < x && gameBoardStartX - 25 + col * gameSpaceScale + gameSpaceScale > x && gameBoardStartY - 25 + row * gameSpaceScale < y && gameBoardStartY - 25 + row * gameSpaceScale + gameSpaceScale > y) {
                    return [row, col];
                }
            }
        }
        return null;
    },

    getPlayer2: function getPlayer2() {
        return this.player1;
    },

    /*
     * Add players to game.
     * Set player which will start the game.
     * Either 1 or 2 or random picked player.
     */
    setPlayers: function setPlayers(p1, p2) {
        this.setTopPossitions();
        this.player1 = p1;
        this.player2 = p2;

        if (global.resolution == '480x800') {
            this.indicatorBase = 25;
            this.indicatorMax = 50;
        } else {
            this.indicatorBase = 40;
            this.indicatorMax = 80;
        }

        this.currentPlayer1Indicator = this.game.add.sprite(this.world.width / 2 - this.indicatorBase, this.indicatorPossitions[0], global.getStonesSpritesToSettings()[1]);
        this.currentPlayer1Indicator.anchor.set(0.5);
        this.currentPlayer2Indicator = this.game.add.sprite(this.world.width / 2 + this.indicatorBase, this.indicatorPossitions[0], global.getStonesSpritesToSettings()[2]);
        this.currentPlayer2Indicator.anchor.set(0.5);







        if (global.storage.data.lastPlayer == null) {
            if (global.settings.playAs == 0) {
                this.currentPlayer = this.player1;
            } else if (global.settings.playAs == 1) {
                this.currentPlayer = this.player2;
            } else {
                var playerPick = Math.floor(Math.random() * 2);
                if (playerPick == 0) {
                    this.currentPlayer = this.player1;
                } else if (playerPick == 1) {
                    this.currentPlayer = this.player2;
                }
            }
        } else {
            if (global.storage.playAs == 0) {
                if (global.storage.data.lastPlayer == 0) {
                    this.currentPlayer = this.player1;
                } else if (global.storage.data.lastPlayer == 1) {
                    this.currentPlayer = this.player2;
                }
            } else {
                if (global.storage.data.lastPlayer == 0) {
                    this.currentPlayer = this.player2;
                } else if (global.storage.data.lastPlayer == 1) {
                    this.currentPlayer = this.player1;
                }
            }

        }
        this.currentPlayer = this.player1;


        this.addPlayerTextIndicator();

        //show checkers on board based on rules
        if (global.rules.capture == 0) {
            this.showPossibleCheckersOnBoard(0);
        } else if (global.rules.capture == 1) {
            this.showPossibleCheckersOnBoard(1);
        } else {
            this.showPossibleCheckersOnBoard(2);
        }
    },

    /*
     * Show animation on the tot which show player on turn.
     */
    changeCurrentPlayer: function changeCurrentPlayer() {
        if (global.resolution == "480x800") {
            var pos = this.world.height / 2 - 260;
        } else {
            var pos = this.indicatorPossitions[1];
        }

        if (this.currentPlayer == this.player1) {
            this.currentPlayer = this.player2;
            if (global.settings.playAs == 1) {
                this.switchTopTokens(1);
                this.tweenTo(this.textPlayer1, this.world.width / 2, pos, false);
                this.tweenTo(this.textPlayer2, this.world.width + 300, pos, false);
            } else {
                this.switchTopTokens(2);
                this.tweenTo(this.textPlayer1, -300, pos, false);
                this.tweenTo(this.textPlayer2, this.world.width / 2, pos, false);
            }
        } else {
            this.currentPlayer = this.player1;
            if (global.settings.playAs == 1) {
                this.switchTopTokens(2);
                this.tweenTo(this.textPlayer1, -300, pos, false);
                this.tweenTo(this.textPlayer2, this.world.width / 2, pos, false);
            } else {
                this.switchTopTokens(1);
                this.tweenTo(this.textPlayer1, this.world.width / 2, pos, false);
                this.tweenTo(this.textPlayer2, this.world.width + 300, pos, false);
            }
        }
    },

    /*
     * Tween top indicator switching tokens.
     */
    switchTopTokens: function switchTopTokens(currentPlayer) {
        var _this4 = this;

        if (_this4.indicatorPossitions == undefined) {
            _this4.setTopPossitions();
        }

        this.tweenTo(this.currentPlayer1Indicator, this.world.width / 2 - _this4.indicatorMax, this.indicatorPossitions[0], false);
        this.tweenTo(this.currentPlayer2Indicator, this.world.width / 2 + _this4.indicatorMax, this.indicatorPossitions[0], false);

        global.createTimer(global.times.TOP_CHECKERS_TWEEN_TIME, function () {
            if (currentPlayer == 2) {
                _this4.game.world.bringToTop(_this4.currentPlayer2Indicator);
            } else {
                _this4.game.world.bringToTop(_this4.currentPlayer1Indicator);
            }
            _this4.tweenTo(_this4.currentPlayer1Indicator, _this4.world.width / 2 - _this4.indicatorBase, _this4.indicatorPossitions[0], false);
            _this4.tweenTo(_this4.currentPlayer2Indicator, _this4.world.width / 2 + _this4.indicatorBase, _this4.indicatorPossitions[0], true);
        }, global.times.TOP_CHECKERS_TWEEN_TIME);
    },

    /*
     * Simple tween.
     */
    tweenTo: function tweenTo(obj, desiredX, desiredY, end) {
        var _this5 = this;
        var tween = this.game.add.tween(obj).to({
            x: desiredX,
            y: desiredY
        }, this.TOP_CHECKERS_TWEEN_TIME, "Quart.easeOut");
        tween.start();
        if (end) {
            tween.onComplete.add(function () {
                window.setTimeout(function () {
                    _this5.gameState.unblockUndo();
                }, 50);
            });
        }
    },

    /*
     * Get current player.
     */
    getCurrentPlayer: function getCurrentPlayer() {
        return this.currentPlayer;
    },

    /*
     * Mark every checker as not to be removed.
     */
    removeAllFromRemoveMarking: function removeAllFromRemoveMarking() {
        for (var row = 0; row <= 7; row++) {
            for (var col = 0; col <= 7; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.boardOfCheckers[row][col].clearToBeRemoved();
                }
            }
        }
    },

    /*
     * Check if game is finished.
     */
    isGameFinished: function isGameFinished() {
        var player1count = 0;
        var player2count = 0;

        var savedPlayer1 = null;
        var savedPlayer2 = null;

        var board = this.boardOfCheckers.map(function (arr) {
            return arr.slice();
        });

        for (var row = 0; row <= 7; row++) {
            for (var col = 0; col <= 7; col++) {
                if (board[row][col] != null) {
                    if (board[row][col].getPlayer() == "2") {
                        player2count++;
                        savedPlayer2 = board[row][col];
                    }
                    if (board[row][col].getPlayer() == "1") {
                        player1count++;
                        savedPlayer1 = board[row][col];
                    }
                }
            }
        }

        if (player2count == 0 && global.settings.playAs == 1 || player2count == 0 && global.settings.playAs == 0) {
            var overlay = this.phaser.add.image(0, 0, "overlay");
            overlay.width = this.phaser.width;
            overlay.height = this.phaser.height;
            overlay.alpha = 0.65;
            overlay.inputEnabled = true;
            overlay.visible = true;
            this.overlay = overlay;
            global.setPauseState(true);

            var player = "1";
            if (global.settings.playAs == 1) {
                var player = "2";
            }

            if (global.rules.aiLevel == 0) {
                if (this.currentPlayer.getPlayer() == 1) {
                    if (global.settings.playAs == 0) {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][0]);
                    } else {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][1]);
                    }
                } else {
                    if (global.settings.playAs == 0) {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][0]);
                    } else {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][1]);
                    }
                }
            } else {
                var win = new ScreenResult(this.game, this.world, this, true, this.gameState);
            }

            win.showScreen();
            return true;
        } else if (player1count == 0 && global.settings.playAs == 0 || player1count == 0 && global.settings.playAs == 1) {
            var overlay = this.phaser.add.image(0, 0, "overlay");
            overlay.width = this.phaser.width;
            overlay.height = this.phaser.height;
            overlay.alpha = 0.65;
            overlay.inputEnabled = true;
            overlay.visible = true;
            this.overlay = overlay;
            global.setPauseState(true);


            if (global.rules.aiLevel == 0) {
                if (this.currentPlayer.getPlayer() == 1) {
                    if (global.settings.playAs == 0) {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][0]);
                    } else {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][1]);
                    }
                } else {
                    if (global.settings.playAs == 0) {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][1]);
                    } else {
                        var win = new ScreenResult(this.game, this.world, this, true, this.gameState, this.winnersTexts[global.settings.stonesSkin][0]);
                    }
                }
            } else {
                var win = new ScreenResult(this.game, this.world, this, false, this.gameState);
            }

            win.showScreen();
            return true;
        } else if (player1count == 1 && player2count == 1) {
            // if (savedPlayer1.whatAmI() == 'king' && savedPlayer2.whatAmI() == 'king') {
            var overlay = this.phaser.add.image(0, 0, "overlay");
            overlay.width = this.phaser.width;
            overlay.height = this.phaser.height;
            overlay.alpha = 0.65;
            overlay.inputEnabled = true;
            overlay.visible = true;
            this.overlay = overlay;
            global.setPauseState(true);

            var win = new ScreenResult(this.game, this.world, this, null, this.gameState);

            win.showScreen();
            return true;
            // }
        }
        return false;
    },

    //--------------------------------------------------------------------------
    //---------------------------ETTERS AND SETTERS-----------------------------
    //--------------------------------------------------------------------------

    /*
     * Get game board checkers.
     */
    getGameBoardCheckers: function getGameBoardCheckers() {
        return this.boardOfCheckers;
    },

    /*
     * Set game board checkers.
     */
    setGameBoardCheckers: function setGameBoardCheckers(moves, data, removed, change) {
        var _this8 = this;

        this.getGameState().blockUndo();

        if (change) {
            this.changeCurrentPlayer();
        }

        this.moving = true;

        var to = global.getRowAndColFromGraph(moves[1]);
        var from = global.getRowAndColFromGraph(moves[0]);

        this.boardOfCheckers[from[0]][from[1]] = this.boardOfCheckers[to[0]][to[1]];
        this.boardOfCheckers[to[0]][to[1]] = null;

        for (var i = 0; i < removed.length; i++) {
            if (removed[i][2] == "1") {
                var pos = this.getXYValue(removed[i][0], removed[i][1]);
                var king = false;
                if (removed[i][3] == "king") {
                    king = true;
                }
                var checker = new Checker(this, row, col, -100, global.getRndInteger(this.world.height / 2 - 150, this.world.height / 2 + 150), global.getStonesSprites(), king, "1");
                this.boardOfCheckers[removed[i][0]][removed[i][1]] = checker;

                this.boardOfCheckers[removed[i][0]][removed[i][1]].tweenTo(pos[0], pos[1]);
            } else if (removed[i][2] == "2") {
                var pos = this.getXYValue(removed[i][0], removed[i][1]);
                var king = false;
                if (removed[i][3] == "king") {
                    king = true;
                }
                var checker = new Checker(this, row, col, this.world.width + 100, global.getRndInteger(this.world.height / 2 - 150, this.world.height / 2 + 150), global.getStonesSprites(), king, "2");
                this.boardOfCheckers[removed[i][0]][removed[i][1]] = checker;

                this.boardOfCheckers[removed[i][0]][removed[i][1]].tweenTo(pos[0], pos[1]);
            }
        }

        for (var row = 0; row < 8; row++) {
            for (var col = 0; col < 8; col++) {
                if (this.boardOfCheckers[row][col] != null) {
                    this.boardOfCheckers[row][col].setRow(data[row][col][0]);
                    this.boardOfCheckers[row][col].setCol(data[row][col][1]);
                    if (data[row][col][4] == "king" && this.boardOfCheckers[row][col].whatAmI() != "king") {
                        this.boardOfCheckers[row][col].changeToKing();
                        var pos = getXYValue(row, col);
                        this.fireParticle(pos[0], pos[1], global.particleAmount);
                    } else if (data[row][col][4] == "checker" && this.boardOfCheckers[row][col].whatAmI() != "checker") {
                        this.boardOfCheckers[row][col].changeToChecker();
                    }
                    this.boardOfCheckers[row][col].undoTweenTo(data[row][col][2], data[row][col][3]);
                }
            }
        }

        this.cleanShowPossibleMovesOnBoardArray();
        this.uncheckEveryCheckedChecker();
        this.cleanShowPossibleMovesOnBoardArray();
        this.stopAllPulsingCheckers();

        global.createTimer(global.times.TOP_CHECKERS_TWEEN_TIME * 3, function () {
            if (global.rules.capture == 0) {
                _this8.showPossibleCheckersOnBoard(0);
            } else if (global.rules.capture == 1) {
                _this8.showPossibleCheckersOnBoard(1);
            } else {
                _this8.showPossibleCheckersOnBoard(2);
            }
            _this8.gameState.unblockUndo();
        });
    },

    getGame: function getGame() {
        return this.game;
    },

    getHistoryObj: function getHistoryObj() {
        return this.gameHistory;
    },

    removeOneCheckerFromBoard: function removeOneCheckerFromBoard(row, col) {
        this.boardOfCheckers[row][col] = null;
    },

    getGameState: function getGameState() {
        return this.gameState;
    },

    getPaths: function getPaths() {
        return this.paths;
    },

    isMoving: function isMoving() {
        return this.moving;
    },

    setMoving: function setMoving(moving) {
        this.moving = moving;
    },

    setTutorial: function setTutorial(tutorial) {
        this.tutorial = tutorial;
    },

    //--------------------------------------------------------------------------
    //-----------------------------PATHS IN GRAPH-------------------------------
    //--------------------------------------------------------------------------

    /*
     * Build string for paths.
     */
    buildGraphStrings: function (_buildGraphStrings) {
        function buildGraphStrings(_x, _x2, _x3) {
            return _buildGraphStrings.apply(this, arguments);
        }

        buildGraphStrings.toString = function () {
            return _buildGraphStrings.toString();
        };

        return buildGraphStrings;
    }(function (arr, parent, c) {
        return arr.reduce(function (r, e) {
            if (e.parent == parent) {
                var children = buildGraphStrings(arr, e.key, c + e.key + '->');
                if (!children.length) result.push(c + e.key);
                r.push(e);
            }
            return r;
        }, []);
    }),


    /*
     * Get all possibles paths.
     */
    getAllPossibleWays: function getAllPossibleWays(data) {
        var buildGraphStrings = function buildGraphStrings(arr, parent, c) {
            return arr.reduce(function (r, e) {
                if (e.parent == parent) {
                    var children = buildGraphStrings(arr, e.key, c + e.key + '->');
                    if (!children.length) result.push(c + e.key);
                    r.push(e);
                }
                return r;
            }, []);
        };
        var result = [];
        buildGraphStrings(data, 'none', '');
        result = [].concat(_toConsumableArray(result));
        var result2 = this.validateResultPaths(result);
        return result2;
    },


    validateResultPaths: function validateResultPaths(paths) {
        var pathsRet = [];
        for (var i = 0; i < paths.length; i++) {
            var board = this.getBoardWithNumbers();
            var possitions = paths[i].split("->");
            var path = possitions[0] + "";
            if (possitions.length == 1 || possitions.length == 2) {
                pathsRet.push(paths[i]);
            } else {
                for (var j = 0; j < possitions.length - 1; j++) {
                    var pos1 = global.getRowAndColFromGraph(possitions[j]);
                    var pos2 = global.getRowAndColFromGraph(possitions[j + 1]);
                    var player = board[pos1[0]][pos1[1]];
                    var toRemove = this.isEnemyCheckerNumberBetween(pos1[0], pos1[1], pos2[0], pos2[1], board, player)
                    if (toRemove != null) {
                        board[toRemove[0]][toRemove[1]] = null;
                        path = path + "->" + possitions[j + 1];

                        if (possitions[j + 1] == possitions[possitions.length - 1]) {
                            pathsRet.push(path);
                        }
                    } else {
                        pathsRet.push(path);
                        break;
                    }
                }
            }
        }

        return pathsRet;
    },

    getBoardWithNumbers: function getBoardWithNumbers() {
        var boardNew = [
            [],
            [],
            [],
            [],
            [],
            [],
            [],
            []
        ];
        for (var i = 0; i < 8; i++) {
            for (var j = 0; j < 8; j++) {
                if (this.boardOfCheckers[i][j] == null) {
                    boardNew[i][j] = null;
                } else {
                    if (this.boardOfCheckers[i][j].getPlayer() == "1") {
                        boardNew[i][j] = 1;
                    } else {
                        boardNew[i][j] = 2;
                    }
                }
            }
        }
        return boardNew;
    },

    isEnemyCheckerNumberBetween: function isEnemyCheckerNumberBetween(rowFrom, colFrom, rowTo, colTo, board, player) {
        var vertical = rowFrom - rowTo;
        var horizontal = colFrom - colTo;
        var moveUp = true;
        var moveLeft = true;
        if (vertical < 0) {
            moveUp = false;
        }
        if (horizontal < 0) {
            moveLeft = false;
        }

        if (moveUp && moveLeft) {
            var i = 0;
            while (rowFrom - i != rowTo && colFrom - i != colTo) {
                if (board[rowFrom - i][colFrom - i] != null) {
                    if (board[rowFrom - i][colFrom - i] != player) {
                        return [[rowFrom - i], [colFrom - i]];
                    }
                }
                i++;
            }
        }
        if (!moveUp && !moveLeft) {
            var i = 0;
            while (rowFrom + i != rowTo && colFrom + i != colTo) {
                if (board[rowFrom + i][colFrom + i] != null) {
                    if (board[rowFrom + i][colFrom + i] != player) {
                        return [[rowFrom + i], [colFrom + i]];
                    }
                }
                i++;
            }
        }
        if (!moveUp && moveLeft) {
            var i = 0;
            while (rowFrom + i != rowTo && colFrom - i != colTo) {
                if (board[rowFrom + i][colFrom - i] != null) {
                    if (board[rowFrom + i][colFrom - i] != player) {
                        return [[rowFrom + i], [colFrom - i]];
                    }
                }
                i++;
            }
        }
        if (moveUp && !moveLeft) {
            var i = 0;
            while (rowFrom - i != rowTo && colFrom + i != colTo) {
                if (board[rowFrom - i][colFrom + i] != null) {
                    if (board[rowFrom - i][colFrom + i] != player) {
                        return [[rowFrom - i], [colFrom + i]];
                    }
                }
                i++;
            }
        }

        return null;
    },

    getBoardSprite: function getBoardSprite() {
        return this.this.boardBackground;
    },

    //----------------------------PARTICLE--------------------------------------
    initParticles: function initParticles() {
        this.getGame().physics.startSystem(Phaser.Physics.ARCADE);
        this.emitter = this.getGame().add.emitter(0, 0, global.particleAmount);
        this.emitter.makeParticles('imgpk', 'win_particle_star1');
        this.emitter.gravity = 0;
    },

    fireParticle: function fireParticle(xPos, yPos, count) {
        this.emitter.x = xPos;
        this.emitter.y = yPos;
        this.emitter.start(true, 600, null, count);
    }

};