'use strict';

var Checker = function Checker(boardObj, row, col, xPos, yPos, imageKey, king, player) {
    this.imageKey = imageKey;
    this.boardObj = boardObj;
    this.xPos = xPos;
    this.yPos = yPos;
    this.row = row;
    this.col = col;
    this.boardOfCheckers = boardObj.getGameBoardCheckers();

    this.game = boardObj.getGame();

    this.blocked = false;
    this.checked = false;

    this.pulsing = false;

    this.player = player;

    //for jumping recognition
    this.king = king;
    this.toBeRemovedList = [];

    this.possibleMovesGraph = [];
    this.possibleMovesGraphKing = [];
    this.possibleSimpleMovesGraph = [];

    this.canKingJumpLimit = 0;
    this.canChecerJumpLimit = 0;

    this.create();
};

Checker.prototype = {
    //--------------------------------------------------------------------------
    //------------------------CORE MOVE HANDELING-------------------------------
    //--------------------------------------------------------------------------
    cleanPossibleMoves: function cleanPossibleMoves() {
        this.possibleMovesGraph = [];
        this.possibleMovesGraphKing = [];
        this.possibleSimpleMovesGraph = [];
    },

    /*
     * Check if clicked checker has possibility to move.
     */
    canCheckerBeMoved: function canCheckerBeMoved(mode) {
        this.boardOfCheckers = this.boardObj.getGameBoardCheckers();

        if (mode == "check" && this.player == this.boardObj.getCurrentPlayer().getPlayer()) {
            if (!this.boardObj.isCheckerMarkedAsPossibleToMove(this)) {
                return false;
            }
        }

        this.cleanPossibleMoves();

        //add first move for jumo movement
        this.possibleMovesGraph.push({
            key: global.boardForGraph[this.row][this.col],
            parent: 'none'
        });

        //add first move for bacic movement
        this.possibleSimpleMovesGraph.push({
            key: global.boardForGraph[this.row][this.col],
            parent: 'none'
        });

        //call this function and fill posible moves arrays
        if (global.rules.king == 2 && this.king) {
            if (this.player == "2") {
                this.canCheckerJump(this.row, this.col);
                this.canCheckerBeMovedDown(this.row, this.col);
                this.canCheckerBeMovedUp(this.row, this.col);
            } else {
                this.canCheckerJump(this.row, this.col);
                this.canCheckerBeMovedDown(this.row, this.col);
                this.canCheckerBeMovedUp(this.row, this.col);
            }
        } else {
            if (this.player == "2") {
                if (global.rules.captureBackwards == true) {
                    this.canCheckerJump(this.row, this.col);
                } else {
                    this.canCheckerJumpDown(this.row, this.col);
                }
                this.canCheckerBeMovedDown(this.row, this.col);
            } else {
                if (global.rules.captureBackwards == true) {
                    this.canCheckerJump(this.row, this.col);
                } else {
                    this.canCheckerJumpUp(this.row, this.col);
                }
                this.canCheckerBeMovedUp(this.row, this.col);
            }
        }

        if (this.possibleMovesGraph.length > 1 || this.possibleSimpleMovesGraph.length > 1) {
            return true;
        } else {
            return false;
        }
    },

    /*
     * Check if clicked king has possibility to move.
     */
    canKingBeMoved: function canKingBeMoved(mode) {
        var bot = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        this.boardOfCheckers = this.boardObj.getGameBoardCheckers();

        if (mode == "check" && this.player == this.boardObj.getCurrentPlayer().getPlayer()) {
            if (!this.boardObj.isCheckerMarkedAsPossibleToMove(this)) {
                return false;
            }
        }

        this.cleanPossibleMoves();

        //add first move
        this.possibleMovesGraphKing.push({
            key: global.boardForGraph[this.row][this.col],
            parent: 'none'
        });

        this.possibleSimpleMovesGraph.push({
            key: global.boardForGraph[this.row][this.col],
            parent: 'none'
        });

        //call this function and fill posible moves arrays
        if (global.rules.king == 2) {
            this.canCheckerBeMoved();
            this.possibleMovesGraphKing = this.possibleMovesGraph;
        } else {
            this.canKingJump(this.row, this.col, bot);
            this.canKingBeMovedUp(this.row, this.col);
            this.canKingBeMovedDown(this.row, this.col);
        }

        if (this.possibleMovesGraphKing.length > 1 || this.possibleSimpleMovesGraph.length > 1) {
            return true;
        } else {
            return false;
        }
    },

    //--------------------------------------------------------------------------
    //--------------------------CHECKER MOVE------------------------------------
    //--------------------------------------------------------------------------
    /*
     * Check if clicked checker has possibility to upwards.
     */
    canCheckerBeMovedUp: function canCheckerBeMovedUp(row, col) {

        if (row - 1 >= 0 && col - 1 >= 0) {
            if (this.boardOfCheckers[row - 1][col - 1] == null) {
                this.possibleSimpleMovesGraph.push({
                    key: global.boardForGraph[row - 1][col - 1],
                    parent: global.boardForGraph[row][col]
                });
            }
        }
        if (row - 1 >= 0 && col + 1 < 8) {
            if (this.boardOfCheckers[row - 1][col + 1] == null) {
                this.possibleSimpleMovesGraph.push({
                    key: global.boardForGraph[row - 1][col + 1],
                    parent: global.boardForGraph[row][col]
                });
            }
        }
    },

    /*
     * Check if clicked checker has possibility to downwards.
     */
    canCheckerBeMovedDown: function canCheckerBeMovedDown(row, col) {
        if (row + 1 < 8 && col + 1 < 8) {
            if (this.boardOfCheckers[row + 1][col + 1] == null) {
                this.possibleSimpleMovesGraph.push({
                    key: global.boardForGraph[row + 1][col + 1],
                    parent: global.boardForGraph[row][col]
                });
            }
        }
        if (row + 1 < 8 && col - 1 >= 0) {
            if (this.boardOfCheckers[row + 1][col - 1] == null) {
                this.possibleSimpleMovesGraph.push({
                    key: global.boardForGraph[row + 1][col - 1],
                    parent: global.boardForGraph[row][col]
                });
            }
        }
    },

    /*
     * Check if clicked checker has possibility to jump up.
     */
    canCheckerJumpUp: function canCheckerJumpUp(row, col) {
        var upLeft = this.canCheckerJumpUpLeft(row, col);
        var upRight = this.canCheckerJumpUpRight(row, col);

        if (upLeft || upRight) {
            if (this.possibleMovesGraph.length - 1 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 1];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJumpUp(possible[0], possible[1]);
                }
            }
            if (this.possibleMovesGraph.length - 2 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 2];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJumpUp(possible[0], possible[1]);
                }
            }
        }
    },

    /*
     * Check if clicked checker has possibility to jump down.
     */
    canCheckerJumpDown: function canCheckerJumpDown(row, col) {

        var downLeft = this.canCheckerJumpDownLeft(row, col);
        var downRight = this.canCheckerJumpDownRight(row, col);

        if (downLeft || downRight) {
            if (this.possibleMovesGraph.length - 1 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 1];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJumpDown(possible[0], possible[1]);
                }
            }
            if (this.possibleMovesGraph.length - 2 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 2];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJumpDown(possible[0], possible[1]);
                }
            }
        }
    },

    /*
     * Check if clicked checker has possibility to jump.
     */
    canCheckerJump: function canCheckerJump(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;


        var upLeft = this.canCheckerJumpUpLeft(row, col, bot);
        var upRight = this.canCheckerJumpUpRight(row, col, bot);
        var downLeft = this.canCheckerJumpDownLeft(row, col, bot);
        var downRight = this.canCheckerJumpDownRight(row, col, bot);

        if (upLeft || upRight || downLeft || downRight) {
            if (this.possibleMovesGraph.length - 1 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 1];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJump(possible[0], possible[1], bot);
                }
            }
            if (this.possibleMovesGraph.length - 2 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 2];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJump(possible[0], possible[1], bot);
                }
            }
            if (this.possibleMovesGraph.length - 3 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 3];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJump(possible[0], possible[1], bot);
                }
            }
            if (this.possibleMovesGraph.length - 4 >= this.canChecerJumpLimit) {
                var possible = this.possibleMovesGraph[this.possibleMovesGraph.length - 4];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canCheckerJump(possible[0], possible[1], bot);
                }
            }
        }
    },

    /*
     * Check if clicked checker has possibility to jump upLeft.
     */
    canCheckerJumpUpLeft: function canCheckerJumpUpLeft(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (row - 2 >= 0 && col - 2 >= 0) {
            if (this.boardOfCheckers[row - 1][col - 1] != null) {
                if (this.boardOfCheckers[row - 1][col - 1].getPlayer() != this.player && this.boardOfCheckers[row - 2][col - 2] == null) {
                    if (this.canJumpOnAlreadyJumped(row, col, row - 2, col - 2) == true) {
                        if (!bot) {
                            this.possibleMovesGraph.push({
                                key: global.boardForGraph[row - 2][col - 2],
                                parent: global.boardForGraph[row][col]
                            });
                        }

                        return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
            return false;
        }
        return false;
    },

    /*
     * Check if clicked checker has possibility to jump upRight.
     */
    canCheckerJumpUpRight: function canCheckerJumpUpRight(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (row == 4 && col == 3) {
            console.log();
        }

        if (row - 2 >= 0 && col + 2 < 8) {
            if (this.boardOfCheckers[row - 1][col + 1] != null) {
                if (this.boardOfCheckers[row - 1][col + 1].getPlayer() != this.player && this.boardOfCheckers[row - 2][col + 2] == null) {
                    if (this.canJumpOnAlreadyJumped(row, col, row - 2, col + 2) == true) {
                        if (!bot) {
                            this.possibleMovesGraph.push({
                                key: global.boardForGraph[row - 2][col + 2],
                                parent: global.boardForGraph[row][col]
                            });
                        }

                        return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
            return false;
        }
        return false;
    },

    /*
     * Check if clicked checker has possibility to jump downRight.
     */
    canCheckerJumpDownRight: function canCheckerJumpDownRight(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (row + 2 < 8 && col + 2 < 8) {
            if (this.boardOfCheckers[row + 1][col + 1] != null) {
                if (this.boardOfCheckers[row + 1][col + 1].getPlayer() != this.player && this.boardOfCheckers[row + 2][col + 2] == null) {
                    if (this.canJumpOnAlreadyJumped(row, col, row + 2, col + 2) == true) {
                        if (!bot) {

                            this.possibleMovesGraph.push({
                                key: global.boardForGraph[row + 2][col + 2],
                                parent: global.boardForGraph[row][col]
                            });
                        }

                        return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
            return false;
        }
        return false;
    },

    /*
     * Check if clicked checker has possibility to jump downLeft.
     */
    canCheckerJumpDownLeft: function canCheckerJumpDownLeft(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (row + 2 < 8 && col - 2 >= 0) {
            if (this.boardOfCheckers[row + 1][col - 1] != null) {
                if (this.boardOfCheckers[row + 1][col - 1].getPlayer() != this.player && this.boardOfCheckers[row + 2][col - 2] == null) {
                    if (this.canJumpOnAlreadyJumped(row, col, row + 2, col - 2) == true) {
                        if (!bot) {
                            this.possibleMovesGraph.push({
                                key: global.boardForGraph[row + 2][col - 2],
                                parent: global.boardForGraph[row][col]
                            });
                        }

                        return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
            return false;
        }
        return false;
    },

    /*
     * Check if can jump on already jumped on.
     */
    canJumpOnAlreadyJumped: function canJumpOnAlreadyJumped(rowFrom, colFrom, rowTo, colTo) {
        var list = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : this.possibleMovesGraph;

        for (var i = 0; i < list.length; i++) {
            //ceck if can do same jump from other side
            if (global.boardForGraph[rowTo][colTo] == list[i].key && global.boardForGraph[rowFrom][colFrom] == list[i].parent) {
                return false;
            }
            if (global.boardForGraph[rowTo][colTo] == list[i].parent && global.boardForGraph[rowFrom][colFrom] == list[i].key) {
                return false;
            }
            //protect infinite loop
            for (var j = list.length - 1; j >= 0; j--) {
                if (global.boardForGraph[rowTo][colTo] == list[j].parent) {
                    return false;
                }
            }
        }

        return true;
    },

    //--------------------------------------------------------------------------
    //-----------------------------KING MOVE------------------------------------
    //--------------------------------------------------------------------------
    /*
     * Check if clicked king has possibility to upwards.
     */
    canKingBeMovedUp: function canKingBeMovedUp(row, col) {

        for (var i = 1; i < 8; i++) {
            if (row - i >= 0 && col - i >= 0) {
                if (this.boardOfCheckers[row - i][col - i] == null) {
                    this.possibleSimpleMovesGraph.push({
                        key: global.boardForGraph[row - i][col - i],
                        parent: global.boardForGraph[row][col]
                    });
                } else {
                    i = 8;
                }
            }
        }
        for (var i = 1; i < 8; i++) {
            if (row - i >= 0 && col + i < 8) {
                if (this.boardOfCheckers[row - i][col + i] == null) {
                    this.possibleSimpleMovesGraph.push({
                        key: global.boardForGraph[row - i][col + i],
                        parent: global.boardForGraph[row][col]
                    });
                } else {
                    i = 8;
                }
            }
        }
    },

    /*
     * Check if clicked king has possibility to downwards.
     */
    canKingBeMovedDown: function canKingBeMovedDown(row, col) {

        for (var i = 1; i < 8; i++) {
            if (row + i < 8 && col + i < 8) {
                if (this.boardOfCheckers[row + i][col + i] == null) {
                    this.possibleSimpleMovesGraph.push({
                        key: global.boardForGraph[row + i][col + i],
                        parent: global.boardForGraph[row][col]
                    });
                } else {
                    i = 8;
                }
            }
        }

        for (var i = 1; i < 8; i++) {
            if (row + i < 8 && col - i >= 0) {
                if (this.boardOfCheckers[row + i][col - i] == null) {
                    this.possibleSimpleMovesGraph.push({
                        key: global.boardForGraph[row + i][col - i],
                        parent: global.boardForGraph[row][col]
                    });
                } else {
                    i = 8;
                }
            }
        }
    },

    /*
     * Check if clicked king has possibility to jump.
     */
    canKingJump: function canKingJump(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        var upLeft = this.canKingJumpUpLeft(row, col, bot);
        var upRight = this.canKingJumpUpRight(row, col, bot);
        var downLeft = this.canKingJumpDownLeft(row, col, bot);
        var downRight = this.canKingJumpDownRight(row, col, bot);
        
        if (upLeft || upRight || downLeft || downRight) {
            if (this.possibleMovesGraphKing.length - 1 >= this.canKingJumpLimit) {
                var possible = this.possibleMovesGraphKing[this.possibleMovesGraphKing.length - 1];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canKingJump(possible[0], possible[1], bot);
                }
            }
            if (this.possibleMovesGraphKing.length - 2 >= this.canKingJumpLimit) {
                var possible = this.possibleMovesGraphKing[this.possibleMovesGraphKing.length - 2];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canKingJump(possible[0], possible[1], bot);
                }
            }
            if (this.possibleMovesGraphKing.length - 3 >= this.canKingJumpLimit) {
                var possible = this.possibleMovesGraphKing[this.possibleMovesGraphKing.length - 3];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canKingJump(possible[0], possible[1], bot);
                }
            }
            if (this.possibleMovesGraphKing.length - 4 >= this.canKingJumpLimit) {
                var possible = this.possibleMovesGraphKing[this.possibleMovesGraphKing.length - 4];
                possible = global.getRowAndColFromGraph(possible.key);
                if (possible[0] != row && possible[1] != col) {
                    this.canKingJump(possible[0], possible[1], bot);
                }
            }
            return true;
        }
        return false;
    },

    /*
     * Check if clicked king has possibility to jump upLeft.
     */
    canKingJumpUpLeft: function canKingJumpUpLeft(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        for (var i = 1; i < 8; i++) {
            if (row - i >= 1 && col - i >= 1) {
                if (this.boardOfCheckers[row - i][col - i] != null) {

                    if (!bot) {
                        if (this.boardOfCheckers[row - i][col - i].getPlayer() == this.boardObj.getCurrentPlayer().getPlayer()) {
                            return false;
                        }
                    }

                    //check same player in way
                    if (this.boardOfCheckers[row - i][col - i] != null) {
                        if (this.boardOfCheckers[row - i][col - i].getPlayer() == this.player) {
                            return false;
                        }
                    }

                    //check if two enemies in row, that can't be overjumped
                    if (this.boardOfCheckers[row - i - 1][col - i - 1] != null) {
                        if (this.boardOfCheckers[row - i][col - i].getPlayer() != this.player && this.boardOfCheckers[row - i - 1][col - i - 1].getPlayer() != this.player) {
                            return false;
                        }
                    }

                    if (global.rules.king == 2) {
                        if (this.boardOfCheckers[row - 1][col - 1] == null) {
                            return false;
                        }
                    }

                    if (this.canJumpOnAlreadyJumped(row, col, row - i - 1, col - i - 1, this.possibleMovesGraphKing) == true) {

                        if (this.boardOfCheckers[row - i][col - i].getPlayer() != this.player && this.boardOfCheckers[row - i - 1][col - i - 1] == null && this.boardOfCheckers[row - i][col - i].getToBeRemoved().indexOf(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row - i][col - i]) + " " + global.boardForGraph[row - i][col - i]) == -1) {

                            if (this.boardOfCheckers[row][col] != null) {
                                if (this.boardOfCheckers[row][col].getChecked() == true) {
                                    return false;
                                }
                            }

                            if (global.rules.king == 0) {
                                for (var op = 1; op <= 8; op++) {

                                    if (row - i - op >= 0 && col - i - op >= 0 && this.boardOfCheckers[row - i - op][col - i - op] == null) {
                                        var pu = {
                                            key: global.boardForGraph[row - i - op][col - i - op],
                                            parent: global.boardForGraph[row][col]
                                        };
                                        if (this.doContains(pu)) {
                                            this.possibleMovesGraphKing.push(pu);
                                        }
                                    } else {
                                        break;
                                    }
                                }
                            } else if (global.rules.king == 1) {
                                var pu = {
                                    key: global.boardForGraph[row - i - 1][col - i - 1],
                                    parent: global.boardForGraph[row][col]
                                };
                                if (this.doContains(pu)) {
                                    this.possibleMovesGraphKing.push(pu);
                                }
                            }

                            this.boardOfCheckers[row - i][col - i].setToBeRemoved(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row - i][col - i]) + " " + global.boardForGraph[row - i][col - i]);

                            return true;
                        }
                    }
                }
            }
        }
        return false;
    },

    /*
     * Check if clicked king has possibility to jump upRight.
     */
    canKingJumpUpRight: function canKingJumpUpRight(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        for (var i = 1; i < 8; i++) {
            if (row - i >= 1 && col + i < 7) {
                if (this.boardOfCheckers[row - i][col + i] != null) {

                    if (!bot) {
                        if (this.boardOfCheckers[row - i][col + i].getPlayer() == this.boardObj.getCurrentPlayer().getPlayer()) {
                            return false;
                        }
                    }

                    //check same player in way
                    if (this.boardOfCheckers[row - i][col + i] != null) {
                        if (this.boardOfCheckers[row - i][col + i].getPlayer() == this.player) {
                            return false;
                        }
                    }

                    if (this.boardOfCheckers[row - i - 1][col + i + 1] != null) {
                        if (this.boardOfCheckers[row - i][col + i].getPlayer() != this.player && this.boardOfCheckers[row - i - 1][col + i + 1].getPlayer() != this.player) {
                            return false;
                        }
                    }

                    if (global.rules.king == 2) {
                        if (this.boardOfCheckers[row - 1][col + 1] == null) {
                            return false;
                        }
                    }

                    if (this.canJumpOnAlreadyJumped(row, col, row - i - 1, col + i + 1, this.possibleMovesGraphKing) == true) {

                        if (this.boardOfCheckers[row - i][col + i].getPlayer() != this.player && this.boardOfCheckers[row - i - 1][col + i + 1] == null && this.boardOfCheckers[row - i][col + i].getToBeRemoved().indexOf(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row - i][col + i]) + " " + global.boardForGraph[row - i][col + i]) == -1) {

                            if (this.boardOfCheckers[row][col] != null) {
                                if (this.boardOfCheckers[row][col].getChecked() == true) {
                                    return false;
                                }
                            }

                            if (global.rules.king == 0) {
                                for (var op = 1; op <= 8; op++) {
                                    if (row - i - op >= 0 && col + i + op < 8 && this.boardOfCheckers[row - i - op][col + i + op] == null) {
                                        var pu = {
                                            key: global.boardForGraph[row - i - op][col + i + op],
                                            parent: global.boardForGraph[row][col]
                                        };
                                        if (this.doContains(pu)) {
                                            this.possibleMovesGraphKing.push(pu);
                                        }
                                    } else {
                                        break;
                                    }
                                }
                            } else if (global.rules.king == 1) {
                                var pu = {
                                    key: global.boardForGraph[row - i - 1][col + i + 1],
                                    parent: global.boardForGraph[row][col]
                                };
                                if (this.doContains(pu)) {
                                    this.possibleMovesGraphKing.push(pu);
                                }
                            }

                            this.boardOfCheckers[row - i][col + i].setToBeRemoved(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row - i][col + i]) + " " + global.boardForGraph[row - i][col + i]);

                            return true;
                        }
                    }
                }
            }
        }
        return false;
    },

    /*
     * Check if clicked king has possibility to jump downRight.
     */
    canKingJumpDownRight: function canKingJumpDownRight(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        for (var i = 1; i < 8; i++) {
            if (row + i < 7 && col + i < 7) {
                if (this.boardOfCheckers[row + i][col + i] != null) {

                    if (!bot) {
                        if (this.boardOfCheckers[row + i][col + i].getPlayer() == this.boardObj.getCurrentPlayer().getPlayer()) {
                            return false;
                        }
                    }

                    //check same player in way
                    if (this.boardOfCheckers[row + i][col + i] != null) {
                        if (this.boardOfCheckers[row + i][col + i].getPlayer() == this.player) {
                            return false;
                        }
                    }

                    //check if two enemies in row, that can't be overjumped
                    if (this.boardOfCheckers[row + i + 1][col + i + 1] != null) {
                        if (this.boardOfCheckers[row + i][col + i].getPlayer() != this.player && this.boardOfCheckers[row + i + 1][col + i + 1].getPlayer() != this.player) {
                            return false;
                        }
                    }

                    if (global.rules.king == 2) {
                        if (this.boardOfCheckers[row + 1][col + 1] == null) {
                            return false;
                        }
                    }

                    if (this.canJumpOnAlreadyJumped(row, col, row + i + 1, col + i + 1, this.possibleMovesGraphKing) == true) {
                        if (this.boardOfCheckers[row + i][col + i].getPlayer() != this.player && this.boardOfCheckers[row + i + 1][col + i + 1] == null && this.boardOfCheckers[row + i][col + i].getToBeRemoved().indexOf(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row + i][col + i]) + " " + global.boardForGraph[row + i][col + i]) == -1) {

                            if (this.boardOfCheckers[row][col] != null) {
                                if (this.boardOfCheckers[row][col].getChecked() == true) {
                                    return false;
                                }
                            }

                            if (global.rules.king == 0) {
                                for (var op = 1; op <= 8; op++) {
                                    if (row + i + op < 8 && col + i + op < 8 && this.boardOfCheckers[row + i + op][col + i + op] == null) {
                                        var pu = {
                                            key: global.boardForGraph[row + i + op][col + i + op],
                                            parent: global.boardForGraph[row][col]
                                        };
                                        if (this.doContains(pu)) {
                                            this.possibleMovesGraphKing.push(pu);
                                        }
                                    } else {
                                        break;
                                    }
                                }
                            } else if (global.rules.king == 1) {
                                var pu = {
                                    key: global.boardForGraph[row + i + 1][col + i + 1],
                                    parent: global.boardForGraph[row][col]
                                };
                                if (this.doContains(pu)) {
                                    this.possibleMovesGraphKing.push(pu);
                                }
                            }

                            this.boardOfCheckers[row + i][col + i].setToBeRemoved(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row + i][col + i]) + " " + global.boardForGraph[row + i][col + i]);

                            return true;
                        }
                    }
                }
            }
        }
        return false;
    },

    /*
     * Check if clicked king has possibility to jump downLeft.
     */
    canKingJumpDownLeft: function canKingJumpDownLeft(row, col) {
        var bot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        for (var i = 1; i < 8; i++) {
            if (row + i < 7 && col - i >= 1) {
                if (this.boardOfCheckers[row + i][col - i] != null) {

                    if (!bot) {
                        if (this.boardOfCheckers[row + i][col - i].getPlayer() == this.boardObj.getCurrentPlayer().getPlayer()) {
                            return false;
                        }
                    }

                    //check same player in way
                    if (this.boardOfCheckers[row + i][col - i] != null) {
                        if (this.boardOfCheckers[row + i][col - i].getPlayer() == this.player) {
                            return false;
                        }
                    }

                    //check if two enemies in row, that can't be overjumped
                    if (this.boardOfCheckers[row + i + 1][col - i - 1] != null) {
                        if (this.boardOfCheckers[row + i][col - i].getPlayer() != this.player && this.boardOfCheckers[row + i + 1][col - i - 1].getPlayer() != this.player) {
                            return false;
                        }
                    }

                    if (global.rules.king == 2) {
                        if (this.boardOfCheckers[row + 1][col - 1] == null) {
                            return false;
                        }
                    }

                    if (this.canJumpOnAlreadyJumped(row, col, row + i + 1, col - i - 1, this.possibleMovesGraphKing) == true) {
                        //TODO: need to change players bacause checking the same players so return false, instead of player
                        if (this.boardOfCheckers[row + i][col - i].getPlayer() != this.player && this.boardOfCheckers[row + i + 1][col - i - 1] == null && this.boardOfCheckers[row + i][col - i].getToBeRemoved().indexOf(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row + i][col - i]) + " " + global.boardForGraph[row + i][col - i]) == -1) {

                            if (this.boardOfCheckers[row][col] != null) {
                                if (this.boardOfCheckers[row][col].getChecked() == true) {
                                    return false;
                                }
                            }

                            if (global.rules.king == 0) {
                                for (var op = 1; op <= 8; op++) {
                                    if (row + i + op < 8 && col - i - op >= 0 && this.boardOfCheckers[row + i + op][col - i - op] == null) {
                                        var pu = {
                                            key: global.boardForGraph[row + i + op][col - i - op],
                                            parent: global.boardForGraph[row][col]
                                        };
                                        if (this.doContains(pu)) {
                                            this.possibleMovesGraphKing.push(pu);
                                        }
                                    } else {
                                        break;
                                    }
                                }
                            } else if (global.rules.king == 1) {
                                var pu = {
                                    key: global.boardForGraph[row + i + 1][col - i - 1],
                                    parent: global.boardForGraph[row][col]
                                };
                                if (this.doContains(pu)) {
                                    this.possibleMovesGraphKing.push(pu);
                                }
                            }

                            this.boardOfCheckers[row + i][col - i].setToBeRemoved(global.getDiagonal(global.boardForGraph[row][col], global.boardForGraph[row + i][col - i]) + " " + global.boardForGraph[row + i][col - i]);

                            return true;
                        }
                    }
                }
            }
        }
        return false;
    },

    /*
     * Change checker to king.
     */
    changeToKing: function changeToKing() {
        this.king = true;
        global.managers.audio.playSound("jingle_dama");
        if (global.settings.playAs == 0) {
            if (this.player == "1") {
                this.checkerSprite.loadTexture(this.image, 4);
            } else {
                this.checkerSprite.loadTexture(this.image, 5);
            }
        } else {
            if (this.player == "1") {
                this.checkerSprite.loadTexture(this.image, 5);
            } else {
                this.checkerSprite.loadTexture(this.image, 4);
            }
        }
        // this.boardObj.isGameFinished();
    },

    /*
     * Change king to checker.
     */
    changeToChecker: function changeToChecker() {
        this.king = false;
        if (global.settings.playAs == 0) {
            if (this.player == "1") {
                this.checkerSprite.loadTexture(this.image, 0);
            } else {
                this.checkerSprite.loadTexture(this.image, 1);
            }
        } else {
            if (this.player == "1") {
                this.checkerSprite.loadTexture(this.image, 1);
            } else {
                this.checkerSprite.loadTexture(this.image, 0);
            }
        }
    },

    /*
     * Get data for history.
     */
    getDataToHistory: function getDataToHistory() {
        if (this.king) {
            return [this.row, this.col, this.xPos, this.yPos, "king"];
        } else {
            return [this.row, this.col, this.xPos, this.yPos, "checker"];
        }
    },

    /*
     * Create checker object and put it to board.
     */
    create: function create() {
        this.checkerSprite = this.game.add.sprite(this.xPos, this.yPos, this.imageKey, 0);

        if (global.settings.playAs == 0) {
            if (!this.king) {
                if (this.player === '1') {
                    this.checkerSprite.frame = 0;
                } else {
                    this.checkerSprite.frame = 1;
                }
            } else {
                if (this.player === '1') {
                    this.checkerSprite.frame = 4;
                } else {
                    this.checkerSprite.frame = 5;
                }
            }
        } else {
            if (!this.king) {
                if (this.player === '1') {
                    this.checkerSprite.frame = 1;
                } else {
                    this.checkerSprite.frame = 0;
                }
            } else {
                if (this.player === '1') {
                    this.checkerSprite.frame = 5;
                } else {
                    this.checkerSprite.frame = 4;
                }
            }
        }

        this.checkerSprite.inputEnabled = true;
        this.checkerSprite.anchor.setTo(0.5);

        if (Number(this.player) == 1 || global.rules.aiLevel == 0) {
            this.checkerSprite.events.onInputDown.add(this.onTouch, this);
        }
    },

    setInput: function setInput(val) {
        this.checkerSprite.inputEnabled = val;
    },

    /*
     * On touch listener.
     * Check if checker can be moved and check that checker.
     */
    onTouch: function onTouch() {
        this.boardObj.removeAllFromRemoveMarking();
        this.boardObj.cleanShowPossibleMovesOnBoardArray();

        if (this.boardObj.isPossibleCheckersShown()) {
            this.boardObj.getGame().world.bringToTop(this.checkerSprite);
            global.managers.audio.playSound("figurina_tapnutie");

            if (this.checked == true) {
                this.checkerUncheck(this);
                //clean possible moves form board
                this.boardObj.cleanShowPossibleMovesOnBoardArray();
                this.boardObj.uncheckEveryCheckedChecker();
                this.boardObj.removeAllFromRemoveMarking();

                this.cleanPossibleMoves();

                var captureMode = 0;
                if (global.rules.capture == 0) {
                    captureMode = 0;
                } else if (global.rules.capture == 1) {
                    captureMode = 1;
                } else {
                    captureMode = 2;
                }
                this.boardObj.showPossibleCheckersOnBoard(captureMode);
                return;
            }

            this.boardObj.stopAllPulsingCheckers();

            this.boardObj.cleanShowPossibleMovesOnBoardArray();
            this.boardObj.uncheckEveryCheckedChecker();
            this.boardObj.removeAllFromRemoveMarking();

            this.cleanPossibleMoves();

            var captureMode = 2;
            if (!this.boardObj.isMoving()) {
                if (global.rules.capture == 0) {
                    captureMode = 0;
                } else if (global.rules.capture == 1) {
                    captureMode = 1;
                } else {
                    captureMode = 2;
                }
            }

            if (!this.boardObj.isMoving()) {
                if (!this.king && this.canCheckerBeMoved("check") && !this.blocked && this.boardObj.getCurrentPlayer().getPlayer() == this.player) {
                    this.checkerCheck();
                    this.checked = true;
                    this.boardObj.showPossibleMovesOnBoard(this.possibleMovesGraph, this.possibleSimpleMovesGraph, captureMode);
                } else if (this.king && this.canKingBeMoved("check") && !this.blocked && this.boardObj.getCurrentPlayer().getPlayer() == this.player) {
                    this.checkerCheck();
                    this.checked = true;

                    console.log("THIS IS #1 BULLSHIT__THIS IS #1 BULLSHIT__THIS IS #1 BULLSHIT");
                    if (this.possibleMovesGraphKing.length > 1) {
                        for (var i = 0; i < this.possibleMovesGraphKing.length; i++) {
                            var pos = global.getRowAndColFromGraph(this.possibleMovesGraphKing[i].key);
                            this.canKingJump(pos[0], pos[1]);
                        }
                    }

                    this.boardObj.showPossibleMovesOnBoard(this.possibleMovesGraphKing, this.possibleSimpleMovesGraph, captureMode);
                } else {
                    this.shakeChecker();
                    if (!this.boardObj.isMoving()) {
                        this.boardObj.showPossibleCheckersOnBoard(captureMode);
                    }
                }
            }
        } else {
            console.log("negative");
            global.managers.audio.playSound("response_negative");
        }
    },

    onTouchBot: function onTouchBot(captureMode) {
        var player = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.boardObj.getCurrentPlayer().getPlayer();
        this.boardObj.getGame().world.bringToTop(this.checkerSprite);

        this.checkerUncheck(this);
        //clean possible moves form board

        this.boardObj.cleanShowPossibleMovesOnBoardArray();
        this.boardObj.uncheckEveryCheckedChecker();
        this.boardObj.removeAllFromRemoveMarking();

        this.cleanPossibleMoves();

        this.blocked = false;

        if (player != this.boardObj.getCurrentPlayer().getPlayer()) {
            //only jumping moves for bot purposes

            if (!this.king && this.canCheckerBeMoved("check") && !this.blocked && player == this.player) {
                this.checkerCheck();
                this.checked = true;
                this.boardObj.showPossibleMovesOnBoard(this.possibleMovesGraph, [], captureMode, false);
            } else if (this.king && this.canKingBeMoved("check", true) && !this.blocked && player == this.player) {
                this.checkerCheck();
                this.checked = true;

                console.log("THIS IS #2 BULLSHIT__THIS IS #2 BULLSHIT__THIS IS #2 BULLSHIT");
                if (this.possibleMovesGraphKing.length > 1) {
                    for (var i = 0; i < this.possibleMovesGraphKing.length; i++) {
                        var pos = global.getRowAndColFromGraph(this.possibleMovesGraphKing[i].key);
                        this.canKingJump(pos[0], pos[1], true);
                    }
                }

                console.log("THIS IS #1 BULLSHIT__THIS IS #1 BULLSHIT__THIS IS #1 BULLSHIT");
                if (this.possibleMovesGraphKing.length > 1) {
                    for (var i = 0; i < this.possibleMovesGraphKing.length; i++) {
                        var pos = global.getRowAndColFromGraph(this.possibleMovesGraphKing[i].key);
                        this.canKingJump(pos[0], pos[1], true);
                    }
                }

                this.boardObj.showPossibleMovesOnBoard(this.possibleMovesGraphKing, [], captureMode, false);
            } else {
                // this.shakeChecker();
                if (!this.boardObj.isMoving()) {
                    this.boardObj.showPossibleCheckersOnBoard(captureMode);
                }
            }
        } else {
            if (!this.king && this.canCheckerBeMoved("check") && !this.blocked && player == this.player) {
                this.checkerCheck();
                this.checked = true;

                this.boardObj.showPossibleMovesOnBoard(this.possibleMovesGraph, this.possibleSimpleMovesGraph, captureMode, false);
            } else if (this.king && this.canKingBeMoved("check", true) && !this.blocked && player == this.player) {
                this.checkerCheck();
                this.checked = true;

                console.log("THIS IS #3 BULLSHIT__THIS IS #3 BULLSHIT__THIS IS #3 BULLSHIT");
                if (this.possibleMovesGraphKing.length > 1) {
                    for (var i = 0; i < this.possibleMovesGraphKing.length; i++) {
                        var pos = global.getRowAndColFromGraph(this.possibleMovesGraphKing[i].key);
                        this.canKingJump(pos[0], pos[1], true);
                    }
                }

                this.boardObj.showPossibleMovesOnBoard(this.possibleMovesGraphKing, this.possibleSimpleMovesGraph, captureMode, false);
            } else {
                // this.shakeChecker();
                if (!this.boardObj.isMoving()) {
                    this.boardObj.showPossibleCheckersOnBoard(captureMode);
                }
            }
        }
        this.blocked = true;
    },

    displayBoardInConsole: function displayBoardInConsole() {
        var board = this.boardObj.getGameBoardCheckers();
        for (var i = 0; i < 8; i++) {
            var str = "";
            for (var j = 0; j < 8; j++) {
                if (board[i][j] == null) {
                    str += " ";
                } else {
                    str += board[i][j].getPlayer();
                }
            }
        }
    },

    shakeChecker: function shakeChecker() {
        var shake = new Phaser.Plugin.Shake(this.boardObj.getGame());
        this.boardObj.getGame().plugins.add(shake);
        shake.shake(); // default displacement
        shake.shake(10); // 40 pixels max displacement
        shake.shake(15, this.checkerSprite);
    },

    /*
     * Check checker.
     */
    checkerCheck: function checkerCheck() {
        this.image = global.getStonesSprites();
        if (global.settings.playAs == 1) {
            if (!this.king) {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 3);
                } else {
                    this.checkerSprite.loadTexture(this.image, 2);
                }
            } else {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 7);
                } else {
                    this.checkerSprite.loadTexture(this.image, 6);
                }
            }
        } else {
            if (!this.king) {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 2);
                } else {
                    this.checkerSprite.loadTexture(this.image, 3);
                }
            } else {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 6);
                } else {
                    this.checkerSprite.loadTexture(this.image, 7);
                }
            }
        }
    },

    /*
     * Uncheck checker.
     */
    checkerUncheck: function checkerUncheck() {
        this.image = global.getStonesSprites();
        if (global.settings.playAs == 1) {
            if (!this.king) {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 1);
                } else {
                    this.checkerSprite.loadTexture(this.image, 0);
                }
            } else {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 5);
                } else {
                    this.checkerSprite.loadTexture(this.image, 4);
                }
            }
        } else {
            if (!this.king) {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 0);
                } else {
                    this.checkerSprite.loadTexture(this.image, 1);
                }
            } else {
                if (this.player == "1") {
                    this.checkerSprite.loadTexture(this.image, 4);
                } else {
                    this.checkerSprite.loadTexture(this.image, 5);
                }
            }
        }
    },

    /*
     * Move checker smoothly to position.
     */
    tweenTo: function tweenTo(desiredX, desiredY) {
        this.boardObj.getGameState().blockUndo();

        this.xPos = desiredX;
        this.yPos = desiredY;

        var tween = this.boardObj.getGame().add.tween(this.checkerSprite).to({
            x: desiredX,
            y: desiredY
        }, global.times.TOP_CHECKERS_TWEEN_TIME * 3, "Quart.easeOut");
        tween.onStart.add(function () {
            global.managers.audio.playSound("suchnutie1");
        });
        return tween;
    },

    /*
     * Tween checker back to board.
     */
    undoTweenTo: function undoTweenTo(desiredX, desiredY) {
        global.managers.audio.playSound("suchnutie1");
        this.boardObj.getGameState().blockUndo();

        var _this = this;

        var tween = this.boardObj.getGame().add.tween(this.checkerSprite).to({
            x: desiredX,
            y: desiredY
        }, global.times.TOP_CHECKERS_TWEEN_TIME * 3, "Quart.easeOut");
        tween.start();

        tween.onComplete.add(function () {
            _this.boardObj.setMoving(false);
            _this.boardObj.uncheckEveryCheckedChecker();
            _this.boardObj.storeBoard();
        });

        this.xPos = desiredX;
        this.yPos = desiredY;

        //clean possible moves form board
        this.boardObj.cleanShowPossibleMovesOnBoardArray();

        this.toBeRemovedList = [];
    },

    /*
     * Tween checker out from the board.
     */
    tweenFromBoard: function tweenFromBoard(desiredX, desiredY) {
        var _this2 = this;

        this.boardObj.getGameState().blockUndo();

        global.createTimer(100, function () {
            _this2.boardObj.getGame().world.bringToTop(_this2.checkerSprite);
        });

        this.checkerSprite.inputEnabled = false;

        this.xPos = desiredX;
        this.yPos = desiredY;

        //clean possible moves form board
        this.boardObj.cleanShowPossibleMovesOnBoardArray();

        var tw = this.boardObj.getGame().add.tween(this.checkerSprite).to({
            x: desiredX,
            y: desiredY
        }, global.times.TOP_CHECKERS_TWEEN_TIME * 3, "Quart.easeOut");

        tw.onStart.add(function () {
            global.managers.audio.playSound("odkotulanie");
        });

        return tw;
    },

    startPulseChecker: function startPulseChecker() {
        this.pulsing = true;
        this.playTween1 = this.boardObj.getGame().add.tween(this.checkerSprite.scale).to({
            x: [1.1, 1.1],
            y: [1.1, 1.1]
        }, 200, Phaser.Easing.Linear.None, false, 0);
        this.playTween1.onComplete.add(function () {
            this.playTween2.start();
        }, this);
        this.playTween2 = this.boardObj.getGame().add.tween(this.checkerSprite.scale).to({
            x: [1, 1],
            y: [1, 1]
        }, 200, Phaser.Easing.Linear.None, false, 0);
        this.playTween2.onComplete.add(function () {
            if (this.pulsing == true) {
                this.playTween1.start();
            }
        }, this);
        this.playTween1.start();
    },

    stopPulseChecker: function stopPulseChecker() {
        this.pulsing = false;
    },

    /*
     * Destroy sprite.
     */
    destroySprite: function destroySprite() {
        this.boardObj.removeOneCheckerFromBoard(this.row, this.col);
    },

    /* ---------- GETTERs and SETTERs ---------- */
    getCheckerSprite: function getCheckerSprite() {
        return this.checkerSprite;
    },

    setImage: function setImage(image) {
        this.imageKey = image;
        this.create();
    },

    getRow: function getRow() {
        return this.row;
    },

    getCol: function getCol() {
        return this.col;
    },

    getImage: function getImage() {
        return this.imageKey;
    },

    getChecked: function getChecked() {
        return this.checked;
    },

    setChecked: function setChecked(ch) {
        this.checked = ch;
    },

    setPosition: function setPosition(x, y) {
        this.xPos = x;
        this.yPos = y;
        this.create();
    },

    setRow: function setRow(desiredRow) {
        this.row = desiredRow;
    },

    setCol: function setCol(desiredCol) {
        this.col = desiredCol;
    },

    setBlocked: function setBlocked(blocked) {
        this.blocked = blocked;
    },

    getBlocked: function getBlocked() {
        return this.blocked;
    },

    getPlayer: function getPlayer() {
        return this.player;
    },

    getToBeRemoved: function getToBeRemoved() {
        return this.toBeRemovedList;
    },

    setToBeRemoved: function setToBeRemoved(remove) {
        this.toBeRemovedList.push(remove);
    },

    clearToBeRemoved: function setToBeRemoved() {
        this.toBeRemovedList = [];
    },

    getPossibleMovesGraph: function getPossibleMovesGraph() {
        return this.possibleMovesGraph;
    },

    getPossibleMovesGraphKing: function getPossibleMovesGraphKing() {
        return this.possibleMovesGraphKing;
    },

    getPossibleSimpleMovesGraph: function getPossibleSimpleMovesGraph() {
        return this.possibleSimpleMovesGraph;
    },

    /*
     * Get who Am I checker neither king.
     */
    whatAmI: function whatAmI() {
        if (this.king == true) {
            return "king";
        } else {
            return "checker";
        }
    },

    doContains: function doContains(node) {
        // for (var i = 0; i < this.possibleMovesGraphKing.length; i++) {
        //     if (node.key == this.possibleMovesGraphKing[i].key && node.parent == this.possibleMovesGraphKing[i].parent) {
        //         console.log("false");
        //         return false;
        //     }
        //     if (node.key == this.possibleMovesGraphKing[i].parent && node.parent == this.possibleMovesGraphKing[i].key) {
        //         console.log("false");
        //         return false;
        //     }
        // }
        return true;
    }

};