'use strict';

var ScreenExit = function ScreenExit(game, world, menuState) {
    this.screenGroup = null;
    this.screenName = 'ScreenExit';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();
};

ScreenExit.prototype = {

    create: function create() {
        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;
        // ------------------------------------------------

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize((this.world.height / 2), this.world.height, 50);
        } else {
            var positions = global.calculateDialogSize((this.world.height / 2) + 100, this.world.height - ((this.world.height / 2) / 15), 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);

        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
        }

        //back button
        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 185, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 272, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        }


        //Instructions title
        // var exitTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 8, global.managers.language.get("Prompt"), textStyle);
        // exitTitle.maxWidth = 400;
        // exitTitle.anchor.set(0.5);
        // this.settingsTitle = exitTitle;
        // this.screenGroup.addChild(exitTitle);



        if (global.resolution == "480x800") {
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1], 32, 3);
        } else {
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1] + 50, 48, 3);
        }

        //exit question
        var exitQuestion = this.gameObj.add.text(this.world.width / 2, buttonsPositions[0], global.managers.language.get("Prompt"), textStyle);
        exitQuestion.maxWidth = 400;
        exitQuestion.anchor.set(0.5);
        this.exitQuestion = exitQuestion;

        this.yesButton = this.gameObj.add.button(this.world.width / 2, buttonsPositions[1] - 10, 'bt_instruction_on', this.yesBtn, this);
        this.yesButton.anchor.set(0.5);
        var textYes = this.gameObj.add.text(this.world.width / 2, buttonsPositions[1] - 8, global.managers.language.get("Yes"), textStyle);
        textYes.anchor.set(0.5, 0.5);
        this.textYes = textYes;

        this.noButton = this.gameObj.add.button(this.world.width / 2, buttonsPositions[2], 'bt_instruction_off', this.noBtn, this);
        this.noButton.anchor.set(0.5);
        var textNo = this.gameObj.add.text(this.world.width / 2, buttonsPositions[2] + 2, global.managers.language.get("No"), textStyle);
        textNo.anchor.set(0.5, 0.5);
        this.textNo = textNo;

        this.screenGroup.addChild(this.yesButton);
        this.screenGroup.addChild(textYes);
        this.screenGroup.addChild(this.noButton);
        this.screenGroup.addChild(textNo);
        this.screenGroup.addChild(backBtn);
        this.screenGroup.addChild(exitQuestion);
        // this.screenGroup.addChild(exitTitle);

    },

    getButtons: function getButtons() {
        return [this.yesButton, this.noButton];
    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    yesBtn: function yesBtn() {
        global.managers.audio.playSound("click");
        if (global.version == 1) {
            // Cocoon.App.exit();
            navigator.app.exitApp();
        }
        this.backBtnClicked();
    },

    noBtn: function noBtn() {
        global.managers.audio.playSound("click");
        this.backBtnClicked();
    },

    backBtnClicked: function backBtnClicked() {
            global.managers.audio.playSound("click");
            var _this = this;

            this.hideScreen();

            window.setTimeout(function() {
                _this.menuState.showMenuButtons();
            }, 100);

    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function() {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }
                global.screen = -1;
            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function() {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
            }, this);
        }
        this.updateTexts();
    },

    updateTexts: function updateTexts() {
        this.exitQuestion.setText(global.managers.language.get("Prompt"));
        this.textYes.setText(global.managers.language.get("Yes"));
        this.textNo.setText(global.managers.language.get("No"));
    }
};