"use strict";

var ScreenInstructions = function ScreenInstructions(game, world, menuState) {
    this.stonesTexts = ["skin1", "skin2"];
    this.boardTexts = ["skin1", "skin2"];
    this.stonesTextsCounter = global.settings.stonesSkin;
    this.boardTextsCounter = global.settings.boardSkin;

    this.screenGroup = null;
    this.screenName = 'ScreenRules';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();
};

ScreenInstructions.prototype = {

    create: function create() {
        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;


        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize(60, this.world.height - 20, 50);
        } else {
            var positions = global.calculateDialogSize(100, this.world.height - 50, 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);


        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
            var add = 10;
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
            var add = 15;
        }

        //back button
        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 185, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 272, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        }


        //Instructions title
        var istructionsTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 8, global.managers.language.get("Instructions"), textStyle);
        istructionsTitle.maxWidth = 400;
        istructionsTitle.anchor.set(0.5);
        this.settingsTitle = istructionsTitle;
        this.screenGroup.addChild(istructionsTitle);
        this.istructionsTitle = istructionsTitle;

        var textTitle = this.gameObj.add.text(this.world.width / 2, positions[1] + 15, global.managers.language.get("Title"), textStyle);
        textTitle.anchor.set(0.5);
        textTitle.maxWidth = 400;
        this.textTitle = textTitle;
        this.screenGroup.addChild(textTitle);

        // ----- first line -----
        var line1 = this.gameObj.add.button((this.world.width / 2) - 180, positions[2] + add, 'white');
        line1.anchor.set(0);
        line1.scale.x = 360;
        line1.scale.y = 1;
        this.screenGroup.addChild(line1);
        this.line1 = line1;

        // var textContent = this.gameObj.add.text(this.world.width / 2,  positions[1] + 15, global.managers.language.get("InstructionsContent"), textStyle);

        if (global.resolution == "480x800") {
            var style = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: 'center',
                wordWrap: true,
                wordWrapWidth: 350
            };
            this.instructContent = this.gameObj.add.text((this.world.width / 2), positions[3], global.managers.language.get("InstructionsContent"), style);
            this.instructContent.anchor.set(0.5, 0);
            this.screenGroup.addChild(this.instructContent);
        } else {
            var style = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: 'center',
                wordWrap: true,
                wordWrapWidth: 450
            };
            this.instructContent = this.gameObj.add.text((this.world.width / 2), positions[3], global.managers.language.get("InstructionsContent"), style);
            this.instructContent.anchor.set(0.5, 0);
            this.screenGroup.addChild(this.instructContent);
        }

    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    backBtnClicked: function backBtnClicked() {
        global.managers.audio.playSound("click");
        var _this = this;

        this.hideScreen();
        window.setTimeout(function () {
            _this.menuState.showMenuButtons();
        }, 100);
    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }

            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
            }, this);
        }
        this.updateTexts();
    },

    updateTexts: function updateTexts() {
        this.istructionsTitle.setText(global.managers.language.get("Instructions"));
        this.istructionsTitle.setText(global.managers.language.get("Continue"));
        this.instructContent.setText(global.managers.language.get("InstructionsContent"));

        if (global.resolution == "700x1440") {
            if (global.settings.language == 3) {
                this.instructContent.fontSize = 28;
            } else {
                this.instructContent.fontSize = 30;
            }
            console.log(this.instructContent.fontSize);
        }
    }
};