'use strict';

var ScreenNewGame = function ScreenNewGame(game, world, menuState) {
    this.screenGroup = null;
    this.screenName = 'ScreenNewGame';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();
};

ScreenNewGame.prototype = {

    create: function create() {
        var _this2 = this;

        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;
        // ------------------------------------------------

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize((this.world.height / 2), this.world.height, 50);
        } else {
            var positions = global.calculateDialogSize((this.world.height / 2), this.world.height - ((this.world.height / 2) / 15), 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);


        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
        }

        if (global.resolution == "480x800") {
            var newTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 6, global.managers.language.get("Start_new"), textStyle);
        } else {
            var newTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 4, global.managers.language.get("Start_new"), textStyle);
        }
        newTitle.maxWidth = 400;
        newTitle.anchor.set(0.5);
        this.newTitle = newTitle;
        this.screenGroup.addChild(newTitle);

        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 187, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 270, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.backBtn = backBtn;
        }

        if (global.resolution == "480x800") {
            var buttonsPositions = global.calculatePositions(positions[0] + 20, positions[positions.length - 1] + 20, 32, 4);
        } else {
            var buttonsPositions = global.calculatePositions(positions[0] + 18, positions[positions.length - 1] + 30, 48, 4);
        }


        var hardButton = this.gameObj.add.button(this.world.width / 2, buttonsPositions[0], 'bt_instruction_off', this.hardBtnClicked, this);
        hardButton.anchor.set(0.5);
        var textHard = this.gameObj.add.text(this.world.width / 2, buttonsPositions[0] + 2, global.managers.language.get("Diff_3"), textStyle);
        textHard.anchor.set(0.5, 0.5);
        this.textHard = textHard;

        var mediumBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[1], 'bt_instruction_off', this.mediumBtnClicked, this);
        mediumBtn.anchor.set(0.5);
        var textMedium = this.gameObj.add.text(this.world.width / 2, buttonsPositions[1] + 2, global.managers.language.get("Diff_2"), textStyle);
        textMedium.anchor.set(0.5, 0.5);
        this.textMedium = textMedium;

        var easyBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[2], 'bt_instruction_off', this.easyBtnClicked, this);
        easyBtn.anchor.set(0.5);
        var textEasy = this.gameObj.add.text(this.world.width / 2, buttonsPositions[2] + 2, global.managers.language.get("Diff_1"), textStyle);
        textEasy.anchor.set(0.5, 0.5);
        this.textEasy = textEasy;

        var playersBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[3], 'bt_instruction_off', this.playersBtnClicked, this);
        var playersIcon = this.gameObj.add.button(this.world.width / 2, buttonsPositions[3], 'players_icon', this.playersBtnClicked, this);
        playersBtn.anchor.set(0.5);
        playersIcon.anchor.set(0.5);


        this.screenGroup.addChild(hardButton);
        this.screenGroup.addChild(textHard);
        this.screenGroup.addChild(mediumBtn);
        this.screenGroup.addChild(textMedium);
        this.screenGroup.addChild(easyBtn);
        this.screenGroup.addChild(textEasy);
        this.screenGroup.addChild(playersBtn);
        this.screenGroup.addChild(playersIcon);
        this.screenGroup.addChild(backBtn);

        this.hardButton = hardButton;
        this.mediumBtn = mediumBtn;
        this.easyBtn = easyBtn;
        this.textEasy = textEasy;
        this.playersBtn = playersBtn;
        this.backBtn = backBtn;

        if (global.storage.data.tutorial != 'done') {
            this.tutorialGroup = this.gameObj.add.group();
            this.tutorialGroup.name = 'Tutorial';

            this.tutorial = new Tutorial(this.world, this.gameObj);
            this.tutorial.showOverlay();


            global.createTimer(global.times.TUTORIAL_HAND_TIME - 310, function () {
                _this2.tutorialGroup.addChild(_this2.easyBtn);
                _this2.tutorialGroup.addChild(_this2.textEasy);
                _this2.gameObj.world.bringToTop(_this2.tutorialGroup);
                _this2.tutorialGroup.addChild(_this2.tutorial.getHand());
                _this2.gameObj.world.bringToTop(_this2.tutorial.getHand());
            });


            this.tutorial.createHand(this.world.width + 100, this.world.height + 100);
            if (global.resolution == "480x800") {
                this.tutorial.moveHandTo(this.world.width / 2 + 120, buttonsPositions[2] + 48, global.times.TUTORIAL_HAND_TIME - 300);
            } else {
                this.tutorial.moveHandTo(this.world.width / 2 + 175, buttonsPositions[2] + 70, global.times.TUTORIAL_HAND_TIME - 300);
            }

        }

        this.menuState.setNewGameButtons(this.getButtons());
    },

    getButtons: function getButtons() {
        return [this.hardButton, this.mediumBtn, this.easyBtn, this.playersBtn];
    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    hardBtnClicked: function hardBtnClicked() {
        global.screen = -10;
        global.managers.audio.playSound("click");
        global.rules.aiLevel = 1;

        this.gameObj.camera.fade(0x000000, 200, false);
        this.gameObj.time.events.add(200, function () {
            this.gameObj.state.start('Game');
        }, this);
    },

    mediumBtnClicked: function mediumBtnClicked() {
        global.screen = -10;
     
        global.managers.audio.playSound("click");
        global.rules.aiLevel = 2;

        this.gameObj.camera.fade(0x000000, 200, false);
        this.gameObj.time.events.add(200, function () {
            this.gameObj.state.start('Game');
        }, this);
    },

    easyBtnClicked: function easyBtnClicked() {
        global.screen = -10;
      
        global.managers.audio.playSound("click");
        global.rules.aiLevel = 3;

        this.gameObj.camera.fade(0x000000, 200, false);
        this.gameObj.time.events.add(200, function () {
            this.gameObj.state.start('Game');
        }, this);
    },

    playersBtnClicked: function playersBtnClicked() {
        global.screen = -10;
     
        global.managers.audio.playSound("click");
        global.rules.aiLevel = 0;

        this.gameObj.camera.fade(0x000000, 200, false);
        this.gameObj.time.events.add(200, function () {
            this.gameObj.state.start('Game');
        }, this);
    },

    backBtnClicked: function backBtnClicked() {
    
        global.managers.audio.playSound("click");
        var _this = this;

        this.hideScreen();
        window.setTimeout(function () {
            _this.menuState.showMenuButtons();
        }, 100);
    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }

            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
            }, this);
        }

        this.updateTexts();
    },

    updateTexts: function updateTexts() {
        this.newTitle.setText(global.managers.language.get("Start_new"));

        this.textHard.setText(global.managers.language.get("Diff_3"));
        this.textMedium.setText(global.managers.language.get("Diff_2"));
        this.textEasy.setText(global.managers.language.get("Diff_1"));
    }
};