'use strict';

var ScreenResult = function ScreenResult(game, world, menuState, win, state) {
    var winner = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : null;

    this.screenGroup = null;
    this.screenName = 'ScreenResult';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.gameState = state;
    this.winner = winner;
    this.win = win;

    this.create();
};

ScreenResult.prototype = {

    create: function create() {
        global.setPauseState(true);

        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;
        // ------------------------------------------------

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize(this.world.height / 2 - 250, this.world.height / 2 + 250, 50);

            var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
            newGameBackgroundTop.frame = 0;
            newGameBackgroundTop.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackgroundTop);

            for (var i = 1; i < positions.length - 1; i++) {
                var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
                newGameBackground.frame = 1;
                newGameBackground.anchor.set(0.5);
                this.screenGroup.addChild(newGameBackground);
            }

            var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
            newGameBackgroundBottom.frame = 2;
            newGameBackgroundBottom.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackgroundBottom);

            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };

            //Congratulations title
            // var titleText = "Sorry";
            var resImg = "LOSE";
            if (this.win == null) {
                // titleText = "Sorry";
                resImg = "DRAW";
            } else if (this.win) {
                // titleText = "Congratulations";
                resImg = "WIN";
            }

            if (this.winner != null) {
                var resultTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 8, this.winner, textStyle);
                resultTitle.maxWidth = 400;
                resultTitle.anchor.set(0.5);
                this.resultTitle = resultTitle;
                this.screenGroup.addChild(resultTitle);
            }

            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1], 32, 5);

            var resultImage = this.gameObj.add.sprite(this.world.width / 2, buttonsPositions[1], resImg);
            resultImage.anchor.set(0.5);

            var newBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[3], 'PLAY_new', this.newBtnClick, this);
            newBtn.anchor.set(0.5);
            var textNew = this.gameObj.add.text(this.world.width / 2, buttonsPositions[3] + 2, global.managers.language.get("Start_new_question"), textStyle);
            textNew.anchor.set(0.5, 0.5);
            this.textNew = textNew;

            var menuBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[4], 'bt_instruction_off', this.menuBtnClick, this);
            menuBtn.anchor.set(0.5);
            var textMenu = this.gameObj.add.text(this.world.width / 2, buttonsPositions[4] + 2, global.managers.language.get("Back_2"), textStyle);
            textMenu.anchor.set(0.5, 0.5);
            this.textMenu = textMenu;

            this.screenGroup.addChild(newBtn);
            this.screenGroup.addChild(menuBtn);
            this.screenGroup.addChild(textNew);
            this.screenGroup.addChild(textMenu);
            this.screenGroup.addChild(resultImage);

            this.newBtn = newBtn;
            this.menuBtn = menuBtn;

            this.gameState.setResultGameButtons(this.getButtons());
        } else {
            var positions = global.calculateDialogSize(this.world.height / 2 - 300, this.world.height / 2 + 300, 69);
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1] + 50, 48, 5);

            var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
            newGameBackgroundTop.frame = 0;
            newGameBackgroundTop.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackgroundTop);

            for (var i = 1; i < positions.length - 1; i++) {
                var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
                newGameBackground.frame = 1;
                newGameBackground.anchor.set(0.5);
                this.screenGroup.addChild(newGameBackground);
            }

            var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
            newGameBackgroundBottom.frame = 2;
            newGameBackgroundBottom.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackgroundBottom);

            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };

            if (this.winner != null) {
                var resultTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 8, this.winner, textStyle);
                resultTitle.maxWidth = 400;
                resultTitle.anchor.set(0.5);
                this.resultTitle = resultTitle;
                this.screenGroup.addChild(resultTitle);
            }

            //Congratulations title
            // var titleText = "Sorry";
            var resImg = "result_lose";
            var lineColor = "result_red";
            if (this.win == null) {
                // titleText = "Sorry";
                lineColor = "result_blue";
                resImg = "result_draw";
            } else if (this.win) {
                // titleText = "Congratulations";
                lineColor = "result_green";
                resImg = "result_win";
            }

            var background = this.gameObj.add.sprite(this.world.width / 2, buttonsPositions[1], 'result_bg');
            background.anchor.set(0.5);
            this.screenGroup.addChild(background);

            var line = this.gameObj.add.sprite(this.world.width / 2, background.position.y + 40, lineColor);
            line.anchor.set(0.5);
            this.screenGroup.addChild(line);

            var text = this.gameObj.add.sprite(this.world.width / 2, buttonsPositions[1], resImg);
            text.anchor.set(0.5);
            // text.alpha = 0;
            this.screenGroup.addChild(text);
            // this.gameObj.add.tween(text).to( { alpha: 1 }, 1000, Phaser.Easing.Linear.None, true, 0);
            var shake = new Phaser.Plugin.Shake(this.gameObj);
            this.gameObj.plugins.add(shake);

            //--------CHECKERS SPRITES-------


            var stone1 = this.gameObj.add.sprite(this.world.width / 2 - 70, background.position.y + 80, global.getStonesSprites());
            stone1.anchor.set(0.5);
            stone1.frame = 0;
            stone1.scale.x = 0;
            stone1.scale.y = 0;
            this.screenGroup.addChild(stone1);

            var stone4 = this.gameObj.add.sprite(this.world.width / 2 + 70, background.position.y + 80, global.getStonesSprites());
            stone4.frame = 1;
            stone4.scale.x = 0;
            stone4.scale.y = 0;stone4.anchor.set(0.5);
            this.screenGroup.addChild(stone4);

            var stone2 = this.gameObj.add.sprite(this.world.width / 2 - 30, background.position.y + 80, global.getStonesSprites());
            stone2.frame = 5;
            stone2.scale.x = 0;
            stone2.scale.y = 0;
            stone2.anchor.set(0.5);
            this.screenGroup.addChild(stone2);

            var stone3 = this.gameObj.add.sprite(this.world.width / 2 + 30, background.position.y + 80, global.getStonesSprites());
            stone3.frame = 4;
            stone3.anchor.set(0.5);
            stone3.scale.x = 0;
            stone3.scale.y = 0;
            this.screenGroup.addChild(stone3);

            var this1 = this;
            window.setTimeout(function () {
                this1.gameObj.add.tween(stone1.scale).to({ x: 1, y: 1 }, 1000, Phaser.Easing.Elastic.Out, true);
                this1.gameObj.add.tween(stone4.scale).to({ x: 1, y: 1 }, 1000, Phaser.Easing.Elastic.Out, true);
            }, 70);

            window.setTimeout(function () {
                this1.gameObj.add.tween(stone2.scale).to({ x: 1, y: 1 }, 1000, Phaser.Easing.Elastic.Out, true);
                var t = this1.gameObj.add.tween(stone3.scale).to({ x: 1, y: 1 }, 1000, Phaser.Easing.Elastic.Out, true);
                t.onComplete.add(function () {
                    menuBtn.visible = true;
                    newBtn.visible = true;
                    textNew.visible = true;
                    textMenu.visible = true;
                    shake.shake(); // default displacement
                    shake.shake(70); // 40 pixels max displacement
                    shake.shake(10, text);
                    this1.gameObj.add.tween(menuBtn).to({ alpha: 1 }, 1000, Phaser.Easing.Linear.None, true, 0);
                    this1.gameObj.add.tween(newBtn).to({ alpha: 1 }, 1000, Phaser.Easing.Linear.None, true, 0);
                    this1.gameObj.add.tween(textNew).to({ alpha: 1 }, 1000, Phaser.Easing.Linear.None, true, 0);
                    this1.gameObj.add.tween(textMenu).to({ alpha: 1 }, 1000, Phaser.Easing.Linear.None, true, 0);
                });
            }, 570);

            //---------------------------------------------------------------------------
            var newBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[3], 'PLAY_new', this.newBtnClick, this);
            newBtn.anchor.set(0.5);
            newBtn.visible = false;
            newBtn.alpha = 0;
            var textNew = this.gameObj.add.text(this.world.width / 2, buttonsPositions[3] + 2, global.managers.language.get("Start_new_question"), textStyle);
            textNew.alpha = 0;
            textNew.visible = false;
            textNew.anchor.set(0.5, 0.5);
            this.textNew = textNew;

            var menuBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[4], 'bt_instruction_off', this.menuBtnClick, this);
            menuBtn.visible = false;
            menuBtn.alpha = 0;
            menuBtn.anchor.set(0.5);
            var textMenu = this.gameObj.add.text(this.world.width / 2, buttonsPositions[4] + 2, global.managers.language.get("Back_2"), textStyle);
            textMenu.visible = false;
            textMenu.alpha = 0;
            textMenu.anchor.set(0.5, 0.5);
            this.textMenu = textMenu;

            this.screenGroup.addChild(newBtn);
            this.screenGroup.addChild(menuBtn);
            this.screenGroup.addChild(textNew);
            this.screenGroup.addChild(textMenu);

            this.newBtn = newBtn;
            this.menuBtn = menuBtn;

            // this.gameState.setResultGameButtons(this.getButtons());
        }
    },

    getButtons: function getButtons() {
        return [this.newBtn, this.menuBtn];
    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    newBtnClick: function newBtnClick() {

        this.hideScreen();
        var _this = this;
        // this.menuState.returnFromDialogScreen();
        window.setTimeout(function () {

            global.storage.data.board = null;
            global.storage.data.lastGameAi = null;
            global.storage.data.lastPlayer = null;

            global.storage.data.historyMoves = null;
            global.storage.data.removedCheckers = null;
            global.storage.data.historyBoardOfValues = null;

            _this.gameObj.state.restart();
        }, 100);
    },

    menuBtnClick: function menuBtnClick() {

        global.managers.audio.playSound("click");
        global.storage.data.board = null;
        global.storage.data.lastGameAi = null;
        global.storage.data.lastPlayer = null;

        global.storage.data.historyMoves = null;
        global.storage.data.removedCheckers = null;
        global.storage.data.historyBoardOfValues = null;
        global.managers.storage.save();

        this.gameObj.camera.fade(0x000000, 200, false);
        this.gameObj.time.events.add(200, function () {
            this.gameObj.state.start('Menu');
        }, this);
    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        if (this.win == null) {
            global.managers.audio.playSound("result_neutral");
        } else if (this.win) {
            global.managers.audio.playSound("result_win");
        } else {
            global.managers.audio.playSound("result_lose");
        }

        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }
            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {

        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
            }, this);
        }

        this.updateTexts();
    },

    updateTexts: function updateTexts() {

        global.storage.data.board = null;
        global.storage.data.lastGameAi = null;
        global.storage.data.lastPlayer = null;

        global.storage.data.historyMoves = null;
        global.storage.data.removedCheckers = null;
        global.storage.data.historyBoardOfValues = null;

        global.managers.storage.save();

        if (global.resolution == "480x800") {
            this.textNew.fontSize = 20;
            this.textMenu.fontSize = 20;
            var limit = 175;
        } else {
            this.textNew.fontSize = 30;
            this.textMenu.fontSize = 30;
            var limit = 240;
        }

        this.textNew.setText(global.managers.language.get("Start_new_question"));
        while (this.textNew.width > limit) {
            this.textNew.fontSize--;
        }this.textMenu.setText(global.managers.language.get("Back_2"));
        while (this.textMenu.width > limit) {
            this.textMenu.fontSize--;
        }
    }
};