"use strict";

var ScreenRules = function ScreenRules(game, world, menuState) {
    this.captureTexts = [global.managers.language.get("Rule_2_opt_1"), global.managers.language.get("Rule_2_opt_2"), global.managers.language.get("Rule_2_opt_3")];
    this.kingsTexts = [global.managers.language.get("Rule_3_opt_1"), global.managers.language.get("Rule_3_opt_2"), global.managers.language.get("Rule_3_opt_3")];
    this.captureTextCounter = global.rules.capture;
    this.kingsTextCounter = global.rules.king;

    this.shown = false;
    this.screenGroup = null;
    this.screenName = 'ScreenRules';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();

};

ScreenRules.prototype = {

    create: function create() {
        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize(60, this.world.height - 20, 50);
        } else {
            var positions = global.calculateDialogSize(250, this.world.height - 250, 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);

        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
            var textStyleSmall = {
                font: "16px saira_medium",
                fill: "#eeeeee",
                align: "center"
            };
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
            var textStyleSmall = {
                font: "24px saira_medium",
                fill: "#eeeeee",
                align: "center"
            };
        }

        //back button
        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 186, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 272, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        }

        //rules title
        var rulesTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 8, global.managers.language.get("Menu_btn_1"), textStyle);
        rulesTitle.maxWidth = 400;
        rulesTitle.anchor.set(0.5);
        this.rulesTitle = rulesTitle;
        this.screenGroup.addChild(rulesTitle);

        if (global.resolution == "480x800") {
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1], 32, 8);
        } else {
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1], 48, 8);
        }


        if (global.resolution == "480x800") {
            var backwardsText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 33, buttonsPositions[0] + 5, global.managers.language.get("Rule_1"), textStyle);
            backwardsText.maxWidth = 400;
            this.backwardsText = backwardsText;

            var checkboxBackwards = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 115, buttonsPositions[0], 'on_off1', this.clickBackwards, this);
            this.checkboxBackwards = checkboxBackwards;
            if (global.rules.captureBackwards == false) {
                checkboxBackwards.loadTexture('on_off1', 0);
            } else {
                checkboxBackwards.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(backwardsText);
            this.screenGroup.addChild(checkboxBackwards);

            // ----- first line -----
            var line1 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[1] + 15, 'white');
            line1.anchor.set(0);
            line1.scale.x = 360;
            line1.scale.y = 1;
            this.screenGroup.addChild(line1);
            this.line1 = line1;

            // ----- second rule -----
            var captureTextTitle = this.gameObj.add.text(this.world.width / 2, buttonsPositions[2], global.managers.language.get("Rule_2"), textStyle);
            captureTextTitle.anchor.set(0.5);
            captureTextTitle.maxWidth = 400;
            this.captureTextTitle = captureTextTitle;
            this.screenGroup.addChild(captureTextTitle);

            //backgroung for texts
            var textBgStartCapt = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 97, buttonsPositions[3], 'language_bg', 0);
            textBgStartCapt.anchor.set(0.5);
            this.textBgStartCapt = textBgStartCapt;
            this.screenGroup.addChild(textBgStartCapt);
            for (var i = 0; i <= 6; i++) {
                var textBgCapt = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 127 + i * 30, buttonsPositions[3], 'language_bg', 1);
                textBgCapt.anchor.set(0.5);

                this.textBgCapt = textBgCapt;
                this.screenGroup.addChild(textBgCapt);
            }
            var textBgEndCapt = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[3], 'language_bg', 2);
            textBgEndCapt.anchor.set(0.5);
            this.textBgEndCapt = textBgEndCapt;
            this.screenGroup.addChild(textBgEndCapt);

            var captureText = this.gameObj.add.text(this.world.width / 2, buttonsPositions[3] + 3, this.captureTexts[this.captureTextCounter], textStyleSmall);
            captureText.maxWidth = 100;
            captureText.anchor.set(0.5);
            captureText.lineSpacing = -5;
            this.captureText = captureText;

            var captureLeft = this.gameObj.add.button(this.world.width / 2 - 165, buttonsPositions[3], 'bt_arrow_l', this.leftClickCapture, this);
            captureLeft.anchor.set(0.5);
            this.captureLeft = captureLeft;
            var captureRight = this.gameObj.add.button(this.world.width / 2 + 165, buttonsPositions[3], 'bt_arrow_r', this.rightClickCapture, this);
            captureRight.anchor.set(0.5);
            this.captureRight = captureRight;

            this.screenGroup.addChild(captureTextTitle);
            this.screenGroup.addChild(captureText);
            this.screenGroup.addChild(captureLeft);
            this.screenGroup.addChild(captureRight);

            // ----- third rule -----
            var kingsTextTitle = this.gameObj.add.text(this.world.width / 2, buttonsPositions[4], global.managers.language.get("Rule_3"), textStyle);
            kingsTextTitle.anchor.set(0.5);
            kingsTextTitle.maxWidth = 400;
            this.kingsTextTitle = kingsTextTitle;
            this.screenGroup.addChild(kingsTextTitle);

            //backgroung for texts
            var textBgStartKings = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 97, buttonsPositions[5], 'language_bg', 0);
            textBgStartKings.anchor.set(0.5);
            this.textBgStartKings = textBgStartKings;
            this.screenGroup.addChild(textBgStartKings);
            for (var i = 0; i <= 6; i++) {
                var textBgKings = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 127 + i * 30, buttonsPositions[5], 'language_bg', 1);
                textBgKings.anchor.set(0.5);
                this.textBgKings = textBgKings;
                this.screenGroup.addChild(textBgKings);
            }
            var textBgEndKings = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[5], 'language_bg', 2);
            textBgEndKings.anchor.set(0.5);
            this.textBgEndKings = textBgEndKings;
            this.screenGroup.addChild(textBgEndKings);

            var kingsText = this.gameObj.add.text(this.world.width / 2, buttonsPositions[5] + 3, this.kingsTexts[this.kingsTextCounter], textStyleSmall);
            kingsText.maxWidth = 50;
            kingsText.anchor.set(0.5);
            this.kingsText = kingsText;

            var kingsLeft = this.gameObj.add.button(this.world.width / 2 - 165, buttonsPositions[5], 'bt_arrow_l', this.leftClickKings, this);
            kingsLeft.anchor.set(0.5);
            this.kingsLeft = kingsLeft;
            var kingsRight = this.gameObj.add.button(this.world.width / 2 + 165, buttonsPositions[5], 'bt_arrow_r', this.rightClickKings, this);
            kingsRight.anchor.set(0.5);
            this.kingsRight = kingsRight;

            this.screenGroup.addChild(kingsText);
            this.screenGroup.addChild(kingsLeft);
            this.screenGroup.addChild(kingsRight);

            // ----- second line -----
            var line2 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[6] + 15, 'white');
            line2.anchor.set(0);
            line2.scale.x = 360;
            line2.scale.y = 1;
            this.screenGroup.addChild(line2);
            this.line2 = line2;

            // ----- forth rule -----
            var kingText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 33, buttonsPositions[7] + 5, global.managers.language.get("Rule_4"), textStyle);
            kingText.maxWidth = 400;
            this.kingText = kingText;

            var checkboxKing = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 115, buttonsPositions[7], 'on_off1', this.clickKing, this);
            this.checkboxKing = checkboxKing;
            if (global.rules.kingPriority == false) {
                checkboxKing.loadTexture('on_off1', 0);
            } else {
                checkboxKing.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(kingText);
            this.screenGroup.addChild(checkboxKing);
        } else {
            var backwardsText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 60, buttonsPositions[0] + 15, global.managers.language.get("Rule_1"), textStyle);
            backwardsText.maxWidth = 400;
            this.backwardsText = backwardsText;

            var checkboxBackwards = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 180, buttonsPositions[0], 'on_off1', this.clickBackwards, this);
            this.checkboxBackwards = checkboxBackwards;
            if (global.rules.captureBackwards == false) {
                checkboxBackwards.loadTexture('on_off1', 0);
            } else {
                checkboxBackwards.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(backwardsText);
            this.screenGroup.addChild(checkboxBackwards);

            // ----- first line -----
            var line1 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[1] + 30, 'white');
            line1.anchor.set(0);
            line1.scale.x = 360;
            line1.scale.y = 1;
            this.screenGroup.addChild(line1);
            this.line1 = line1;

            // ----- second rule -----
            var captureTextTitle = this.gameObj.add.text(this.world.width / 2, buttonsPositions[2], global.managers.language.get("Rule_2"), textStyle);
            captureTextTitle.anchor.set(0.5);
            captureTextTitle.maxWidth = 400;
            this.captureTextTitle = captureTextTitle;
            this.screenGroup.addChild(captureTextTitle);

            //backgroung for texts
            var textBgStartCapt = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 162, buttonsPositions[3], 'language_bg', 0);
            textBgStartCapt.anchor.set(0.5);
            this.textBgStartCapt = textBgStartCapt;
            this.screenGroup.addChild(textBgStartCapt);
            for (var i = 0; i <= 5; i++) {
                var textBgCapt = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 204 + i * 42, buttonsPositions[3], 'language_bg', 1);
                textBgCapt.anchor.set(0.5);

                this.textBgCapt = textBgCapt;
                this.screenGroup.addChild(textBgCapt);
            }
            var textBgEndCapt = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 198 + 6 * 43, buttonsPositions[3], 'language_bg', 2);
            textBgEndCapt.anchor.set(0.5);
            this.textBgEndCapt = textBgEndCapt;
            this.screenGroup.addChild(textBgEndCapt);

            var captureText = this.gameObj.add.text(this.world.width / 2, buttonsPositions[3], this.captureTexts[this.captureTextCounter], textStyleSmall);
            captureText.maxWidth = 100;
            captureText.anchor.set(0.5);
            captureText.lineSpacing = -5;
            this.captureText = captureText;

            var captureLeft = this.gameObj.add.button(this.world.width / 2 - 225, buttonsPositions[3], 'bt_arrow_l', this.leftClickCapture, this);
            captureLeft.anchor.set(0.5);
            this.captureLeft = captureLeft;
            var captureRight = this.gameObj.add.button(this.world.width / 2 + 225, buttonsPositions[3], 'bt_arrow_r', this.rightClickCapture, this);
            captureRight.anchor.set(0.5);
            this.captureRight = captureRight;

            this.screenGroup.addChild(captureTextTitle);
            this.screenGroup.addChild(captureText);
            this.screenGroup.addChild(captureLeft);
            this.screenGroup.addChild(captureRight);

            // ----- third rule -----
            var kingsTextTitle = this.gameObj.add.text(this.world.width / 2, buttonsPositions[4], global.managers.language.get("Rule_3"), textStyle);
            kingsTextTitle.anchor.set(0.5);
            kingsTextTitle.maxWidth = 400;
            this.kingsTextTitle = kingsTextTitle;
            this.screenGroup.addChild(kingsTextTitle);

            //backgroung for texts
            var textBgStartKings = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 162, buttonsPositions[5], 'language_bg', 0);
            textBgStartKings.anchor.set(0.5);
            this.textBgStartKings = textBgStartKings;
            this.screenGroup.addChild(textBgStartKings);
            for (var i = 0; i <= 5; i++) {
                var textBgKings = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 204 + i * 42, buttonsPositions[5], 'language_bg', 1);
                textBgKings.anchor.set(0.5);
                this.textBgKings = textBgKings;
                this.screenGroup.addChild(textBgKings);
            }
            var textBgEndKings = this.gameObj.add.sprite(this.world.width / 2 - newGameBackgroundTop.width / 2 + 198 + 6 * 43, buttonsPositions[5], 'language_bg', 2);
            textBgEndKings.anchor.set(0.5);
            this.textBgEndKings = textBgEndKings;
            this.screenGroup.addChild(textBgEndKings);

            var kingsText = this.gameObj.add.text(this.world.width / 2, buttonsPositions[5], this.kingsTexts[this.kingsTextCounter], textStyleSmall);
            kingsText.maxWidth = 50;
            kingsText.anchor.set(0.5);
            this.kingsText = kingsText;

            var kingsLeft = this.gameObj.add.button(this.world.width / 2 - 225, buttonsPositions[5], 'bt_arrow_l', this.leftClickKings, this);
            kingsLeft.anchor.set(0.5);
            this.kingsLeft = kingsLeft;
            var kingsRight = this.gameObj.add.button(this.world.width / 2 + 225, buttonsPositions[5], 'bt_arrow_r', this.rightClickKings, this);
            kingsRight.anchor.set(0.5);
            this.kingsRight = kingsRight;

            this.screenGroup.addChild(kingsText);
            this.screenGroup.addChild(kingsLeft);
            this.screenGroup.addChild(kingsRight);

            // ----- second line -----
            var line2 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[6] + 30, 'white');
            line2.anchor.set(0);
            line2.scale.x = 360;
            line2.scale.y = 1;
            this.screenGroup.addChild(line2);
            this.line2 = line2;

            // ----- forth rule -----
            var kingText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 60, buttonsPositions[7] + 15, global.managers.language.get("Rule_4"), textStyle);
            kingText.maxWidth = 400;
            this.kingText = kingText;

            var checkboxKing = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 180, buttonsPositions[7], 'on_off1', this.clickKing, this);
            this.checkboxKing = checkboxKing;
            if (global.rules.kingPriority == false) {
                checkboxKing.loadTexture('on_off1', 0);
            } else {
                checkboxKing.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(kingText);
            this.screenGroup.addChild(checkboxKing);
        }
    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    clickBackwards: function clickBackwards() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (global.rules.captureBackwards == false) {
                global.setRules1(true);
                this.checkboxBackwards.loadTexture('on_off2', 0);
            } else {
                global.setRules1(false);
                this.checkboxBackwards.loadTexture('on_off1', 0);
            }
        }
    },

    clickKing: function clickKing() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (global.rules.kingPriority == false) {
                global.setRules4(true);
                this.checkboxKing.loadTexture('on_off2', 0);
            } else {
                global.setRules4(false);
                this.checkboxKing.loadTexture('on_off1', 0);
            }
        }
    },

    rightClickCapture: function rightClickCapture() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.captureTextCounter < this.captureTexts.length - 1) {
                this.captureTextCounter++;
                this.captureText.setText(this.captureTexts[this.captureTextCounter]);
            } else {
                this.captureTextCounter = 0
                this.captureText.setText(this.captureTexts[this.captureTextCounter]);
            }
            global.setRules2(this.captureTextCounter);
            this.updateTexts();
            console.log(this.captureTextCounter)
        }
    },

    leftClickCapture: function leftClickCapture() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.captureTextCounter > 0) {
                this.captureTextCounter--;
                this.captureText.setText(this.captureTexts[this.captureTextCounter]);
            } else {
                this.captureTextCounter = this.captureTexts.length - 1;
                this.captureText.setText(this.captureTexts[this.captureTextCounter]);
            }
            global.setRules2(this.captureTextCounter);
            this.updateTexts();
            console.log(this.captureTextCounter)
        }
    },

    rightClickKings: function rightClickKings() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.kingsTextCounter < this.kingsTexts.length - 1) {
                this.kingsTextCounter++;
                this.kingsText.setText(this.kingsTexts[this.kingsTextCounter]);
            } else {
                this.kingsTextCounter = 0
                this.kingsText.setText(this.kingsTexts[this.kingsTextCounter]);
            }
            global.setRules3(this.kingsTextCounter);
            this.updateTexts();
            console.log(this.kingsTextCounter)
        }
    },

    leftClickKings: function leftClickKings() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.kingsTextCounter > 0) {
                this.kingsTextCounter--;
                this.kingsText.setText(this.kingsTexts[this.kingsTextCounter]);
            } else {
                this.kingsTextCounter = this.kingsTexts.length - 1;
                this.kingsText.setText(this.kingsTexts[this.kingsTextCounter]);
            }
            global.setRules3(this.kingsTextCounter);
            this.updateTexts();
            console.log(this.kingsTextCounter)
        }
    },

    backBtnClicked: function backBtnClicked() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            var _this = this;

            global.storage.data.captureBackwards = global.rules.captureBackwards;
            global.storage.data.capture = global.rules.capture;
            global.storage.data.king = global.rules.king;
            global.storage.data.kingPriority = global.rules.kingPriority;
            global.managers.storage.save();

            this.hideScreen();
            window.setTimeout(function () {
                _this.menuState.showMenuButtons();
            }, 100);
        }
    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.screenGroup.visible = true;
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        var self = this;

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }
                self.shown = false;
            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        var self = this;

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
                self.shown = true;
            }, this);
        }

        this.updateTexts();
    },

    updateTexts: function updateTexts() {
        if (global.resolution == "480x800") {
            this.rulesTitle.fontSize = 20;
            this.backwardsText.fontSize = 20;
            this.captureTextTitle.fontSize = 20;
            this.captureText.fontSize = 20;
            this.kingsTextTitle.fontSize = 20;
            this.kingsText.fontSize = 20;
            this.kingText.fontSize = 20;

            var normalLimit = this.world.width / 2;
            var windowLimit = 225;
        } else {
            this.rulesTitle.fontSize = 30;
            this.backwardsText.fontSize = 30;
            this.captureTextTitle.fontSize = 30;
            this.captureText.fontSize = 30;
            this.kingsTextTitle.fontSize = 30;
            this.kingsText.fontSize = 30;
            this.kingText.fontSize = 30;

            var normalLimit = this.world.width / 2 - 60;
            var windowLimit = 270;
        }

        this.captureTexts = [global.managers.language.get("Rule_2_opt_1"), global.managers.language.get("Rule_2_opt_2"), global.managers.language.get("Rule_2_opt_3")];
        this.kingsTexts = [global.managers.language.get("Rule_3_opt_1"), global.managers.language.get("Rule_3_opt_2"), global.managers.language.get("Rule_3_opt_3")];

        this.rulesTitle.setText(global.managers.language.get("Menu_btn_1"))

        this.backwardsText.setText(global.managers.language.get("Rule_1"))
        while (this.backwardsText.width > normalLimit)
            this.backwardsText.fontSize--;

        this.captureTextTitle.setText(global.managers.language.get("Rule_2"))
        this.captureText.setText(this.captureTexts[this.captureTextCounter])
        while (this.captureText.width > windowLimit)
            this.captureText.fontSize--;

        this.kingsTextTitle.setText(global.managers.language.get("Rule_3"))
        this.kingsText.setText(this.kingsTexts[this.kingsTextCounter])
        while (this.kingsText.width > windowLimit)
            this.kingsText.fontSize--;

        this.kingText.setText(global.managers.language.get("Rule_4"))
        while (this.kingText.width > normalLimit)
            this.kingText.fontSize--;

    }
};