'use strict';

var ScreenSavedGame = function ScreenSavedGame(game, world, menuState) {
    this.screenGroup = null;
    this.screenName = 'ScreenSavedGame';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();
};

ScreenSavedGame.prototype = {

    create: function create() {
        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;
        // ------------------------------------------------

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize((this.world.height / 2), this.world.height, 50);
        } else {
            var positions = global.calculateDialogSize((this.world.height / 2) + 100, this.world.height - ((this.world.height / 2) / 15), 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);

        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
        }

        if (global.resolution == "480x800") {
            var newTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 6, global.managers.language.get("Start_new_question"), textStyle);
        } else {
            var newTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 4, global.managers.language.get("Start_new_question"), textStyle);
        }
        newTitle.maxWidth = 400;
        newTitle.anchor.set(0.5);
        this.newTitle = newTitle;
        this.screenGroup.addChild(newTitle);

        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 187, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 270, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.backBtn = backBtn;
        }

        if (global.resolution == "480x800") {
            var buttonsPositions = global.calculatePositions(positions[1], positions[positions.length - 1], 32, 2);
        } else {
            var buttonsPositions = global.calculatePositions(positions[1], positions[positions.length - 1], 48, 2);
        }

        var continueBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[0], 'bt_instruction_off', this.continueBtnClicked, this);
        continueBtn.anchor.set(0.5);
        var textContinue = this.gameObj.add.text(this.world.width / 2, buttonsPositions[0] + 2, global.managers.language.get("Continue"), textStyle);
        textContinue.anchor.set(0.5, 0.5);
        this.textContinue = textContinue;

        var newBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[1], 'bt_instruction_off', this.newBtnClicked, this);
        newBtn.anchor.set(0.5);
        var textNew = this.gameObj.add.text(this.world.width / 2, buttonsPositions[1] + 2, global.managers.language.get("Start_new"), textStyle);
        textNew.anchor.set(0.5, 0.5);
        this.textNew = textNew;

        this.screenGroup.addChild(newBtn);
        this.screenGroup.addChild(textNew);
        this.screenGroup.addChild(continueBtn);
        this.screenGroup.addChild(textContinue);
        this.screenGroup.addChild(backBtn);

        this.newBtn = newBtn;
        this.continueBtn = continueBtn;
        this.backBtn = backBtn;

    },

    getButtons: function getButtons() {
        return [this.newBtn, this.continueBtn];
    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    continueBtnClicked: function continueBtnClicked() {
        global.screen = -10;
        global.managers.audio.playSound("click");
        this.gameObj.camera.fade(0x000000, 200, false);
        this.gameObj.time.events.add(200, function () {
            this.gameObj.state.start('Game');
        }, this);
    },

    newBtnClicked: function newBtnClicked() {
        global.managers.audio.playSound("click");
        global.storage.data.board = null;
        global.storage.data.lastGameAi = null;
        global.storage.data.lastPlayer = null;

        global.storage.data.historyMoves = null;
        global.storage.data.removedCheckers = null;
        global.storage.data.historyBoardOfValues = null;

        global.screen = 10;
        if (this.newGame == null || this.newGame == undefined) {
            this.newGame = new ScreenNewGame(this.gameObj, this.world, this.menuState);
        }
        this.newGame.showScreen();
        this.screenGroup.visible = false;
    },

    backBtnClicked: function backBtnClicked() {
        global.managers.audio.playSound("click");
        var _this = this;

        this.hideScreen();
        window.setTimeout(function () {
            _this.menuState.showMenuButtons();
        }, 100);
    },

    getNewGame: function getNewGame(){
        return this.newGame;
    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }

            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
            }, this);
        }
        this.updateTexts();
    },

    updateTexts: function updateTexts() {
        if (global.resolution == "480x800") {
            this.textContinue.fontSize = 20;
            this.textNew.fontSize = 20;
            var limit = 175;
        } else {
            this.textContinue.fontSize = 30;
            this.textNew.fontSize = 30;
            var limit = 270;
        }

        this.newTitle.setText(global.managers.language.get("Start_new_question"));

        this.textContinue.setText(global.managers.language.get("Continue"));
        while (this.textContinue.width > limit)
            this.textContinue.fontSize--;

        this.textNew.setText(global.managers.language.get("Start_new"));
        while (this.textNew.width > limit)
            this.textNew.fontSize--;
    }
};