'use strict';

var ScreenSettings = function ScreenSettings(game, world, menuState) {
    global.managers.storage.load();

    this.stonesTextsCounter = global.settings.stonesSkin;
    this.boardTextsCounter = global.settings.boardSkin;
    this.languagesCounter = global.settings.language;

    this.shown = false; 
    this.screenGroup = null;
    this.screenName = 'ScreenRules';
    this.world = world;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();
};

ScreenSettings.prototype = {

    create: function create() {
        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;

        var checkers = global.getStonesSpritesToSettings();

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize(60, this.world.height - 10, 50);
        } else {
            var positions = global.calculateDialogSize(100, this.world.height - 50, 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);


        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center",
                wordWrap: true,
                wordWrapWidth: 110
            };
            var tabWidth = 120;
            this.tabWidth = tabWidth;
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center",
                wordWrap: true,
                wordWrapWidth: 148
            };
            var tabWidth = 168;
            this.tabWidth = tabWidth;
        }

        //back button
        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 186, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 272, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        }

        //settings title
        var settingsTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 8, global.managers.language.get("OptionsKey"), textStyle);
        settingsTitle.maxWidth = 400;
        settingsTitle.anchor.set(0.5);
        this.settingsTitle = settingsTitle;
        this.screenGroup.addChild(settingsTitle);

        if (global.resolution == "480x800") {
            var buttonsPositions1 = global.calculatePositions(positions[0], this.world.height / 2 + 150, 32, 4);
        } else {
            var buttonsPositions1 = global.calculatePositions(positions[0], this.world.height / 2 + 150, 48, 4);
        }

        if (global.resolution == "480x800") {
            var buttonsPositions2 = global.calculatePositions(this.world.height / 2 + 50, positions[positions.length - 1], 32, 5);
        } else {
            var buttonsPositions2 = global.calculatePositions(this.world.height / 2, positions[positions.length - 1], 48, 5);
        }

        var buttonsPositions = buttonsPositions1.concat(buttonsPositions2);

        // ----- language setting -----
        //playAsText
        if (global.resolution == "480x800") {
            //backgroung for texts
            var textBgStartLang = this.gameObj.add.sprite(this.world.width / 2 - 180, buttonsPositions[0], 'language_bg', 0);
            textBgStartLang.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartLang);
            for (var i = 0; i <= 2; i++) {
                var textBgLang = this.gameObj.add.sprite(this.world.width / 2 - 150 + i * 30, buttonsPositions[0], 'language_bg', 1);
                textBgLang.anchor.set(0.5);
                this.screenGroup.addChild(textBgLang);
            }
            var textBgEndLang = this.gameObj.add.sprite(this.world.width / 2 - 60, buttonsPositions[0], 'language_bg', 2);
            textBgEndLang.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndLang);

            var language = this.gameObj.add.text(this.world.width / 2 - (tabWidth), buttonsPositions[0], global.managers.language.get("Setting_3"), textStyle);
            language.maxWidth = 100;
            language.anchor.set(0.5);
            this.language = language;
            this.screenGroup.addChild(language);

            var languageLeft = this.gameObj.add.button(this.world.width / 2, buttonsPositions[0], 'bt_arrow_l', this.leftClickLanguage, this);
            languageLeft.anchor.set(0.5);
            var languageRight = this.gameObj.add.button(this.world.width / 2 + 160, buttonsPositions[0], 'bt_arrow_r', this.rightClickLanguage, this);
            languageRight.anchor.set(0.5);
            var languageBg = this.gameObj.add.sprite(this.world.width / 2 + 80, buttonsPositions[0], 'language_bg_full');
            languageBg.anchor.set(0.5);

            var languageSkin = this.gameObj.add.sprite(this.world.width / 2 + 80, buttonsPositions[0], 'language');
            languageSkin.frame = global.settings.language;
            languageSkin.anchor.set(0.5);
            this.languageSkin = languageSkin;
        } else {
            //backgroung for texts
            var textBgStartLang = this.gameObj.add.sprite(this.world.width / 2 - 260, buttonsPositions[0], 'language_bg', 0);
            textBgStartLang.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartLang);
            for (var i = 0; i <= 2; i++) {
                var textBgLang = this.gameObj.add.sprite(this.world.width / 2 - 218 + i * 42, buttonsPositions[0], 'language_bg', 1);
                textBgLang.anchor.set(0.5);
                this.screenGroup.addChild(textBgLang);
            }
            var textBgEndLang = this.gameObj.add.sprite(this.world.width / 2 - 92, buttonsPositions[0], 'language_bg', 2);
            textBgEndLang.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndLang);

            var language = this.gameObj.add.text(this.world.width / 2 - (tabWidth) - 10, buttonsPositions[0], global.managers.language.get("Setting_3"), textStyle);
            language.maxWidth = 100;
            this.language = language;
            language.anchor.set(0.5);
            this.screenGroup.addChild(language);

            var languageLeft = this.gameObj.add.button(this.world.width / 2 + 10, buttonsPositions[0], 'bt_arrow_l', this.leftClickLanguage, this);
            languageLeft.anchor.set(0.5);
            var languageRight = this.gameObj.add.button(this.world.width / 2 + 230, buttonsPositions[0], 'bt_arrow_r', this.rightClickLanguage, this);
            languageRight.anchor.set(0.5);
            var languageBg = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[0], 'language_bg_full');
            languageBg.anchor.set(0.5);

            var languageSkin = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[0], 'language');
            languageSkin.frame = global.settings.language;
            languageSkin.anchor.set(0.5);
            this.languageSkin = languageSkin;
        }

        this.screenGroup.addChild(languageLeft);
        this.screenGroup.addChild(languageRight);
        this.screenGroup.addChild(languageBg);
        this.screenGroup.addChild(languageSkin);

        // ----- first setting -----
        //backgroung for texts
        if (global.resolution == "480x800") {
            var textBgStartPlay = this.gameObj.add.sprite(this.world.width / 2 - 180, buttonsPositions[1], 'language_bg', 0);
            textBgStartPlay.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartPlay);
            for (var i = 0; i <= 2; i++) {
                var textBgPlay = this.gameObj.add.sprite(this.world.width / 2 - 150 + i * 30, buttonsPositions[1], 'language_bg', 1);
                textBgPlay.anchor.set(0.5);
                this.screenGroup.addChild(textBgPlay);
            }
            var textBgEndPlay = this.gameObj.add.sprite(this.world.width / 2 - 60, buttonsPositions[1], 'language_bg', 2);
            textBgEndPlay.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndPlay);

            //playAsText
            var playAs = this.gameObj.add.text(this.world.width / 2 - (tabWidth), buttonsPositions[1], global.managers.language.get("Setting_5"), textStyle);
            playAs.maxWidth = 100;
            this.playAs = playAs;
            playAs.anchor.set(0.5);
            this.screenGroup.addChild(playAs);

            //Pick player
            var player1Pick = this.gameObj.add.button(this.world.width / 2 + 10, buttonsPositions[1], 'bg_square_on', this.playAs1, this);
            player1Pick.anchor.set(0.5);
            this.player1Picker = player1Pick;
            this.screenGroup.addChild(player1Pick);

            var playAs = this.gameObj.add.text(this.world.width / 2 + 80, buttonsPositions[1] + 3, "VS", textStyle);
            playAs.maxWidth = 100;
            playAs.anchor.set(0.5);
            this.screenGroup.addChild(playAs);

            var player2Pick = this.gameObj.add.button(this.world.width / 2 + 150, buttonsPositions[1], 'bg_square_off', this.playAs2, this);
            player2Pick.anchor.set(0.5);
            this.player2Picker = player2Pick;
            this.screenGroup.addChild(player2Pick);


            var player1PickChecker = this.gameObj.add.button(this.world.width / 2 + 10, buttonsPositions[1], checkers[1], this.playAs1, this);
            player1PickChecker.scale.x = 0.5;
            player1PickChecker.scale.y = 0.5;
            player1PickChecker.anchor.set(0.5);
            this.player1PickChecker = player1PickChecker;
            this.screenGroup.addChild(player1PickChecker);

            var player2PickChecker = this.gameObj.add.button(this.world.width / 2 + 150, buttonsPositions[1], checkers[2], this.playAs2, this);
            player2PickChecker.scale.x = 0.5;
            player2PickChecker.scale.y = 0.5;
            player2PickChecker.anchor.set(0.5);
            this.player2PickChecker = player2PickChecker;
            this.screenGroup.addChild(player2PickChecker);

            var checkSprite = this.gameObj.add.sprite(this.world.width / 2 + 30, buttonsPositions[1] + 25, 'icons', 7);
            checkSprite.anchor.set(0.5);
            this.checkSprite = checkSprite;
            this.screenGroup.addChild(checkSprite);
        } else {
            var textBgStartPlay = this.gameObj.add.sprite(this.world.width / 2 - 260, buttonsPositions[1], 'language_bg', 0);
            textBgStartPlay.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartPlay);
            for (var i = 0; i <= 2; i++) {
                var textBgPlay = this.gameObj.add.sprite(this.world.width / 2 - 218 + i * 42, buttonsPositions[1], 'language_bg', 1);
                textBgPlay.anchor.set(0.5);
                this.screenGroup.addChild(textBgPlay);
            }
            var textBgEndPlay = this.gameObj.add.sprite(this.world.width / 2 - 92, buttonsPositions[1], 'language_bg', 2);
            textBgEndPlay.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndPlay);

            //playAsText
            var playAs = this.gameObj.add.text(this.world.width / 2 - (tabWidth) - 10, buttonsPositions[1], global.managers.language.get("Setting_5"), textStyle);
            playAs.maxWidth = 100;
            this.playAs = playAs;
            this.screenGroup.addChild(playAs);
            playAs.anchor.set(0.5);

            //Pick player
            var player1Pick = this.gameObj.add.button(this.world.width / 2 + 30, buttonsPositions[1], 'bg_square_on', this.playAs1, this);
            player1Pick.anchor.set(0.5);
            this.player1Picker = player1Pick;
            this.screenGroup.addChild(player1Pick);

            var playAs = this.gameObj.add.text(this.world.width / 2 + 120, buttonsPositions[1] + 3, "VS", textStyle);
            playAs.maxWidth = 100;
            playAs.anchor.set(0.5);
            this.screenGroup.addChild(playAs);

            var player2Pick = this.gameObj.add.button(this.world.width / 2 + 210, buttonsPositions[1], 'bg_square_off', this.playAs2, this);
            player2Pick.anchor.set(0.5);
            this.player2Picker = player2Pick;
            this.screenGroup.addChild(player2Pick);


            var player1PickChecker = this.gameObj.add.button(this.world.width / 2 + 30, buttonsPositions[1], checkers[1], this.playAs1, this);
            player1PickChecker.scale.x = 0.5;
            player1PickChecker.scale.y = 0.5;
            player1PickChecker.anchor.set(0.5);
            this.player1PickChecker = player1PickChecker;
            this.screenGroup.addChild(player1PickChecker);

            var player2PickChecker = this.gameObj.add.button(this.world.width / 2 + 210, buttonsPositions[1], checkers[2], this.playAs2, this);
            player2PickChecker.scale.x = 0.5;
            player2PickChecker.scale.y = 0.5;
            player2PickChecker.anchor.set(0.5);
            this.player2PickChecker = player2PickChecker;
            this.screenGroup.addChild(player2PickChecker);

            var checkSprite = this.gameObj.add.sprite(this.world.width / 2 + 60, buttonsPositions[1] + 35, 'icons', 7);
            checkSprite.anchor.set(0.5);
            this.checkSprite = checkSprite;
            this.screenGroup.addChild(checkSprite);
        }





        // ----- second setting -----
        if (global.resolution == "480x800") {
            var textBgStartStones = this.gameObj.add.sprite(this.world.width / 2 - 180, buttonsPositions[2], 'language_bg', 0);
            textBgStartStones.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartStones);
            for (var i = 0; i <= 2; i++) {
                var textBgStones = this.gameObj.add.sprite(this.world.width / 2 - 150 + i * 30, buttonsPositions[2], 'language_bg', 1);
                textBgStones.anchor.set(0.5);
                this.screenGroup.addChild(textBgStones);
            }
            var textBgEndStones = this.gameObj.add.sprite(this.world.width / 2 - 60, buttonsPositions[2], 'language_bg', 2);
            textBgEndStones.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndStones);

            //playAsText
            var stonesSkinText = this.gameObj.add.text(this.world.width / 2 - (tabWidth), buttonsPositions[2], global.managers.language.get("Setting_6"), textStyle);
            stonesSkinText.maxWidth = 100;
            stonesSkinText.anchor.set(0.5);
            this.stonesSkinText = stonesSkinText;
            this.screenGroup.addChild(stonesSkinText);

            var stonesLeft = this.gameObj.add.button(this.world.width / 2, buttonsPositions[2], 'bt_arrow_l', this.leftClickStones, this);
            stonesLeft.anchor.set(0.5);
            var stonesRight = this.gameObj.add.button(this.world.width / 2 + 160, buttonsPositions[2], 'bt_arrow_r', this.rightClickStones, this);
            stonesRight.anchor.set(0.5);
            var stonesBg = this.gameObj.add.sprite(this.world.width / 2 + 80, buttonsPositions[2], 'language_bg_full');
            stonesBg.anchor.set(0.5);

            var stonesSkin = this.gameObj.add.sprite(this.world.width / 2 + 80, buttonsPositions[2], checkers[0]);
            stonesSkin.anchor.set(0.5);
            this.stonesSkin = stonesSkin;
        } else {
            var textBgStartStones = this.gameObj.add.sprite(this.world.width / 2 - 260, buttonsPositions[2], 'language_bg', 0);
            textBgStartStones.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartStones);
            for (var i = 0; i <= 2; i++) {
                var textBgStones = this.gameObj.add.sprite(this.world.width / 2 - 218 + i * 42, buttonsPositions[2], 'language_bg', 1);
                textBgStones.anchor.set(0.5);
                this.screenGroup.addChild(textBgStones);
            }
            var textBgEndStones = this.gameObj.add.sprite(this.world.width / 2 - 92, buttonsPositions[2], 'language_bg', 2);
            textBgEndStones.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndStones);

            //playAsText
            var stonesSkinText = this.gameObj.add.text(this.world.width / 2 - (tabWidth) - 10, buttonsPositions[2], global.managers.language.get("Setting_6"), textStyle);
            stonesSkinText.maxWidth = 100;
            this.stonesSkinText = stonesSkinText;
            this.stonesSkinText.anchor.set(0.5);
            this.screenGroup.addChild(stonesSkinText);

            var stonesLeft = this.gameObj.add.button(this.world.width / 2 + 10, buttonsPositions[2], 'bt_arrow_l', this.leftClickStones, this);
            stonesLeft.anchor.set(0.5);
            var stonesRight = this.gameObj.add.button(this.world.width / 2 + 230, buttonsPositions[2], 'bt_arrow_r', this.rightClickStones, this);
            stonesRight.anchor.set(0.5);
            var stonesBg = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[2], 'language_bg_full');
            stonesBg.anchor.set(0.5);

            var stonesSkin = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[2], checkers[0]);
            stonesSkin.anchor.set(0.5);
            this.stonesSkin = stonesSkin;
        }

        this.screenGroup.addChild(stonesLeft);
        this.screenGroup.addChild(stonesRight);
        this.screenGroup.addChild(stonesBg);
        this.screenGroup.addChild(stonesSkin);


        // ----- third setting -----
        if (global.resolution == "480x800") {
            var textBgStartBoard = this.gameObj.add.sprite(this.world.width / 2 - 180, buttonsPositions[3], 'language_bg', 0);
            textBgStartBoard.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartBoard);
            for (var i = 0; i <= 2; i++) {
                var textBgBoard = this.gameObj.add.sprite(this.world.width / 2 - 150 + i * 30, buttonsPositions[3], 'language_bg', 1);
                textBgBoard.anchor.set(0.5);
                this.screenGroup.addChild(textBgBoard);
            }
            var textBgEndBoard = this.gameObj.add.sprite(this.world.width / 2 - 60, buttonsPositions[3], 'language_bg', 2);
            textBgEndBoard.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndBoard);

            //playAsText
            var boardSkinText = this.gameObj.add.text(this.world.width / 2 - (tabWidth), buttonsPositions[3], global.managers.language.get("Setting_7"), textStyle);
            boardSkinText.maxWidth = 100;
            boardSkinText.anchor.set(0.5);
            this.boardSkinText = boardSkinText;
            this.screenGroup.addChild(boardSkinText);

            var boardLeft = this.gameObj.add.button(this.world.width / 2, buttonsPositions[3], 'bt_arrow_l', this.leftClickBoard, this);
            boardLeft.anchor.set(0.5);
            var boardRight = this.gameObj.add.button(this.world.width / 2 + 160, buttonsPositions[3], 'bt_arrow_r', this.rightClickBoard, this);
            boardRight.anchor.set(0.5);
            var boardBg = this.gameObj.add.sprite(this.world.width / 2 + 80, buttonsPositions[3], 'language_bg_full');
            boardBg.anchor.set(0.5);

            var globalBoardSkin = global.getBoardSprite();
            var boardSkin = this.gameObj.add.sprite(this.world.width / 2 + 81, buttonsPositions[3], globalBoardSkin + "_n");
            boardSkin.anchor.set(0.5);
            this.boardSkin = boardSkin;
        } else {
            var textBgStartBoard = this.gameObj.add.sprite(this.world.width / 2 - 260, buttonsPositions[3], 'language_bg', 0);
            textBgStartBoard.anchor.set(0.5);
            this.screenGroup.addChild(textBgStartBoard);
            for (var i = 0; i <= 2; i++) {
                var textBgBoard = this.gameObj.add.sprite(this.world.width / 2 - 218 + i * 42, buttonsPositions[3], 'language_bg', 1);
                textBgBoard.anchor.set(0.5);
                this.screenGroup.addChild(textBgBoard);
            }
            var textBgEndBoard = this.gameObj.add.sprite(this.world.width / 2 - 92, buttonsPositions[3], 'language_bg', 2);
            textBgEndBoard.anchor.set(0.5);
            this.screenGroup.addChild(textBgEndBoard);

            //playAsText
            var boardSkinText = this.gameObj.add.text(this.world.width / 2 - (tabWidth) - 10, buttonsPositions[3], global.managers.language.get("Setting_7"), textStyle);
            boardSkinText.maxWidth = 100;
            this.boardSkinText = boardSkinText;
            this.screenGroup.addChild(boardSkinText);
            this.boardSkinText.anchor.set(0.5);

            var boardLeft = this.gameObj.add.button(this.world.width / 2 + 10, buttonsPositions[3], 'bt_arrow_l', this.leftClickBoard, this);
            boardLeft.anchor.set(0.5);
            var boardRight = this.gameObj.add.button(this.world.width / 2 + 230, buttonsPositions[3], 'bt_arrow_r', this.rightClickBoard, this);
            boardRight.anchor.set(0.5);
            var boardBg = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[3], 'language_bg_full');
            boardBg.anchor.set(0.5);

            var globalBoardSkin = global.getBoardSprite();
            var boardSkin = this.gameObj.add.sprite(this.world.width / 2 + 120, buttonsPositions[3] + 1, globalBoardSkin + "_n");
            boardSkin.anchor.set(0.5);
            this.boardSkin = boardSkin;
        }

        this.screenGroup.addChild(boardLeft);
        this.screenGroup.addChild(boardRight);
        this.screenGroup.addChild(boardBg);
        this.screenGroup.addChild(boardSkin);


        if (global.resolution == '480x800') {
            // ----- forth setting -----
            var highlightText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 33, buttonsPositions[4] + 5, global.managers.language.get("Setting_4"), textStyle);
            this.highlightText = highlightText;
            highlightText.maxWidth = 400;

            var checkboxHighlight = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 115, buttonsPositions[4], 'on_off1', this.highlightButton, this);
            this.checkboxHighlight = checkboxHighlight;
            if (global.settings.highlight == false) {
                checkboxHighlight.loadTexture('on_off1', 0);
            } else {
                checkboxHighlight.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(highlightText);
            this.screenGroup.addChild(checkboxHighlight);

            // ----- first line -----
            var line1 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[5] + 15, 'white');
            line1.anchor.set(0);
            line1.scale.x = 360;
            line1.scale.y = 1;
            this.screenGroup.addChild(line1);
            this.line1 = line1;

            // ----- fifth setting -----
            var musicText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 33, buttonsPositions[6] + 5, global.managers.language.get("Music"), textStyle);
            musicText.maxWidth = 400;
            this.musicText = musicText;

            var checkboxMusic = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 115, buttonsPositions[6], 'on_off1', this.musicButton, this);
            this.checkboxMusic = checkboxMusic;
            if (!global.settings.music) {
                checkboxMusic.loadTexture('on_off1', 0);
            } else {
                checkboxMusic.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(musicText);
            this.screenGroup.addChild(checkboxMusic);

            // ----- second line -----
            var line2 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[7] + 15, 'white');
            line2.anchor.set(0);
            line2.scale.x = 360;
            line2.scale.y = 1;
            this.screenGroup.addChild(line2);
            this.line2 = line2;

            // ----- sixth setting -----
            var soundsText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 33, buttonsPositions[8] + 5, global.managers.language.get("Sounds"), textStyle);
            soundsText.maxWidth = 400;
            this.soundsText = soundsText;

            var checkboxSounds = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 115, buttonsPositions[8], 'on_off1', this.soundsButton, this);
            this.checkboxSounds = checkboxSounds;
            if (!global.settings.sounds) {
                checkboxSounds.loadTexture('on_off1', 0);
            } else {
                checkboxSounds.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(soundsText);
            this.screenGroup.addChild(checkboxSounds);

        } else {
            // ----- forth setting -----
            var highlightText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 60, buttonsPositions[4] + 17, global.managers.language.get("Setting_4"), textStyle);
            highlightText.maxWidth = 400;
            this.highlightText = highlightText;

            var checkboxHighlight = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 180, buttonsPositions[4], 'on_off1', this.highlightButton, this);
            this.checkboxHighlight = checkboxHighlight;
            if (global.settings.highlight == false) {
                checkboxHighlight.loadTexture('on_off1', 0);
            } else {
                checkboxHighlight.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(highlightText);
            this.screenGroup.addChild(checkboxHighlight);

            // ----- first line -----
            var line1 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[5] + 30, 'white');
            line1.anchor.set(0);
            line1.scale.x = 360;
            line1.scale.y = 1;
            this.screenGroup.addChild(line1);
            this.line1 = line1;

            // ----- fifth setting -----
            var musicText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 60, buttonsPositions[6] + 17, global.managers.language.get("Music"), textStyle);
            musicText.maxWidth = 400;
            this.musicText = musicText;

            var checkboxMusic = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 180, buttonsPositions[6], 'on_off1', this.musicButton, this);
            this.checkboxMusic = checkboxMusic;
            if (!global.settings.music) {
                checkboxMusic.loadTexture('on_off1', 0);
            } else {
                checkboxMusic.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(musicText);
            this.screenGroup.addChild(checkboxMusic);

            // ----- second line -----
            var line2 = this.gameObj.add.button((this.world.width / 2) - 180, buttonsPositions[7] + 30, 'white');
            line2.anchor.set(0);
            line2.scale.x = 360;
            line2.scale.y = 1;
            this.screenGroup.addChild(line2);
            this.line2 = line2;

            // ----- sixth setting -----
            var soundsText = this.gameObj.add.text(this.world.width / 2 - newGameBackgroundTop.width / 2 + 60, buttonsPositions[8] + 17, global.managers.language.get("Sounds"), textStyle);
            soundsText.maxWidth = 400;
            this.soundsText = soundsText;

            var checkboxSounds = this.gameObj.add.button(this.world.width / 2 + newGameBackgroundTop.width / 2 - 180, buttonsPositions[8], 'on_off1', this.soundsButton, this);
            this.checkboxSounds = checkboxSounds;
            if (!global.settings.sounds) {
                checkboxSounds.loadTexture('on_off1', 0);
            } else {
                checkboxSounds.loadTexture('on_off2', 0);
            }
            this.screenGroup.addChild(soundsText);
            this.screenGroup.addChild(checkboxSounds);
        }


        if (global.settings.playAs == 0) {
            this.playAs1();
        } else {
            this.playAs2();
        }


    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    playAs1: function playAs1() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            this.player1Picker.loadTexture("bg_square_on", 0);
            this.player2Picker.loadTexture("bg_square_off", 0);
            if (global.resolution == '480x800') {
                this.checkSprite.x = this.world.width / 2 + 30;
            } else {
                this.checkSprite.x = this.world.width / 2 + 60;
            }
            global.settings.playAs = 0;
        }
    },

    playAs2: function playAs2() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            this.player2Picker.loadTexture("bg_square_on", 0);
            this.player1Picker.loadTexture("bg_square_off", 0);
            if (global.resolution == '480x800') {
                this.checkSprite.x = this.world.width / 2 + 170;
            } else {
                this.checkSprite.x = this.world.width / 2 + 240;
            }
            global.settings.playAs = 1;
        }
    },

    leftClickLanguage: function leftClickLanguage() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.languagesCounter > 0) {
                this.languagesCounter--;
            } else {
                this.languagesCounter = 5;
            }
            global.managers.language.setLanguage(this.languagesCounter);
            this.languageSkin.frame = this.languagesCounter;
            global.settings.language = this.languagesCounter;
            this.updateTexts();
        }
    },

    rightClickLanguage: function rightClickLanguage() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.languagesCounter <= 4) {
                this.languagesCounter++;
            } else {
                this.languagesCounter = 0;
            }
            global.managers.language.setLanguage(this.languagesCounter);
            this.languageSkin.frame = this.languagesCounter;
            global.settings.language = this.languagesCounter;
            this.updateTexts();
        }
    },

    leftClickStones: function leftClickStones() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.stonesTextsCounter > 0) {
                this.stonesTextsCounter--;
            } else {
                this.stonesTextsCounter = 5;
            }
            global.setStonesSkin(this.stonesTextsCounter);
            this.stonesSkin.loadTexture(global.getStonesSpritesToSettings()[0], 0);
            this.player1PickChecker.loadTexture(global.getStonesSpritesToSettings()[1], 0);
            this.player2PickChecker.loadTexture(global.getStonesSpritesToSettings()[2], 0);
        }
    },

    rightClickStones: function rightClickStones() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.stonesTextsCounter <= 3) {
                this.stonesTextsCounter++;
            } else {
                this.stonesTextsCounter = 0;
            }
            global.setStonesSkin(this.stonesTextsCounter);
            this.stonesSkin.loadTexture(global.getStonesSpritesToSettings()[0], 0);
            this.player1PickChecker.loadTexture(global.getStonesSpritesToSettings()[1], 0);
            this.player2PickChecker.loadTexture(global.getStonesSpritesToSettings()[2], 0);
        }
    },

    leftClickBoard: function leftClickBoard() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.boardTextsCounter > 0) {
                this.boardTextsCounter--;
            } else {
                this.boardTextsCounter = 4;
            }
            global.setBoardSkin(this.boardTextsCounter);
            this.boardSkin.loadTexture(global.getBoardSprite() + "_n", 0);
        }
    },

    rightClickBoard: function rightClickBoard() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (this.boardTextsCounter <= 3) {
                this.boardTextsCounter++;
            } else {
                this.boardTextsCounter = 0;
            }
            global.setBoardSkin(this.boardTextsCounter);
            this.boardSkin.loadTexture(global.getBoardSprite() + "_n", 0);
        }
    },

    highlightButton: function highlightButton() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            if (global.settings.highlight == false) {
                global.setHighlight(true);
                this.checkboxHighlight.loadTexture('on_off2', 0);
            } else {
                global.setHighlight(false);
                this.checkboxHighlight.loadTexture('on_off1', 0);
            }
        }
    },

    musicButton: function musicButton() {
        if (this.shown) {
            global.managers.audio.toggleMusic();
            if (!global.settings.music) {
                global.managers.audio.playSound("click");
                global.setMusic(true);
                this.checkboxMusic.loadTexture('on_off2', 0);
                global.settings.music = true;
            } else {
                global.managers.audio.playSound("click");
                global.setMusic(false);
                this.checkboxMusic.loadTexture('on_off1', 0);
                global.settings.music = false;
            }
        }
    },

    soundsButton: function soundsButton() {
        if (this.shown) {
            global.managers.audio.toggleSounds();
            if (!global.settings.sounds) {
                global.managers.audio.playSound("click");
                global.setSounds(true);
                this.checkboxSounds.loadTexture('on_off2', 0);
                global.settings.sounds = true;
            } else {
                global.setSounds(false);
                this.checkboxSounds.loadTexture('on_off1', 0);
                global.settings.sounds = false;
            }
        }
    },

    backBtnClicked: function backBtnClicked() {
        if (this.shown) {
            global.managers.audio.playSound("click");
            var _this = this;

            global.storage.data.language = global.settings.language;
            global.storage.data.playAs = global.settings.playAs;
            global.storage.data.stonesSkin = global.settings.stonesSkin;
            global.storage.data.boardSkin = global.settings.boardSkin;
            global.storage.data.highlight = global.settings.highlight;
            global.storage.data.music = global.settings.music;
            global.storage.data.sounds = global.settings.sounds;
            global.managers.storage.save();

            this.hideScreen();
            window.setTimeout(function () {
                _this.menuState.showMenuButtons();
            }, 100);
        }
    },

    // ------------------------------------------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        var self = this;
        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }
                self.shown = false;
            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        var self = this;

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
                self.shown = true;
            }, this);
        }

        this.updateTexts();
    },

    updateTexts: function updateTexts() {
        if (global.resolution == "480x800") {
            this.language.fontSize = 20;
            this.playAs.fontSize = 20;
            this.stonesSkinText.fontSize = 20;
            this.boardSkinText.fontSize = 20;
            var limit = 85;
        } else {
            this.language.fontSize = 30;
            this.playAs.fontSize = 30;
            this.stonesSkinText.fontSize = 30;
            this.boardSkinText.fontSize = 30;
            var limit = 115;
        }

        this.settingsTitle.setText(global.managers.language.get("OptionsKey"));

        this.language.setText(global.managers.language.get("Setting_3"));
        while (this.language.width > limit)
            this.language.fontSize--;

        this.playAs.setText(global.managers.language.get("Setting_5"));
        while (this.playAs.width > limit)
            this.playAs.fontSize--;

        this.stonesSkinText.setText(global.managers.language.get("Setting_6"));
        while (this.stonesSkinText.width > limit)
            this.stonesSkinText.fontSize--;

        this.boardSkinText.setText(global.managers.language.get("Setting_7"));
        if (global.resolution == "480x800") {
            while (this.boardSkinText.width > limit - 5)
                this.boardSkinText.fontSize--;
        } else {
            while (this.boardSkinText.width > limit - 10)
                this.boardSkinText.fontSize--;
        }

        this.highlightText.setText(global.managers.language.get("Setting_4"));
        this.musicText.setText(global.managers.language.get("Music"));
        this.soundsText.setText(global.managers.language.get("Sounds"));

    }
};