'use strict';

var ScreenSure = function ScreenSure(game, world, menuState, cameFrom) {
    this.screenGroup = null;
    this.screenName = 'ScreenSure';
    this.world = world;
    this.cameFrom = cameFrom;
    this.gameObj = game;
    this.menuState = menuState;
    this.create();
};

ScreenSure.prototype = {

    create: function create() {
        this.screenGroup = this.gameObj.add.group();
        this.screenGroup.name = this.screenName;
        this.screenGroup.visible = false;
        // ------------------------------------------------

        if (global.resolution == "480x800") {
            var positions = global.calculateDialogSize(this.world.height / 2 - 175, this.world.height / 2 + 175, 50);
        } else {
            var positions = global.calculateDialogSize(this.world.height / 2 - 300, this.world.height / 2 + 300, 69);
        }

        var newGameBackgroundTop = this.gameObj.add.sprite(this.world.width / 2, positions[0], 'dialog_bg');
        newGameBackgroundTop.frame = 0;
        newGameBackgroundTop.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundTop);

        for (var i = 1; i < positions.length - 1; i++) {
            var newGameBackground = this.gameObj.add.sprite(this.world.width / 2, positions[i], 'dialog_bg');
            newGameBackground.frame = 1;
            newGameBackground.anchor.set(0.5);
            this.screenGroup.addChild(newGameBackground);
        }

        var newGameBackgroundBottom = this.gameObj.add.sprite(this.world.width / 2, positions[positions.length - 1], 'dialog_bg');
        newGameBackgroundBottom.frame = 2;
        newGameBackgroundBottom.anchor.set(0.5);
        this.screenGroup.addChild(newGameBackgroundBottom);

        if (global.resolution == "480x800") {
            var textStyle = {
                font: global.fonts.small,
                fill: "#FFFFFF",
                align: "center"
            };
        } else {
            var textStyle = {
                font: global.fonts.main,
                fill: "#FFFFFF",
                align: "center"
            };
        }

        //back button
        if (global.resolution == "480x800") {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 185, positions[0] + 5, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        } else {
            var backBtn = this.gameObj.add.button(this.world.width / 2 + 272, positions[0] + 4, 'X-ko', this.backBtnClicked, this);
            backBtn.anchor.set(0.5);
            this.screenGroup.addChild(backBtn);
            this.backBtn = backBtn;
        }

        // Exit title
        this.title;
        if (this.cameFrom == "restart") {
            this.title = global.managers.language.get("restart");
        } else {
            this.title = global.managers.language.get("Back_2");
        }

        var restartTitle = this.gameObj.add.text(this.world.width / 2, positions[0] + 5, this.title, textStyle);
        restartTitle.maxWidth = 400;
        restartTitle.anchor.set(0.5);
        this.restartTitle = restartTitle;

        if (global.resolution == "480x800") {
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1], 32, 3);
        } else {
            var buttonsPositions = global.calculatePositions(positions[0], positions[positions.length - 1] + 50, 48, 3);
        }

        //Instructions title
        var restartQuestion = this.gameObj.add.text(this.world.width / 2, buttonsPositions[0], global.managers.language.get("Prompt"), textStyle);
        restartQuestion.maxWidth = 400;
        restartQuestion.anchor.set(0.5);
        this.restartQuestion = restartQuestion;

        var yesBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[1], 'bt_instruction_on', this.yesBtn, this);
        yesBtn.anchor.set(0.5);
        var textYes = this.gameObj.add.text(this.world.width / 2, buttonsPositions[1] + 2, global.managers.language.get("Yes"), textStyle);
        textYes.anchor.set(0.5, 0.5);
        this.textYes = textYes;

        var noBtn = this.gameObj.add.button(this.world.width / 2, buttonsPositions[2], 'bt_instruction_off', this.noBtn, this);
        noBtn.anchor.set(0.5);
        var textNo = this.gameObj.add.text(this.world.width / 2, buttonsPositions[2] + 2, global.managers.language.get("No"), textStyle);
        textNo.anchor.set(0.5, 0.5);
        this.textNo = textNo;

        this.screenGroup.addChild(yesBtn);
        this.screenGroup.addChild(textYes);
        this.screenGroup.addChild(noBtn);
        this.screenGroup.addChild(textNo);
        this.screenGroup.addChild(backBtn);
        this.screenGroup.addChild(restartTitle);
        this.screenGroup.addChild(restartQuestion);

        this.yesBtn = yesBtn;
        this.noBtn = noBtn;
    },

    getButtons: function getButtons() {
        return [this.yesBtn, this.noBtn];
    },

    // ---------------- BUTTONS FUNCTIONS ----------------
    yesBtn: function yesBtn() {
        global.managers.audio.playSound("click");
        var _this = this;

        if (this.cameFrom == "restart") {
            this.hideScreen();
            // this.menuState.returnFromDialogScreen();
            window.setTimeout(function () {

                global.storage.data.board = null;
                global.storage.data.lastGameAi = null;
                global.storage.data.lastPlayer = null;

                global.storage.data.historyMoves = null;
                global.storage.data.removedCheckers = null;
                global.storage.data.historyBoardOfValues = null;

                global.screen = -10;
                _this.gameObj.state.restart();
            }, 100);
        } else {
            global.screen = -1;
            this.hideScreen();
            // this.menuState.returnFromDialogScreen();
            window.setTimeout(function () {
                global.screen = -1;
                _this.menuState.goToMenu();
            }, 100);
        }
    },

    noBtn: function noBtn() {
        global.screen = -10;
        global.managers.audio.playSound("click");
        this.backBtnClicked();
        global.setPauseState(false);
    },

    backBtnClicked: function backBtnClicked() {
        global.screen = -10;

        global.managers.audio.playSound("click");
        this.hideScreen();
        this.menuState.returnFromDialogScreen();
        global.setPauseState(false);
    },

    // ---------------- ---------------- ----------------
    showScreen: function showScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.showScreenFromBottom(this.screenGroup, true, 250, 50, Phaser.Easing.Quintic.Out, this);
    },

    hideScreen: function hideScreen() {
        global.managers.audio.playSound("menu_swoosh");
        this.hideScreenToBottom(this.screenGroup, true, 150, 0, Phaser.Easing.Linear.None, this);
    },

    // ---------------- SHOWING AND HIDING PAUSE PANEL ----------------
    hideScreenToBottom: function hideScreenToBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;

        var tw = this.gameObj.add.tween(grp).to({
            y: this.world.height
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) {
                    context.screenGroup.hideOverClbck.call(context);
                }
            }, this);
        }
    },

    showScreenFromBottom: function showScreenFromBottom(grp, onCompleteCallEnabled, duration, delay, ease, context) {
        if (duration === null) duration = 300;
        if (delay === null) delay = 150;
        if (ease === null) ease = Phaser.Easing.Linear.None;
        grp.y = this.world.height;

        grp.visible = true;

        var tw = this.gameObj.add.tween(grp).to({
            y: 0
        }, duration, ease, true, delay);

        if (onCompleteCallEnabled === true) {
            tw.onComplete.add(function () {
                if (context.screenGroup.hideOverClbck !== undefined) context.screenGroup.hideOverClbck.call(context);
            }, this);
        }

        this.updateTexts();
    },

    getGroup: function getGroup() {
        return this.screenGroup;
    },

    updateTexts: function updateTexts() {
        this.restartQuestion.setText(global.managers.language.get("Prompt"));
        this.textYes.setText(global.managers.language.get("Yes"));
        this.textNo.setText(global.managers.language.get("No"));
    }
};