"use strict";

var GameState = function GameState() { };

GameState.prototype = {

    create: function create() {
        global.screen = -10;    
        var _this = this;

        global.addBackground("game_bg");

        this.screenBlocked = false;

        this.undoBlocked = true;
        this.interbUndoBlocked = false;

        global.game.self = this;
        this.phaser = global.phaser;

        var textStyle = {
            font: "14px saira_medium",
            fill: "#eeeeee"
        };

        this.startGame();

        if (global.resolution == "480x800") {
            var bottomPositions = global.calculatePositions((this.world.height / 2) + 264, this.world.height - 30, 32, 2);
            var scale = 150;
        } else {
            var bottomPositions = global.calculatePositions((this.world.height / 2) + 384, this.world.height - 30, 48, 2);
            var scale = 220;
        }

        var buttonPause = this.game.add.button(this.world.width / 2 - scale, bottomPositions[1], 'bt_undo2', this.undoButtonAction, this);
        buttonPause.anchor.set(0.5);
        this.buttonPause = buttonPause;
        // var textPause = this.game.add.text(this.world.width / 2 - scale, bottomPositions[1] + 15, "UNDO", textStyle);
        // textPause.anchor.set(0.5);

        var buttonRestart = this.game.add.button(this.world.width / 2, bottomPositions[1], 'bt_restart2', this.clickRestartGame, this);
        buttonRestart.anchor.set(0.5);
        this.buttonRestart = buttonRestart;
        // var textRestart = this.game.add.text(this.world.width / 2, bottomPositions[1] + 15 , "RESTART", textStyle);
        // textRestart.anchor.set(0.5);

        var buttonMenu = this.game.add.button(this.world.width / 2 + scale, bottomPositions[1], 'bt_menu2', this.clickMenu, this);
        buttonMenu.anchor.set(0.5);
        this.buttonMenu = buttonMenu;
        // var menuText = this.game.add.text(this.world.width / 2 + scale, bottomPositions[1] + 15 , "MENU", textStyle);
        // menuText.anchor.set(0.5);

        if (global.storage.data.tutorial != 'done') {
            this.tutorialGroup = this.game.add.group();
            this.tutorialGroup.name = 'Tutorial';

            this.tutorial = new Tutorial(this.world, this.game);
            this.tutorial.showOverlay();
            this.tutorialGroup.addChild(this.board.getGameBoardCheckers()[5][2].getCheckerSprite());


            global.createTimer(global.times.TUTORIAL_HAND_TIME - 310, function () {
                _this.game.world.bringToTop(_this.tutorialGroup);
                _this.game.world.bringToTop(_this.tutorial.getHand());
            });

            var pos = this.board.getXYValue(5, 2);
            this.tutorial.createHand(this.world.width + 100, this.world.height + 100);

            if (global.resolution == "480x800") {
                this.tutorial.moveHandTo(pos[0] + 39, pos[1] + 40, global.times.TUTORIAL_HAND_TIME - 300);
            } else {
                this.tutorial.moveHandTo(pos[0] + 55, pos[1] + 55, global.times.TUTORIAL_HAND_TIME - 300);
            }

            this.tutorialGroup.addChild(this.tutorial.getHand());

            this.board.setTutorial(this.tutorial);
        }

        var this1 = this;
        if (global.version != global.versions.HTML && global.inGameListener == false) {
            global.inGameListener = true;
            document.addEventListener("backbutton", function () {
                this1.androidBackHandler();
            }, false);
        }

    },

    removeCheckerFromGroup: function removeCheckerFromGroup() {
        // this.game.world.bringToTop(this.tutorialGroup);
        this.tutorialGroup.remove(this.board.getGameBoardCheckers()[5][2].getCheckerSprite());
        this.board.getGameBoardCheckers()[5][2].create();
    },

    androidBackHandler: function androidBackHandler() {
        console.log("Before -> "+ global.screen);
        switch (global.screen) {
            case -10:
                this.clickMenu();
                break;
            case 21:
                this.menu.backBtnClicked();
                global.screen = -10;
                break;
            case 20:
                this.restart.backBtnClicked();
                global.screen = -10;    
                break;     
        }
        console.log("After -> "+ global.screen);
    },

    update: function update() {
        if (this.buttonPause.input.pointerOver()) {
            this.buttonPause.tint = 0xffffff;
        } else {
            this.buttonPause.tint = 0xdbdbdb;
        }

        if (this.buttonMenu.input.pointerOver()) {
            this.buttonMenu.tint = 0xffffff;
        } else {
            this.buttonMenu.tint = 0xdbdbdb;
        }

        if (this.buttonRestart.input.pointerOver()) {
            this.buttonRestart.tint = 0xffffff;
        } else {
            this.buttonRestart.tint = 0xdbdbdb;
        }

        if (this.sureButtons != null || this.sureButtons != undefined) {
            for (var i = 0; i < this.sureButtons.length; i++) {
                if (this.sureButtons[i].input.pointerOver()) {
                    this.sureButtons[i].tint = 0xffffff;
                } else {
                    this.sureButtons[i].tint = 0xdbdbdb;
                }
            }
        }

        if (this.resultGameButtons != null || this.resultGameButtons != undefined) {
            for (var i = 0; i < this.resultGameButtons.length; i++) {
                if (this.resultGameButtons[i].input.pointerOver()) {
                    this.resultGameButtons[i].tint = 0xffffff;
                } else {
                    this.resultGameButtons[i].tint = 0xdbdbdb;
                }
            }
        }
    },

    getWorld: function getWorld() {
        return this.world;
    },

    /*
     * Main, start game.
     */
    startGame: function startGame() {
        var playerStones = global.getStonesSprites();
        this.board = new Board(this.game, this.world, this);

        var player1;
        if (global.rules.aiLevel == 1) {
            player1 = new PlayerBot(this.game, this.board, playerStones, "1", 1);
        } else if (global.rules.aiLevel == 2) {
            player1 = new PlayerBot(this.game, this.board, playerStones, "1", 2);
        } else if (global.rules.aiLevel == 3) {
            player1 = new PlayerBot(this.game, this.board, playerStones, "1", 3);
        } else {
            player1 = new Player(this.game, this.board, playerStones, "1", 0);
        }
        var player2 = new Player(this.game, this.board, playerStones, "2", 0);

        this.historyObj = this.board.getHistoryObj();
        this.board.setPlayers(player1, player2);
    },

    /*
     * Restart button listener.
     */
    clickRestartGame: function clickRestartGame() {
        global.managers.audio.playSound("click");

        if (this.screenBlocked == false) {
            global.screen = 20;
            var _this = this;
            global.setPauseState(true);
            var overlay = this.phaser.add.image(0, 0, "overlay");
            overlay.width = this.phaser.width;
            overlay.height = this.phaser.height;
            overlay.alpha = 0.65;
            overlay.inputEnabled = true;
            overlay.visible = true;
            this.overlay = overlay;

            this.restart = new ScreenSure(this.game, this.world, this, "restart")
            this.sureButtons = this.restart.getButtons();
            this.restart.showScreen();
            _this.screenBlocked = true;

            window.setTimeout(function () {
                _this.screenBlocked = false;
            }, 100);
        }
    },

    /*
     * Menu button listener.
     */
    clickMenu: function clickMenu() {
            global.managers.audio.playSound("click");

            if (this.screenBlocked == false) {
                global.screen = 21;
                var _this = this;
                global.setPauseState(true);
                var overlay = this.phaser.add.image(0, 0, "overlay");
                overlay.width = this.phaser.width;
                overlay.height = this.phaser.height;
                overlay.alpha = 0.65;
                overlay.inputEnabled = true;
                overlay.visible = true;
                this.overlay = overlay;

                this.menu = new ScreenSure(this.game, this.world, this, "menu");
                this.sureButtons = this.menu.getButtons();
                this.menu.showScreen();
                _this.screenBlocked = true;

                window.setTimeout(function () {
                    _this.screenBlocked = false;
                }, 100);
            }
    },

    returnFromDialogScreen: function returnFromDialogScreen() {
        this.overlay.inputEnabled = false;
        this.overlay.visible = false;
    },

    undoButtonAction: function undoButtonAction() {
        var _this2 = this;

        if (!this.interbUndoBlocked) {
            if (global.rules.aiLevel == 1 || global.rules.aiLevel == 2 || global.rules.aiLevel == 3) {
                if (this.historyObj.getHistorySize() > 1 && !this.undoBlocked && !this.board.isMoving()) {
                    global.managers.audio.playSound("click");
                    this.board.setGameBoardCheckers(this.historyObj.getMoves(), this.historyObj.getBoardValues(), this.historyObj.getRemoved(), false);
                    this.board.setGameBoardCheckers(this.historyObj.getMoves(), this.historyObj.getBoardValues(), this.historyObj.getRemoved(), false);
                    this.blockUndo();
                    this.interbUndoBlocked = true;
                    global.createTimer(100, function () {
                        _this2.interbUndoBlocked = false;
                    });
                } else {
                    global.managers.audio.playSound("response_negative");
                }
            } else {
                if (this.historyObj.getHistorySize() > 0 && !this.undoBlocked && !this.board.isMoving()) {
                    global.managers.audio.playSound("click");
                    this.board.setGameBoardCheckers(this.historyObj.getMoves(), this.historyObj.getBoardValues(), this.historyObj.getRemoved(), true);
                    this.blockUndo();
                    this.interbUndoBlocked = true;
                    global.createTimer(100, function () {
                        _this2.interbUndoBlocked = false;
                    });
                } else {
                    global.managers.audio.playSound("response_negative");
                }
            }
        } else {
            global.managers.audio.playSound("response_negative");
        }
    },

    goToMenu: function goToMenu() {
        this.camera.fade(0x000000, 200, false);
        this.time.events.add(200, function () {
            this.game.state.start('Menu');
        }, this);
    },

    blockUndo: function blockUndo() {
        this.undoBlocked = true;
    },

    unblockUndo: function unblockUndo() {
        this.undoBlocked = false;
    },

    getBoard: function getBoard() {
        return this.board;
    },

    setResultGameButtons: function setResultGameButtons(buttons) {
        this.resultGameButtons = buttons;
    }
};