var PreloaderState = function () {

};

PreloaderState.prototype = {
    preload: function () {
        
        this.game.load.atlas('imgpk', 'assets/images/480x800/spritesheet.png', 'assets/images/480x800/sprites.json');
        this.game.load.image("game_bg", "assets/images/" + global.resolution + "/images/game_bg.png");
        this.game.load.image("I", "assets/images/I.png");

        this.preloadResources();

        for (var key in global.audio) {
            if (global.audio.hasOwnProperty(key)) {
                value = global.audio[key];
                this.game.load.audio(key, [value.path + ".ogg", value.path + ".mp3"]);
            }
        }


        global.navigatorHelp = navigator;

        global.preloader.self = this;
        this.game.load.onLoadStart.add(this.loadingStarted, this);
        this.game.load.onFileComplete.add(this.loadingFileCompleted, this);
        this.game.load.onLoadComplete.add(this.loadingFinished, this);
        this.game.load.start();
    },

    preloadResources: function () {
        this.game.load.text("lang_strings", "assets/texts/m.json");

        if (global.resolution == "480x800") {
            var pack = global.resourcesSmall;
        } else {
            var pack = global.resourcesBig;
        }
        for (var method in pack) {
            pack[method].forEach(function (args) {
                var loader = this.load[method];
                loader && loader.apply(this.load, args);
            }, this);
        }
    },


    loadingStarted: function () {
        this.progress = this.game.add.text(this.game.world.centerX, this.world.height - 100, "0%", {
            font: global.fonts.main,
            fill: "#FFFFFF",
            align: "center"
        });
        this.progress.anchor.setTo(0.5);
    },

    loadingFileCompleted: function (progress, cacheKey) {
        this.progress.setText(progress + "%");

        if (cacheKey == "game_bg") {
            var splash = this.game.add.image(this.game.world.centerX, this.game.world.centerY, cacheKey);
            splash.anchor.setTo(0.5);
            this.splash = splash;

            var diff = Math.max(this.game.width - splash.width, this.game.height - splash.height);

            if (diff > 0) {
                splash.width += diff;
                splash.height += diff;
            }

            if(this.logo != null && this.logo != undefined){
                this.game.world.bringToTop(this.logo);
            }

            this.progress.bringToTop();
        }

        if (cacheKey == "I") {
            var logo = this.game.add.image(this.game.world.centerX, this.game.world.centerY, "I");
            logo.anchor.set(0.5);
            if (global.resolution == "480x800") {
                logo.scale.x = 1;
                logo.scale.y = 1;
            } else {
                logo.scale.x = 1.5;
                logo.scale.y = 1.5;
            }
            this.game.world.bringToTop(logo);
            this.logo = logo;
        }
    },

    loadingFinished: function () {
        var value;
        for (var key in global.audio) {
            if (global.audio.hasOwnProperty(key)) {
                value = global.audio[key];
                if (value.type == "music") {
                    global.managers.audio.addMusic(key, value.volume, value.loop);
                } else {
                    global.managers.audio.addSound(key, value.volume, value.loop);
                }
            }
        }

        global.managers.storage.load();
        this.setSavedRules();
        this.setSavedSettings();


        if(global.storage.data.language == null){
            var lang = navigator.userLanguage || navigator.language;
            global.managers.language.load();
            console.log(lang);
            var languages = ["en", "de", "es", "fr", "it", "pt"];
            global.settings.language = languages.indexOf(lang.substring(0, 2));
            global.managers.language.setLanguage(languages.indexOf(lang.substring(0, 2)));
        }else{
            global.managers.language.load();
            global.managers.language.setLanguage(global.storage.data.language);
        }

      
        this.game.time.events.add(global.preloader.splashTime, function () {
            this.game.state.start("Menu");
        }, this);
    },

    setSavedRules: function () {
        if (global.storage.data.captureBackwards != null) {
            global.rules.captureBackwards = global.storage.data.captureBackwards;
        }
        if (global.storage.data.capture != null) {
            global.rules.capture = global.storage.data.capture;
        }
        if (global.storage.data.king != null) {
            global.rules.king = global.storage.data.king;
        }
        if (global.storage.data.kingPriority != null) {
            global.rules.kingPriority = global.storage.data.kingPriority;
        }
    },



    setSavedSettings: function () {
        if (global.storage.data.language != null) {
            global.settings.language = global.storage.data.language;
        }
        if (global.storage.data.playAs != null) {
            global.settings.playAs = global.storage.data.playAs;
        }
        if (global.storage.data.stonesSkin != null) {
            global.settings.stonesSkin = global.storage.data.stonesSkin;
        }
        if (global.storage.data.boardSkin != null) {
            global.settings.boardSkin = global.storage.data.boardSkin;
        }
        if (global.storage.data.highlight != null) {
            global.settings.highlight = global.storage.data.highlight;
        }
        if (global.storage.data.music != null) {
            global.settings.music = global.storage.data.music;
        }
        if (global.storage.data.sounds != null) {
            global.settings.sounds = global.storage.data.sounds;
        }
    }

};